@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2010       Mehdi Juhoor
  * Copyright (C) 2013       Thibaut Lambert
  *
@@ -35,6 +35,8 @@
 #include "magma.h"
 #endif
 
+#include "starpu_cusolver.h"
+
 /*
  *	Some useful functions
  */
@@ -53,17 +55,22 @@ static struct starpu_task *create_task(starpu_tag_t id)
  *	Create the codelets
  */
 
-static struct starpu_task * create_task_11(starpu_data_handle_t dataA, unsigned k, unsigned reclevel)
+static struct starpu_task * create_task_potrf(starpu_data_handle_t dataA, unsigned k, unsigned reclevel)
 {
-/*	FPRINTF(stdout, "task 11 k = %d TAG = %llx\n", k, (TAG11(k))); */
+/*	FPRINTF(stdout, "task potrf k = %d TAG = %llx\n", k, (TAG_POTRF(k))); */
 
-	struct starpu_task *task = create_task(TAG11_AUX(k, reclevel));
+	struct starpu_task *task = create_task(TAG_POTRF_AUX(k, reclevel));
 
-	task->cl = &cl11;
+	task->cl = &cl_potrf;
 
 	/* which sub-data is manipulated ? */
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
 
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_LIBCUSOLVER)
+	/* Temporary data to save libcusolver from allocating/deallocating memory */
+	task->handles[1] = scratch;
+#endif
+
 	/* this is an important task */
 	if (!noprio_p)
 		task->priority = STARPU_MAX_PRIO;
@@ -71,7 +78,7 @@ static struct starpu_task * create_task_11(starpu_data_handle_t dataA, unsigned
 	/* enforce dependencies ... */
 	if (k > 0)
 	{
-		starpu_tag_declare_deps(TAG11_AUX(k, reclevel), 1, TAG22_AUX(k-1, k, k, reclevel));
+		starpu_tag_declare_deps(TAG_POTRF_AUX(k, reclevel), 1, TAG_GEMM_AUX(k-1, k, k, reclevel));
 	}
 
 	int n = starpu_matrix_get_nx(task->handles[0]);
@@ -80,13 +87,13 @@ static struct starpu_task * create_task_11(starpu_data_handle_t dataA, unsigned
 	return task;
 }
 
-static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned m, unsigned reclevel)
+static int create_task_trsm(starpu_data_handle_t dataA, unsigned k, unsigned m, unsigned reclevel)
 {
 	int ret;
 
-	struct starpu_task *task = create_task(TAG21_AUX(k, m, reclevel));
+	struct starpu_task *task = create_task(TAG_TRSM_AUX(k, m, reclevel));
 
-	task->cl = &cl21;
+	task->cl = &cl_trsm;
 
 	/* which sub-data is manipulated ? */
 	task->handles[0] = starpu_data_get_sub_data(dataA, 2, k, k);
@@ -100,11 +107,11 @@ static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned m, un
 	/* enforce dependencies ... */
 	if (k > 0)
 	{
-		starpu_tag_declare_deps(TAG21_AUX(k, m, reclevel), 2, TAG11_AUX(k, reclevel), TAG22_AUX(k-1, m, k, reclevel));
+		starpu_tag_declare_deps(TAG_TRSM_AUX(k, m, reclevel), 2, TAG_POTRF_AUX(k, reclevel), TAG_GEMM_AUX(k-1, m, k, reclevel));
 	}
 	else
 	{
-		starpu_tag_declare_deps(TAG21_AUX(k, m, reclevel), 1, TAG11_AUX(k, reclevel));
+		starpu_tag_declare_deps(TAG_TRSM_AUX(k, m, reclevel), 1, TAG_POTRF_AUX(k, reclevel));
 	}
 
 	int nx = starpu_matrix_get_nx(task->handles[0]);
@@ -115,22 +122,37 @@ static int create_task_21(starpu_data_handle_t dataA, unsigned k, unsigned m, un
 	return ret;
 }
 
-static int create_task_22(starpu_data_handle_t dataA, unsigned k, unsigned m, unsigned n, unsigned reclevel)
+static int create_task_gemm(starpu_data_handle_t dataA, unsigned k, unsigned m, unsigned n, unsigned reclevel)
 {
 	int ret;
 
-/*	FPRINTF(stdout, "task 22 k,n,m = %d,%d,%d TAG = %llx\nx", k,m,n, TAG22_AUX(k,m,n)); */
+/*	FPRINTF(stdout, "task gemm k,n,m = %d,%d,%d TAG = %llx\nx", k,m,n, TAG_GEMM_AUX(k,m,n)); */
 
-	struct starpu_task *task = create_task(TAG22_AUX(k, m, n, reclevel));
+	struct starpu_task *task = create_task(TAG_GEMM_AUX(k, m, n, reclevel));
 
-	task->cl = &cl22;
+	if (m == n)
+	{
+		task->cl = &cl_syrk;
 
-	/* which sub-data is manipulated ? */
-	task->handles[0] = starpu_data_get_sub_data(dataA, 2, n, k);
-	task->handles[1] = starpu_data_get_sub_data(dataA, 2, m, k);
-	task->handles[2] = starpu_data_get_sub_data(dataA, 2, m, n);
+		/* which sub-data is manipulated ? */
+		task->handles[0] = starpu_data_get_sub_data(dataA, 2, n, k);
+		task->handles[1] = starpu_data_get_sub_data(dataA, 2, n, n);
+		int nx = starpu_matrix_get_nx(task->handles[0]);
+		task->flops = FLOPS_SSYRK(nx, nx);
+	}
+	else
+	{
+		task->cl = &cl_gemm;
 
-	if ( (n == k + 1) && (m == k +1) )
+		/* which sub-data is manipulated ? */
+		task->handles[0] = starpu_data_get_sub_data(dataA, 2, n, k);
+		task->handles[1] = starpu_data_get_sub_data(dataA, 2, m, k);
+		task->handles[2] = starpu_data_get_sub_data(dataA, 2, m, n);
+		int nx = starpu_matrix_get_nx(task->handles[0]);
+		task->flops = FLOPS_SGEMM(nx, nx, nx);
+	}
+
+	if ((n == k + 1) && (m == k +1))
 	{
 		task->priority = STARPU_MAX_PRIO;
 	}
@@ -138,16 +160,13 @@ static int create_task_22(starpu_data_handle_t dataA, unsigned k, unsigned m, un
 	/* enforce dependencies ... */
 	if (k > 0)
 	{
-		starpu_tag_declare_deps(TAG22_AUX(k, m, n, reclevel), 3, TAG22_AUX(k-1, m, n, reclevel), TAG21_AUX(k, n, reclevel), TAG21_AUX(k, m, reclevel));
+		starpu_tag_declare_deps(TAG_GEMM_AUX(k, m, n, reclevel), 3, TAG_GEMM_AUX(k-1, m, n, reclevel), TAG_TRSM_AUX(k, n, reclevel), TAG_TRSM_AUX(k, m, reclevel));
 	}
 	else
 	{
-		starpu_tag_declare_deps(TAG22_AUX(k, m, n, reclevel), 2, TAG21_AUX(k, n, reclevel), TAG21_AUX(k, m, reclevel));
+		starpu_tag_declare_deps(TAG_GEMM_AUX(k, m, n, reclevel), 2, TAG_TRSM_AUX(k, n, reclevel), TAG_TRSM_AUX(k, m, reclevel));
 	}
 
-	int nx = starpu_matrix_get_nx(task->handles[0]);
-	task->flops = FLOPS_SGEMM(nx, nx, nx);
-
 	ret = starpu_task_submit(task);
 	if (ret != -ENODEV) STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	return ret;
@@ -195,10 +214,12 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 
 	starpu_data_map_filters(dataA, 2, &f, &f2);
 
+	cholesky_kernel_init(size / nblocks);
+
 	for (k = 0; k < nbigblocks; k++)
 	{
 		starpu_iteration_push(k);
-		struct starpu_task *task = create_task_11(dataA, k, reclevel);
+		struct starpu_task *task = create_task_potrf(dataA, k, reclevel);
 		/* we defer the launch of the first task */
 		if (k == 0)
 		{
@@ -213,14 +234,14 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 
 		for (m = k+1; m<nblocks; m++)
 		{
-		     	ret = create_task_21(dataA, k, m, reclevel);
+		     	ret = create_task_trsm(dataA, k, m, reclevel);
 			if (ret == -ENODEV) return 77;
 
 			for (n = k+1; n<nblocks; n++)
 			{
 				if (n <= m)
 				{
-				     ret = create_task_22(dataA, k, m, n, reclevel);
+				     ret = create_task_gemm(dataA, k, m, n, reclevel);
 				     if (ret == -ENODEV) return 77;
 				}
 			}
@@ -236,7 +257,8 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 	if (nblocks == nbigblocks)
 	{
 		/* stall the application until the end of computations */
-		starpu_tag_wait(TAG11_AUX(nblocks-1, reclevel));
+		starpu_tag_wait(TAG_POTRF_AUX(nblocks-1, reclevel));
+		cholesky_kernel_fini();
 		starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 		starpu_data_unregister(dataA);
 		return 0;
@@ -254,13 +276,14 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 		for (m = nbigblocks; m < nblocks; m++)
 		{
 			if (n <= m)
-				tag_array[ind++] = TAG22_AUX(nbigblocks - 1, m, n, reclevel);
+				tag_array[ind++] = TAG_GEMM_AUX(nbigblocks - 1, m, n, reclevel);
 		}
 
 		starpu_tag_wait_array(ind, tag_array);
 
 		free(tag_array);
 
+		cholesky_kernel_fini();
 		starpu_data_unpartition(dataA, STARPU_MAIN_RAM);
 		starpu_data_unregister(dataA);
 
@@ -273,7 +296,7 @@ static int cholesky_grain_rec(float *matA, unsigned size, unsigned ld, unsigned
 static int initialize_system(int argc, char **argv, float **A, unsigned pinned)
 {
 	int ret;
-	int flags = STARPU_MALLOC_SIMULATION_FOLDED;
+	int flags = STARPU_MALLOC_SIMULATION_FOLDED|STARPU_MALLOC_SIMULATION_UNIQUE;
 
 #ifdef STARPU_HAVE_MAGMA
 	magma_init();
@@ -289,20 +312,23 @@ static int initialize_system(int argc, char **argv, float **A, unsigned pinned)
 	parse_args(argc, argv);
 
 #ifdef STARPU_USE_CUDA
-	initialize_chol_model(&chol_model_11,"chol_model_11",cpu_chol_task_11_cost,cuda_chol_task_11_cost);
-	initialize_chol_model(&chol_model_21,"chol_model_21",cpu_chol_task_21_cost,cuda_chol_task_21_cost);
-	initialize_chol_model(&chol_model_22,"chol_model_22",cpu_chol_task_22_cost,cuda_chol_task_22_cost);
+	initialize_chol_model(&chol_model_potrf,"chol_model_potrf",cpu_chol_task_potrf_cost,cuda_chol_task_potrf_cost);
+	initialize_chol_model(&chol_model_trsm,"chol_model_trsm",cpu_chol_task_trsm_cost,cuda_chol_task_trsm_cost);
+	initialize_chol_model(&chol_model_syrk,"chol_model_syrk",cpu_chol_task_syrk_cost,cuda_chol_task_syrk_cost);
+	initialize_chol_model(&chol_model_gemm,"chol_model_gemm",cpu_chol_task_gemm_cost,cuda_chol_task_gemm_cost);
 #else
-	initialize_chol_model(&chol_model_11,"chol_model_11",cpu_chol_task_11_cost,NULL);
-	initialize_chol_model(&chol_model_21,"chol_model_21",cpu_chol_task_21_cost,NULL);
-	initialize_chol_model(&chol_model_22,"chol_model_22",cpu_chol_task_22_cost,NULL);
+	initialize_chol_model(&chol_model_potrf,"chol_model_potrf",cpu_chol_task_potrf_cost,NULL);
+	initialize_chol_model(&chol_model_trsm,"chol_model_trsm",cpu_chol_task_trsm_cost,NULL);
+	initialize_chol_model(&chol_model_syrk,"chol_model_syrk",cpu_chol_task_syrk_cost,NULL);
+	initialize_chol_model(&chol_model_gemm,"chol_model_gemm",cpu_chol_task_gemm_cost,NULL);
 #endif
 
 	starpu_cublas_init();
+	starpu_cusolver_init();
 
 	if (pinned)
 		flags |= STARPU_MALLOC_PINNED;
-	starpu_malloc_flags((void **)A, size_p*size_p*sizeof(float), flags);
+	starpu_malloc_flags((void **)A, (size_t)size_p*size_p*sizeof(float), flags);
 
 	return 0;
 }
@@ -331,12 +357,13 @@ int cholesky_grain(float *matA, unsigned size, unsigned ld, unsigned nblocks, un
 
 static void shutdown_system(float **matA, unsigned dim, unsigned pinned)
 {
-	int flags = STARPU_MALLOC_SIMULATION_FOLDED;
+	int flags = STARPU_MALLOC_SIMULATION_FOLDED|STARPU_MALLOC_SIMULATION_UNIQUE;
 	if (pinned)
 		flags |= STARPU_MALLOC_PINNED;
 
-	starpu_free_flags(*matA, dim*dim*sizeof(float), flags);
+	starpu_free_flags(*matA, (size_t)dim*dim*sizeof(float), flags);
 
+	starpu_cusolver_shutdown();
 	starpu_cublas_shutdown();
 	starpu_shutdown();
 }
@@ -353,7 +380,7 @@ int main(int argc, char **argv)
 	if (ret) return ret;
 
 #ifndef STARPU_SIMGRID
-	unsigned m,n;
+	unsigned long long m,n;
 
 	for (n = 0; n < size_p; n++)
 	{
@@ -422,7 +449,7 @@ int main(int argc, char **argv)
 				}
 			}
 		}
-		float *test_mat = malloc(size_p*size_p*sizeof(float));
+		float *test_mat = malloc((size_t)size_p*size_p*sizeof(float));
 		STARPU_ASSERT(test_mat);
 
 		STARPU_SSYRK("L", "N", size_p, size_p, 1.0f,
@@ -457,7 +484,7 @@ int main(int argc, char **argv)
 	                                float err = fabsf(test_mat[m +n*size_p] - orig) / orig;
 	                                if (err > 0.0001)
 					{
-	                                        FPRINTF(stderr, "Error[%u, %u] --> %2.6f != %2.6f (err %2.6f)\n", m, n, test_mat[m +n*size_p], orig, err);
+						FPRINTF(stderr, "Error[%llu, %llu] --> %2.6f != %2.6f (err %2.6f)\n", m, n, test_mat[m +n*size_p], orig, err);
 	                                        assert(0);
 	                                }
 	                        }
