@@ -1,9 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011                                     Inria
- * Copyright (C) 2008-2011,2014                           Université de Bordeaux
- * Copyright (C) 2010-2015                                CNRS
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2008-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -43,8 +42,9 @@
  *
  */
 
-double task_11_cost(struct starpu_task *task, unsigned nimpl)
+double task_getrf_cost(struct starpu_task *task, unsigned nimpl)
 {
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
@@ -54,8 +54,9 @@ double task_11_cost(struct starpu_task *task, unsigned nimpl)
 	return PERTURBATE(cost);
 }
 
-double task_12_cost(struct starpu_task *task, unsigned nimpl)
+double task_trsm_ll_cost(struct starpu_task *task, unsigned nimpl)
 {
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
@@ -63,13 +64,14 @@ double task_12_cost(struct starpu_task *task, unsigned nimpl)
 /*	double cost = ((n*n*n)/1744.695); */
 	double cost = ((n*n*n)/3210.80);
 
-	/* fprintf(stderr, "task 12 predicts %e\n", cost); */
+	/* fprintf(stderr, "task TRSM_LL predicts %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
 
-double task_21_cost(struct starpu_task *task, unsigned nimpl)
+double task_trsm_ru_cost(struct starpu_task *task, unsigned nimpl)
 {
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
@@ -77,14 +79,15 @@ double task_21_cost(struct starpu_task *task, unsigned nimpl)
 /*	double cost = ((n*n*n)/1744.695); */
 	double cost = ((n*n*n)/3691.53);
 
-	/* fprintf(stderr, "task 12 predicts %e\n", cost); */
+	/* fprintf(stderr, "task TRSM_RU predicts %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
 
 
-double task_22_cost(struct starpu_task *task, unsigned nimpl)
+double task_gemm_cost(struct starpu_task *task, unsigned nimpl)
 {
+	(void)nimpl;
 	uint32_t nx, ny, nz;
 
 	nx = starpu_matrix_get_nx(task->handles[2]);
@@ -103,47 +106,55 @@ double task_22_cost(struct starpu_task *task, unsigned nimpl)
  */
 
 
-double task_11_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_getrf_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/1853.7806);
 
-/*	printf("CUDA task 11 ; predict %e\n", cost); */
+/*	printf("CUDA task GETRF ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
-double task_12_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_trsm_ll_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/42838.5718);
 
-/*	printf("CUDA task 12 ; predict %e\n", cost); */
+/*	printf("CUDA task TRSM_LL ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
 
-double task_21_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_trsm_ru_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/49208.667);
 
-/*	printf("CUDA task 21 ; predict %e\n", cost); */
+/*	printf("CUDA task TRSM_RU ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
 
 
-double task_22_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_gemm_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t nx, ny, nz;
 
 	nx = starpu_matrix_get_nx(task->handles[2]);
@@ -152,7 +163,7 @@ double task_22_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch*
 
 	double cost = ((nx*ny*nz)/57523.560);
 
-/*	printf("CUDA task 22 ; predict %e\n", cost); */
+/*	printf("CUDA task GEMM ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
@@ -162,47 +173,55 @@ double task_22_cost_cuda(struct starpu_task *task, struct starpu_perfmodel_arch*
  *
  */
 
-double task_11_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_getrf_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/537.5);
 
-/*	printf("CPU task 11 ; predict %e\n", cost); */
+/*	printf("CPU task GETRF ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
-double task_12_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_trsm_ll_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/6668.224);
 
-/*	printf("CPU task 12 ; predict %e\n", cost); */
+/*	printf("CPU task TRSM_LL ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
 
-double task_21_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_trsm_ru_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t n;
 
 	n = starpu_matrix_get_nx(task->handles[0]);
 
 	double cost = ((n*n*n)/6793.8423);
 
-/*	printf("CPU task 21 ; predict %e\n", cost); */
+/*	printf("CPU task TRSM_RU ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
 
 
-double task_22_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
+double task_gemm_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch* arch, unsigned nimpl)
 {
+	(void)arch;
+	(void)nimpl;
 	uint32_t nx, ny, nz;
 
 	nx = starpu_matrix_get_nx(task->handles[2]);
@@ -211,7 +230,7 @@ double task_22_cost_cpu(struct starpu_task *task, struct starpu_perfmodel_arch*
 
 	double cost = ((nx*ny*nz)/4203.0175);
 
-/*	printf("CPU task 22 ; predict %e\n", cost); */
+/*	printf("CPU task GEMM ; predict %e\n", cost); */
 	return PERTURBATE(cost);
 }
 
@@ -220,6 +239,7 @@ void initialize_lu_kernels_model(struct starpu_perfmodel* model, char * symbol,
 				 double (*cpu_cost_function)(struct starpu_task *, struct starpu_perfmodel_arch*, unsigned),
 				 double (*cuda_cost_function)(struct starpu_task *, struct starpu_perfmodel_arch*, unsigned))
 {
+	(void)cost_function;
 	model->symbol = symbol;
 	model->type = STARPU_HISTORY_BASED;
 
