@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2018                                Université de Bordeaux
- * Copyright (C) 2011,2012                                Inria
- * Copyright (C) 2010-2012,2015,2017,2019                 CNRS
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,8 +21,8 @@
 #include <complex.h>
 
 #ifdef STARPU_USE_CUDA
-#include <cublas.h>
 #include <starpu_cublas_v2.h>
+#include "starpu_cusolver.h"
 #endif
 
 #define str(s) #s
@@ -36,11 +34,13 @@ static const TYPE p1 =  1.0f;
 static const TYPE m1 = -1.0f;
 #endif
 
+starpu_data_handle_t scratch = NULL;
+
 /*
- *   U22
+ *   GEMM
  */
 
-static inline void STARPU_LU(common_u22)(void *descr[], int s, void *_args)
+static inline void STARPU_LU(common_gemm)(void *descr[], int s, void *_args)
 {
 	(void)_args;
 	TYPE *right 	= (TYPE *)STARPU_MATRIX_GET_PTR(descr[0]);
@@ -56,8 +56,7 @@ static inline void STARPU_LU(common_u22)(void *descr[], int s, void *_args)
 	unsigned ld22 = STARPU_MATRIX_GET_LD(descr[2]);
 
 #ifdef STARPU_USE_CUDA
-	cublasStatus status;
-	cudaError_t cures;
+	cublasStatus_t status;
 #endif
 
 	switch (s)
@@ -88,29 +87,29 @@ static inline void STARPU_LU(common_u22)(void *descr[], int s, void *_args)
 	}
 }
 
-void STARPU_LU(cpu_u22)(void *descr[], void *_args)
+void STARPU_LU(cpu_gemm)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u22)(descr, 0, _args);
+	STARPU_LU(common_gemm)(descr, 0, _args);
 }
 
 #ifdef STARPU_USE_CUDA
-void STARPU_LU(cublas_u22)(void *descr[], void *_args)
+void STARPU_LU(cublas_gemm)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u22)(descr, 1, _args);
+	STARPU_LU(common_gemm)(descr, 1, _args);
 }
 #endif /* STARPU_USE_CUDA */
 
-static struct starpu_perfmodel STARPU_LU(model_22) =
+static struct starpu_perfmodel STARPU_LU(model_gemm) =
 {
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
-	.symbol = STARPU_LU_STR(lu_model_22_atlas)
+	.symbol = STARPU_LU_STR(lu_model_gemm_atlas)
 #elif defined(STARPU_GOTO)
-	.symbol = STARPU_LU_STR(lu_model_22_goto)
+	.symbol = STARPU_LU_STR(lu_model_gemm_goto)
 #elif defined(STARPU_OPENBLAS)
-	.symbol = STARPU_LU_STR(lu_model_22_openblas)
+	.symbol = STARPU_LU_STR(lu_model_gemm_openblas)
 #else
-	.symbol = STARPU_LU_STR(lu_model_22)
+	.symbol = STARPU_LU_STR(lu_model_gemm)
 #endif
 };
 
@@ -120,7 +119,7 @@ static int can_execute(unsigned workerid, struct starpu_task *task, unsigned nim
 	(void)task;
 	(void)nimpl;
 	enum starpu_worker_archtype type = starpu_worker_get_type(workerid);
-	if (type == STARPU_CPU_WORKER || type == STARPU_MIC_WORKER)
+	if (type == STARPU_CPU_WORKER)
 		return 1;
 
 #ifdef STARPU_SIMGRID
@@ -146,12 +145,12 @@ static int can_execute(unsigned workerid, struct starpu_task *task, unsigned nim
 
 #define STRINGIFY_(x) #x
 #define STRINGIFY(x) STRINGIFY_(x)
-struct starpu_codelet cl22 =
+struct starpu_codelet cl_gemm =
 {
-	.cpu_funcs = {STARPU_LU(cpu_u22)},
-	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_u22))},
+	.cpu_funcs = {STARPU_LU(cpu_gemm)},
+	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_gemm))},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {STARPU_LU(cublas_u22)},
+	.cuda_funcs = {STARPU_LU(cublas_gemm)},
 	CAN_EXECUTE
 #elif defined(STARPU_SIMGRID)
 	.cuda_funcs = {(void*)1},
@@ -159,14 +158,14 @@ struct starpu_codelet cl22 =
 	.cuda_flags = {STARPU_CUDA_ASYNC},
 	.nbuffers = 3,
 	.modes = {STARPU_R, STARPU_R, STARPU_RW},
-	.model = &STARPU_LU(model_22)
+	.model = &STARPU_LU(model_gemm)
 };
 
 /*
- * U12
+ * TRSM_LL
  */
 
-static inline void STARPU_LU(common_u12)(void *descr[], int s, void *_args)
+static inline void STARPU_LU(common_trsmll)(void *descr[], int s, void *_args)
 {
 	(void)_args;
 	TYPE *sub11;
@@ -182,8 +181,7 @@ static inline void STARPU_LU(common_u12)(void *descr[], int s, void *_args)
 	unsigned ny12 = STARPU_MATRIX_GET_NY(descr[1]);
 
 #ifdef STARPU_USE_CUDA
-	cublasStatus status;
-	cudaError_t cures;
+	cublasStatus_t status;
 #endif
 
 	/* solve L11 U12 = A12 (find U12) */
@@ -211,38 +209,38 @@ static inline void STARPU_LU(common_u12)(void *descr[], int s, void *_args)
 	}
 }
 
-void STARPU_LU(cpu_u12)(void *descr[], void *_args)
+void STARPU_LU(cpu_trsmll)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u12)(descr, 0, _args);
+	STARPU_LU(common_trsmll)(descr, 0, _args);
 }
 
 #ifdef STARPU_USE_CUDA
-void STARPU_LU(cublas_u12)(void *descr[], void *_args)
+void STARPU_LU(cublas_trsmll)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u12)(descr, 1, _args);
+	STARPU_LU(common_trsmll)(descr, 1, _args);
 }
 #endif /* STARPU_USE_CUDA */
 
-static struct starpu_perfmodel STARPU_LU(model_12) =
+static struct starpu_perfmodel STARPU_LU(model_trsm_ll) =
 {
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
-	.symbol = STARPU_LU_STR(lu_model_12_atlas)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ll_atlas)
 #elif defined(STARPU_GOTO)
-	.symbol = STARPU_LU_STR(lu_model_12_goto)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ll_goto)
 #elif defined(STARPU_OPENBLAS)
-	.symbol = STARPU_LU_STR(lu_model_12_openblas)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ll_openblas)
 #else
-	.symbol = STARPU_LU_STR(lu_model_12)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ll)
 #endif
 };
 
-struct starpu_codelet cl12 =
+struct starpu_codelet cl_trsm_ll =
 {
-	.cpu_funcs = {STARPU_LU(cpu_u12)},
-	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_u12))},
+	.cpu_funcs = {STARPU_LU(cpu_trsmll)},
+	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_trsmll))},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {STARPU_LU(cublas_u12)},
+	.cuda_funcs = {STARPU_LU(cublas_trsmll)},
 	CAN_EXECUTE
 #elif defined(STARPU_SIMGRID)
 	.cuda_funcs = {(void*)1},
@@ -250,14 +248,14 @@ struct starpu_codelet cl12 =
 	.cuda_flags = {STARPU_CUDA_ASYNC},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_RW},
-	.model = &STARPU_LU(model_12)
+	.model = &STARPU_LU(model_trsm_ll)
 };
 
 /*
- * U21
+ * TRSM_RU
  */
 
-static inline void STARPU_LU(common_u21)(void *descr[], int s, void *_args)
+static inline void STARPU_LU(common_trsmru)(void *descr[], int s, void *_args)
 {
 	(void)_args;
 	TYPE *sub11;
@@ -273,7 +271,7 @@ static inline void STARPU_LU(common_u21)(void *descr[], int s, void *_args)
 	unsigned ny21 = STARPU_MATRIX_GET_NY(descr[1]);
 
 #ifdef STARPU_USE_CUDA
-	cublasStatus status;
+	cublasStatus_t status;
 #endif
 
 	switch (s)
@@ -300,38 +298,38 @@ static inline void STARPU_LU(common_u21)(void *descr[], int s, void *_args)
 	}
 }
 
-void STARPU_LU(cpu_u21)(void *descr[], void *_args)
+void STARPU_LU(cpu_trsmru)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u21)(descr, 0, _args);
+	STARPU_LU(common_trsmru)(descr, 0, _args);
 }
 
 #ifdef STARPU_USE_CUDA
-void STARPU_LU(cublas_u21)(void *descr[], void *_args)
+void STARPU_LU(cublas_trsmru)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u21)(descr, 1, _args);
+	STARPU_LU(common_trsmru)(descr, 1, _args);
 }
 #endif
 
-static struct starpu_perfmodel STARPU_LU(model_21) =
+static struct starpu_perfmodel STARPU_LU(model_trsm_ru) =
 {
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
-	.symbol = STARPU_LU_STR(lu_model_21_atlas)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ru_atlas)
 #elif defined(STARPU_GOTO)
-	.symbol = STARPU_LU_STR(lu_model_21_goto)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ru_goto)
 #elif defined(STARPU_OPENBLAS)
-	.symbol = STARPU_LU_STR(lu_model_21_openblas)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ru_openblas)
 #else
-	.symbol = STARPU_LU_STR(lu_model_21)
+	.symbol = STARPU_LU_STR(lu_model_trsm_ru)
 #endif
 };
 
-struct starpu_codelet cl21 =
+struct starpu_codelet cl_trsm_ru =
 {
-	.cpu_funcs = {STARPU_LU(cpu_u21)},
-	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_u21))},
+	.cpu_funcs = {STARPU_LU(cpu_trsmru)},
+	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_trsmru))},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {STARPU_LU(cublas_u21)},
+	.cuda_funcs = {STARPU_LU(cublas_trsmru)},
 	CAN_EXECUTE
 #elif defined(STARPU_SIMGRID)
 	.cuda_funcs = {(void*)1},
@@ -339,14 +337,14 @@ struct starpu_codelet cl21 =
 	.cuda_flags = {STARPU_CUDA_ASYNC},
 	.nbuffers = 2,
 	.modes = {STARPU_R, STARPU_RW},
-	.model = &STARPU_LU(model_21)
+	.model = &STARPU_LU(model_trsm_ru)
 };
 
 /*
- *	U11
+ *	GETRF
  */
 
-static inline void STARPU_LU(common_u11)(void *descr[], int s, void *_args)
+static inline void STARPU_LU(common_getrf)(void *descr[], int s, void *_args)
 {
 	(void)_args;
 	TYPE *sub11;
@@ -359,7 +357,7 @@ static inline void STARPU_LU(common_u11)(void *descr[], int s, void *_args)
 	unsigned long z;
 
 #ifdef STARPU_USE_CUDA
-	cublasStatus status;
+	cublasStatus_t status;
 	cublasHandle_t handle;
 	cudaStream_t stream;
 #endif
@@ -383,6 +381,17 @@ static inline void STARPU_LU(common_u11)(void *descr[], int s, void *_args)
 			break;
 #ifdef STARPU_USE_CUDA
 		case 1:
+			/* TODO: use cusolver */
+#ifdef STARPU_HAVE_LIBCUSOLVER
+			{
+				cusolverStatus_t sstatus;
+				CUBLAS_TYPE *cublas_sub11 = (CUBLAS_TYPE *)sub11;
+				CUBLAS_TYPE *workspace = (CUBLAS_TYPE *)STARPU_VARIABLE_GET_PTR(descr[1]);
+
+				sstatus = CUSOLVER_GETRF(starpu_cusolverDn_get_local_handle(), nx, nx, cublas_sub11, ld, workspace, NULL, NULL);
+				STARPU_ASSERT(sstatus == CUSOLVER_STATUS_SUCCESS);
+			}
+#else
 			handle = starpu_cublas_get_local_handle();
 			stream = starpu_cuda_get_local_stream();
 			for (z = 0; z < nx; z++)
@@ -411,7 +420,7 @@ static inline void STARPU_LU(common_u11)(void *descr[], int s, void *_args)
 			}
 
 			cudaStreamSynchronize(stream);
-
+#endif
 			break;
 #endif
 		default:
@@ -420,52 +429,63 @@ static inline void STARPU_LU(common_u11)(void *descr[], int s, void *_args)
 	}
 }
 
-void STARPU_LU(cpu_u11)(void *descr[], void *_args)
+void STARPU_LU(cpu_getrf)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u11)(descr, 0, _args);
+	STARPU_LU(common_getrf)(descr, 0, _args);
 }
 
 #ifdef STARPU_USE_CUDA
-void STARPU_LU(cublas_u11)(void *descr[], void *_args)
+void STARPU_LU(cublas_getrf)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u11)(descr, 1, _args);
+	STARPU_LU(common_getrf)(descr, 1, _args);
 }
 #endif /* STARPU_USE_CUDA */
 
-static struct starpu_perfmodel STARPU_LU(model_11) =
+static struct starpu_perfmodel STARPU_LU(model_getrf) =
 {
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
-	.symbol = STARPU_LU_STR(lu_model_11_atlas)
+	.symbol = STARPU_LU_STR(lu_model_getrf_atlas)
 #elif defined(STARPU_GOTO)
-	.symbol = STARPU_LU_STR(lu_model_11_goto)
+	.symbol = STARPU_LU_STR(lu_model_getrf_goto)
 #elif defined(STARPU_OPENBLAS)
-	.symbol = STARPU_LU_STR(lu_model_11_openblas)
+	.symbol = STARPU_LU_STR(lu_model_getrf_openblas)
 #else
-	.symbol = STARPU_LU_STR(lu_model_11)
+	.symbol = STARPU_LU_STR(lu_model_getrf)
 #endif
 };
 
-struct starpu_codelet cl11 =
+struct starpu_codelet cl_getrf =
 {
-	.cpu_funcs = {STARPU_LU(cpu_u11)},
-	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_u11))},
+	.cpu_funcs = {STARPU_LU(cpu_getrf)},
+	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_getrf))},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {STARPU_LU(cublas_u11)},
+	.cuda_funcs = {STARPU_LU(cublas_getrf)},
 	CAN_EXECUTE
+#if defined(STARPU_HAVE_LIBCUSOLVER)
+	.cuda_flags = {STARPU_CUDA_ASYNC},
+#endif
 #elif defined(STARPU_SIMGRID)
 	.cuda_funcs = {(void*)1},
 #endif
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_LIBCUSOLVER)
+	.nbuffers = 2,
+#else
 	.nbuffers = 1,
-	.modes = {STARPU_RW},
-	.model = &STARPU_LU(model_11)
+#endif
+	.modes = { STARPU_RW
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_LIBCUSOLVER)
+		, STARPU_SCRATCH | STARPU_NOFOOTPRINT
+#endif
+	},
+	.model = &STARPU_LU(model_getrf)
 };
 
 /*
- *	U11 with pivoting
+ *	GETRF with pivoting
  */
 
-static inline void STARPU_LU(common_u11_pivot)(void *descr[],
+static inline void STARPU_LU(common_getrf_pivot)(void *descr[],
 				int s, void *_args)
 {
 	TYPE *sub11;
@@ -482,7 +502,7 @@ static inline void STARPU_LU(common_u11_pivot)(void *descr[],
 	unsigned first = piv->first;
 
 #ifdef STARPU_USE_CUDA
-	cublasStatus status;
+	cublasStatus_t status;
 	cublasHandle_t handle;
 	cudaStream_t stream;
 #endif
@@ -591,45 +611,46 @@ static inline void STARPU_LU(common_u11_pivot)(void *descr[],
 	}
 }
 
-void STARPU_LU(cpu_u11_pivot)(void *descr[], void *_args)
+void STARPU_LU(cpu_getrf_pivot)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u11_pivot)(descr, 0, _args);
+	STARPU_LU(common_getrf_pivot)(descr, 0, _args);
 }
 
 #ifdef STARPU_USE_CUDA
-void STARPU_LU(cublas_u11_pivot)(void *descr[], void *_args)
+void STARPU_LU(cublas_getrf_pivot)(void *descr[], void *_args)
 {
-	STARPU_LU(common_u11_pivot)(descr, 1, _args);
+	STARPU_LU(common_getrf_pivot)(descr, 1, _args);
 }
 #endif /* STARPU_USE_CUDA */
 
-static struct starpu_perfmodel STARPU_LU(model_11_pivot) =
+static struct starpu_perfmodel STARPU_LU(model_getrf_pivot) =
 {
 	.type = STARPU_HISTORY_BASED,
 #ifdef STARPU_ATLAS
-	.symbol = STARPU_LU_STR(lu_model_11_pivot_atlas)
+	.symbol = STARPU_LU_STR(lu_model_getrf_pivot_atlas)
 #elif defined(STARPU_GOTO)
-	.symbol = STARPU_LU_STR(lu_model_11_pivot_goto)
+	.symbol = STARPU_LU_STR(lu_model_getrf_pivot_goto)
 #elif defined(STARPU_OPENBLAS)
-	.symbol = STARPU_LU_STR(lu_model_11_pivot_openblas)
+	.symbol = STARPU_LU_STR(lu_model_getrf_pivot_openblas)
 #else
-	.symbol = STARPU_LU_STR(lu_model_11_pivot)
+	.symbol = STARPU_LU_STR(lu_model_getrf_pivot)
 #endif
 };
 
-struct starpu_codelet cl11_pivot =
+struct starpu_codelet cl_getrf_pivot =
 {
-	.cpu_funcs = {STARPU_LU(cpu_u11_pivot)},
-	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_u11_pivot))},
+	.cpu_funcs = {STARPU_LU(cpu_getrf_pivot)},
+	// It uses shared-memory cl_arg
+	//.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_getrf_pivot))},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {STARPU_LU(cublas_u11_pivot)},
+	.cuda_funcs = {STARPU_LU(cublas_getrf_pivot)},
 	CAN_EXECUTE
 #elif defined(STARPU_SIMGRID)
 	.cuda_funcs = {(void*)1},
 #endif
 	.nbuffers = 1,
 	.modes = {STARPU_RW},
-	.model = &STARPU_LU(model_11_pivot)
+	.model = &STARPU_LU(model_getrf_pivot)
 };
 
 /*
@@ -652,7 +673,7 @@ static inline void STARPU_LU(common_pivot)(void *descr[],
 	unsigned first = piv->first;
 
 #ifdef STARPU_USE_CUDA
-	cublasStatus status;
+	cublasStatus_t status;
 	cublasHandle_t handle;
 #endif
 
@@ -723,7 +744,8 @@ static struct starpu_perfmodel STARPU_LU(model_pivot) =
 struct starpu_codelet cl_pivot =
 {
 	.cpu_funcs = {STARPU_LU(cpu_pivot)},
-	.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_pivot))},
+	// It uses shared-memory cl_arg
+	//.cpu_funcs_name = {STRINGIFY(STARPU_LU(cpu_pivot))},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {STARPU_LU(cublas_pivot)},
 	CAN_EXECUTE
@@ -735,3 +757,20 @@ struct starpu_codelet cl_pivot =
 	.modes = {STARPU_RW},
 	.model = &STARPU_LU(model_pivot)
 };
+
+void lu_kernel_init(int nb)
+{
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_LIBCUSOLVER)
+	int Lwork = 0;
+	if (starpu_cuda_worker_get_count())
+		CUSOLVER_GETRF_BUFFERSIZE(starpu_cusolverDn_get_local_handle(), nb, nb, NULL, nb, &Lwork);
+	starpu_variable_data_register(&scratch, -1, 0, Lwork * sizeof(TYPE));
+#endif
+}
+
+void lu_kernel_fini(void)
+{
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_LIBCUSOLVER)
+	starpu_data_unregister(scratch);
+#endif
+}
