@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2015  Université de Bordeaux
- * Copyright (C) 2012, 2013, 2016, 2017  CNRS
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,14 +25,14 @@
  *   the boundaries into the block and perform some update loops:
  *
  *     comp. buffer      save. buffers        comp. buffer        save. buffers        comp. buffer
- *   |     ...     |                                                                                      
- *   |             | +------------------+ +------------------+                                            
- *   |     #N+1    | | #N+1 bottom copy====>#N+1 bottom copy |                                            
- *   +-------------+ +------------------+ +------------------+                                            
- *   | #N top copy | |   #N top copy    | |                  |                                            
- *   +-------------+ +------------------+ |                  |                                            
- *                                        | #N               |                                            
- *                                                 ...                                                    
+ *   |     ...     |
+ *   |             | +------------------+ +------------------+
+ *   |     #N+1    | | #N+1 bottom copy====>#N+1 bottom copy |
+ *   +-------------+ +------------------+ +------------------+
+ *   | #N top copy | |   #N top copy    | |                  |
+ *   +-------------+ +------------------+ |                  |
+ *                                        | #N               |
+ *                                                 ...
  *                                        |                  | +----------------+ +----------------------+
  *                                        |                  | | #N bottom copy | | block #N bottom copy |
  * ^                                      +------------------+ +----------------+ +----------------------+
@@ -46,14 +45,14 @@
  *   this block).
  *
  *     comp. buffer      save. buffers        comp. buffer        save. buffers        comp. buffer
- *   |     ...     |                                                                                      
- *   |             | +------------------+ +------------------+                                            
- *   |     #N+1    | | #N+1 bottom copy | | #N+1 bottom copy |                                            
- *   +-------------+ +------------------+ +------------------+                                            
- *   | #N top copy | |   #N top copy   <====                 |                                            
- *   +-------------+ +------------------+ |..................|                                            
- *                                        | #N               |                                            
- *                                                 ...                                                    
+ *   |     ...     |
+ *   |             | +------------------+ +------------------+
+ *   |     #N+1    | | #N+1 bottom copy | | #N+1 bottom copy |
+ *   +-------------+ +------------------+ +------------------+
+ *   | #N top copy | |   #N top copy   <====                 |
+ *   +-------------+ +------------------+ |..................|
+ *                                        | #N               |
+ *                                                 ...
  *                                        |                  | +----------------+ +----------------------+
  *                                        |                  | | #N bottom copy | | block #N bottom copy |
  * ^                                      +------------------+ +----------------+ +----------------------+
@@ -63,14 +62,14 @@
  *
  * - save_cl_bottom, same for the bottom
  *     comp. buffer      save. buffers        comp. buffer        save. buffers        comp. buffer
- *   |     ...     |                                                                                      
- *   |             | +------------------+ +------------------+                                            
- *   |     #N+1    | | #N+1 bottom copy | | #N+1 bottom copy |                                            
- *   +-------------+ +------------------+ +------------------+                                            
- *   | #N top copy | |   #N top copy    | |                  |                                            
- *   +-------------+ +------------------+ |                  |                                            
- *                                        | #N               |                                            
- *                                                 ...                                                    
+ *   |     ...     |
+ *   |             | +------------------+ +------------------+
+ *   |     #N+1    | | #N+1 bottom copy | | #N+1 bottom copy |
+ *   +-------------+ +------------------+ +------------------+
+ *   | #N top copy | |   #N top copy    | |                  |
+ *   +-------------+ +------------------+ |                  |
+ *                                        | #N               |
+ *                                                 ...
  *                                        |..................| +----------------+ +----------------------+
  *                                        |                 ====>#N bottom copy | | block #N bottom copy |
  * ^                                      +------------------+ +----------------+ +----------------------+
@@ -184,17 +183,17 @@ static void update_func_cuda(void *descr[], void *arg)
 {
 	struct block_description *block = arg;
 	int workerid = starpu_worker_get_id_check();
-	DEBUG( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+	DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
 	if (block->bz == 0)
 		FPRINTF(stderr,"!!! DO update_func_cuda z %u CUDA%d !!!\n", block->bz, workerid);
 	else
-		DEBUG( "!!! DO update_func_cuda z %u CUDA%d !!!\n", block->bz, workerid);
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+		DEBUG("!!! DO update_func_cuda z %u CUDA%d !!!\n", block->bz, workerid);
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	int rank = 0;
 	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-	DEBUG( "!!!           RANK %d              !!!\n", rank);
+	DEBUG("!!!           RANK %d              !!!\n", rank);
 #endif
-	DEBUG( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+	DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
 
 	unsigned block_size_z = get_block_size(block->bz);
 	unsigned i;
@@ -235,9 +234,6 @@ static void update_func_cuda(void *descr[], void *arg)
 		cudaMemcpyAsync(newer, old, oldb->nx * oldb->ny * oldb->nz * sizeof(*newer), cudaMemcpyDeviceToDevice, starpu_cuda_get_local_stream());
 #endif /* LIFE */
 	}
-
-	if (block->bz == 0)
-		starpu_top_update_data_integer(starpu_top_achieved_loop, ++achieved_iter);
 }
 #endif /* STARPU_USE_CUDA */
 
@@ -258,9 +254,9 @@ static void load_subblock_from_buffer_opencl(struct starpu_block_interface *bloc
 	cl_mem block_data = (cl_mem)block->dev_handle;
 	cl_mem boundary_data = (cl_mem)boundary->dev_handle;
 
-        cl_command_queue cq;
-        starpu_opencl_get_current_queue(&cq);
-        cl_int ret = clEnqueueCopyBuffer(cq, boundary_data, block_data, 0, offset, boundary_size, 0, NULL, NULL);
+	cl_command_queue cq;
+	starpu_opencl_get_current_queue(&cq);
+	cl_int ret = clEnqueueCopyBuffer(cq, boundary_data, block_data, 0, offset, boundary_size, 0, NULL, NULL);
 	if (ret != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(ret);
 }
 
@@ -271,17 +267,17 @@ static void update_func_opencl(void *descr[], void *arg)
 {
 	struct block_description *block = arg;
 	int workerid = starpu_worker_get_id_check();
-	DEBUG( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+	DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
 	if (block->bz == 0)
 		FPRINTF(stderr,"!!! DO update_func_opencl z %u OPENCL%d !!!\n", block->bz, workerid);
 	else
-		DEBUG( "!!! DO update_func_opencl z %u OPENCL%d !!!\n", block->bz, workerid);
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+		DEBUG("!!! DO update_func_opencl z %u OPENCL%d !!!\n", block->bz, workerid);
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	int rank = 0;
 	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-	DEBUG( "!!!           RANK %d              !!!\n", rank);
+	DEBUG("!!!           RANK %d              !!!\n", rank);
 #endif
-	DEBUG( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+	DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
 
 	unsigned block_size_z = get_block_size(block->bz);
 	unsigned i;
@@ -289,8 +285,8 @@ static void update_func_opencl(void *descr[], void *arg)
 
 	record_who_runs_what(block);
 
-        cl_command_queue cq;
-        starpu_opencl_get_current_queue(&cq);
+	cl_command_queue cq;
+	starpu_opencl_get_current_queue(&cq);
 
 	/*
 	 *	Load neighbours' boundaries : TOP
@@ -323,14 +319,11 @@ static void update_func_opencl(void *descr[], void *arg)
 		opencl_life_update_host(block->bz, old, newer, oldb->nx, oldb->ny, oldb->nz, oldb->ldy, oldb->ldz, i);
 #else
 		cl_event event;
-                cl_int ret = clEnqueueCopyBuffer(cq, old, newer, 0, 0, oldb->nx * oldb->ny * oldb->nz * sizeof(*newer), 0, NULL, &event);
+		cl_int ret = clEnqueueCopyBuffer(cq, old, newer, 0, 0, oldb->nx * oldb->ny * oldb->nz * sizeof(*newer), 0, NULL, &event);
 		if (ret != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(ret);
 
 #endif /* LIFE */
 	}
-
-	if (block->bz == 0)
-		starpu_top_update_data_integer(starpu_top_achieved_loop, ++achieved_iter);
 }
 #endif /* STARPU_USE_OPENCL */
 
@@ -341,17 +334,17 @@ void update_func_cpu(void *descr[], void *arg)
 {
 	struct block_description *block = (struct block_description *) arg;
 	int workerid = starpu_worker_get_id_check();
-	DEBUG( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+	DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
 	if (block->bz == 0)
-		FPRINTF(stderr,"!!! DO update_func_cpu z %u CPU%d !!!\n", block->bz, workerid);
+		FPRINTF(stderr,"!!! DO update_func_cpu z %u worker%d !!!\n", block->bz, workerid);
 	else
-		DEBUG( "!!! DO update_func_cpu z %u CPU%d !!!\n", block->bz, workerid);
-#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID)
+		DEBUG("!!! DO update_func_cpu z %u worker%d !!!\n", block->bz, workerid);
+#if defined(STARPU_USE_MPI) && !defined(STARPU_SIMGRID) && !defined(STARPU_USE_MPI_MASTER_SLAVE)
 	int rank = 0;
 	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-	DEBUG( "!!!           RANK %d            !!!\n", rank);
+	DEBUG("!!!            RANK %d              !!!\n", rank);
 #endif
-	DEBUG( "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
+	DEBUG("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
 
 	unsigned block_size_z = get_block_size(block->bz);
 	unsigned i;
@@ -403,16 +396,13 @@ void update_func_cpu(void *descr[], void *arg)
 		memcpy(newer, old, oldb->nx * oldb->ny * oldb->nz * sizeof(*newer));
 #endif /* LIFE */
 	}
-
-	if (block->bz == 0)
-		starpu_top_update_data_integer(starpu_top_achieved_loop, ++achieved_iter);
 }
 
 /* Performance model and codelet structure */
 static struct starpu_perfmodel cl_update_model =
 {
 	.type = STARPU_HISTORY_BASED,
-	.symbol = "cl_update" 
+	.symbol = "cl_update"
 };
 
 struct starpu_codelet cl_update =
@@ -488,10 +478,10 @@ static void load_subblock_into_buffer_opencl(struct starpu_block_interface *bloc
 	cl_mem block_data = (cl_mem)block->dev_handle;
 	cl_mem boundary_data = (cl_mem)boundary->dev_handle;
 
-        cl_command_queue cq;
-        starpu_opencl_get_current_queue(&cq);
+	cl_command_queue cq;
+	starpu_opencl_get_current_queue(&cq);
 
-        cl_int ret = clEnqueueCopyBuffer(cq, block_data, boundary_data, offset, 0, boundary_size, 0, NULL, NULL);
+	cl_int ret = clEnqueueCopyBuffer(cq, block_data, boundary_data, offset, 0, boundary_size, 0, NULL, NULL);
 	if (ret != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(ret);
 }
 #endif /* STARPU_USE_OPENCL */
@@ -501,13 +491,13 @@ unsigned top_per_worker[STARPU_NMAXWORKERS];
 unsigned bottom_per_worker[STARPU_NMAXWORKERS];
 
 /* top save, CPU version */
-void dummy_func_top_cpu(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
+void dummy_func_top_cpu(void *descr[], void *arg)
 {
 	struct block_description *block = (struct block_description *) arg;
 	int workerid = starpu_worker_get_id_check();
 	top_per_worker[workerid]++;
 
-	DEBUG( "DO SAVE Bottom block %d\n", block->bz);
+	DEBUG("DO SAVE Bottom block %d\n", block->bz);
 
 	/* The offset along the z axis is (block_size_z + K)- K */
 	unsigned block_size_z = get_block_size(block->bz);
@@ -517,14 +507,14 @@ void dummy_func_top_cpu(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
 }
 
 /* bottom save, CPU version */
-void dummy_func_bottom_cpu(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
+void dummy_func_bottom_cpu(void *descr[], void *arg)
 {
 	struct block_description *block = (struct block_description *) arg;
 	(void) block;
 	int workerid = starpu_worker_get_id_check();
 	bottom_per_worker[workerid]++;
 
-	DEBUG( "DO SAVE Top block %d\n", block->bz);
+	DEBUG("DO SAVE Top block %d\n", block->bz);
 
 	load_subblock_into_buffer_cpu(descr[0], descr[2], K);
 	load_subblock_into_buffer_cpu(descr[1], descr[3], K);
@@ -532,13 +522,13 @@ void dummy_func_bottom_cpu(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
 
 /* top save, CUDA version */
 #ifdef STARPU_USE_CUDA
-static void dummy_func_top_cuda(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
+static void dummy_func_top_cuda(void *descr[], void *arg)
 {
 	struct block_description *block = (struct block_description *) arg;
 	int workerid = starpu_worker_get_id_check();
 	top_per_worker[workerid]++;
 
-	DEBUG( "DO SAVE Top block %d\n", block->bz);
+	DEBUG("DO SAVE Top block %d\n", block->bz);
 
 	/* The offset along the z axis is (block_size_z + K)- K */
 	unsigned block_size_z = get_block_size(block->bz);
@@ -548,14 +538,14 @@ static void dummy_func_top_cuda(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg
 }
 
 /* bottom save, CUDA version */
-static void dummy_func_bottom_cuda(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
+static void dummy_func_bottom_cuda(void *descr[], void *arg)
 {
 	struct block_description *block = (struct block_description *) arg;
 	(void) block;
 	int workerid = starpu_worker_get_id_check();
 	bottom_per_worker[workerid]++;
 
-	DEBUG( "DO SAVE Bottom block %d on CUDA\n", block->bz);
+	DEBUG("DO SAVE Bottom block %d on CUDA\n", block->bz);
 
 	load_subblock_into_buffer_cuda(descr[0], descr[2], K);
 	load_subblock_into_buffer_cuda(descr[1], descr[3], K);
@@ -564,14 +554,14 @@ static void dummy_func_bottom_cuda(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *
 
 /* top save, OpenCL version */
 #ifdef STARPU_USE_OPENCL
-static void dummy_func_top_opencl(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
+static void dummy_func_top_opencl(void *descr[], void *arg)
 {
 	struct block_description *block = (struct block_description *) arg;
 	(void) block;
 	int workerid = starpu_worker_get_id_check();
 	top_per_worker[workerid]++;
 
-	DEBUG( "DO SAVE Top block %d\n", block->bz);
+	DEBUG("DO SAVE Top block %d\n", block->bz);
 
 	/* The offset along the z axis is (block_size_z + K)- K */
 	unsigned block_size_z = get_block_size(block->bz);
@@ -581,14 +571,14 @@ static void dummy_func_top_opencl(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *a
 }
 
 /* bottom save, OPENCL version */
-static void dummy_func_bottom_opencl(void *descr[] STARPU_ATTRIBUTE_UNUSED, void *arg)
+static void dummy_func_bottom_opencl(void *descr[], void *arg)
 {
 	struct block_description *block = (struct block_description *) arg;
 	(void) block;
 	int workerid = starpu_worker_get_id_check();
 	bottom_per_worker[workerid]++;
 
-	DEBUG( "DO SAVE Bottom block %d on OPENCL\n", block->bz);
+	DEBUG("DO SAVE Bottom block %d on OPENCL\n", block->bz);
 
 	load_subblock_into_buffer_opencl(descr[0], descr[2], K);
 	load_subblock_into_buffer_opencl(descr[1], descr[3], K);
@@ -599,13 +589,13 @@ static void dummy_func_bottom_opencl(void *descr[] STARPU_ATTRIBUTE_UNUSED, void
 static struct starpu_perfmodel save_cl_bottom_model =
 {
 	.type = STARPU_HISTORY_BASED,
-	.symbol = "save_cl_bottom" 
+	.symbol = "save_cl_bottom"
 };
 
 static struct starpu_perfmodel save_cl_top_model =
 {
 	.type = STARPU_HISTORY_BASED,
-	.symbol = "save_cl_top" 
+	.symbol = "save_cl_top"
 };
 
 struct starpu_codelet save_cl_bottom =
