@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,7 +19,7 @@
 
 #include <starpu_config.h>
 
-#if defined STARPU_USE_CUDA && !defined STARPU_DONT_INCLUDE_CUDA_HEADERS
+#ifdef STARPU_USE_CUDA
 #include <cuda.h>
 #include <cuda_runtime.h>
 #include <cuda_runtime_api.h>
@@ -29,8 +29,7 @@
 #endif
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -40,6 +39,7 @@ extern "C"
 
 /**
    Report a CUBLAS error.
+   See \ref CUDASupport for more details.
 */
 void starpu_cublas_report_error(const char *func, const char *file, int line, int status);
 
@@ -50,6 +50,7 @@ void starpu_cublas_report_error(const char *func, const char *file, int line, in
 
 /**
    Report a CUDA error.
+   See \ref CUDASupport for more details.
 */
 void starpu_cuda_report_error(const char *func, const char *file, int line, cudaError_t status);
 
@@ -68,12 +69,13 @@ void starpu_cuda_report_error(const char *func, const char *file, int line, cuda
    create its own streams. Synchronizing with
    <c>cudaDeviceSynchronize()</c> is allowed, but will reduce the
    likelihood of having all transfers overlapped.
+   See \ref CUDA-specificOptimizations for more details.
 */
 cudaStream_t starpu_cuda_get_local_stream(void);
 
 /**
    Return a pointer to device properties for worker \p workerid
-   (assumed to be a CUDA worker).
+   (assumed to be a CUDA worker). See \ref EnablingImplementationAccordingToCapabilities for more details.
 */
 const struct cudaDeviceProp *starpu_cuda_get_device_properties(unsigned workerid);
 
@@ -85,6 +87,8 @@ const struct cudaDeviceProp *starpu_cuda_get_device_properties(unsigned workerid
    data synchronously. The function returns <c>-EAGAIN</c> if the
    asynchronous launch was successfull. It returns 0 if the synchronous
    copy was successful, or fails otherwise.
+
+   See \ref CUDASupport for more details.
 */
 int starpu_cuda_copy_async_sync(void *src_ptr, unsigned src_node, void *dst_ptr, unsigned dst_node, size_t ssize, cudaStream_t stream, enum cudaMemcpyKind kind);
 
@@ -100,6 +104,8 @@ int starpu_cuda_copy_async_sync(void *src_ptr, unsigned src_node, void *dst_ptr,
    it copies the data synchronously. The function returns <c>-EAGAIN</c> if the
    asynchronous launch was successfull. It returns 0 if the synchronous copy was
    successful, or fails otherwise.
+
+   See \ref CUDASupport for more details.
 */
 int starpu_cuda_copy2d_async_sync(void *src_ptr, unsigned src_node, void *dst_ptr, unsigned dst_node,
 				  size_t blocksize,
@@ -119,6 +125,8 @@ int starpu_cuda_copy2d_async_sync(void *src_ptr, unsigned src_node, void *dst_pt
    it copies the data synchronously. The function returns <c>-EAGAIN</c> if the
    asynchronous launch was successfull. It returns 0 if the synchronous copy was
    successful, or fails otherwise.
+
+   See \ref CUDASupport for more details.
 */
 int starpu_cuda_copy3d_async_sync(void *src_ptr, unsigned src_node, void *dst_ptr, unsigned dst_node,
 				  size_t blocksize,
@@ -130,23 +138,25 @@ int starpu_cuda_copy3d_async_sync(void *src_ptr, unsigned src_node, void *dst_pt
    Call <c>cudaSetDevice(\p devid)</c> or <c>cudaGLSetGLDevice(\p devid)</c>,
    according to whether \p devid is among the field
    starpu_conf::cuda_opengl_interoperability.
+
+   See \ref CUDASupport for more details.
 */
 void starpu_cuda_set_device(unsigned devid);
 
 #ifdef STARPU_HAVE_LIBNVIDIA_ML
 /**
   Return the nvml device for a CUDA device
+  See \ref CUDASupport for more details.
 */
 nvmlDevice_t starpu_cuda_get_nvmldev(unsigned devid);
 #endif
 
-
 /** @} */
 
 #ifdef __cplusplus
 }
 #endif
 
-#endif /* STARPU_USE_CUDA && !STARPU_DONT_INCLUDE_CUDA_HEADERS */
+#endif /* STARPU_USE_CUDA */
 
 #endif /* __STARPU_CUDA_H__ */
