@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Simon Archipoff
  * Copyright (C) 2017       Arthur Chevalier
  *
@@ -26,8 +26,7 @@
 #endif
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -40,10 +39,8 @@ extern "C"
 */
 enum starpu_sched_component_properties
 {
-	/** indicate that all workers have the same starpu_worker_archtype */
-	STARPU_SCHED_COMPONENT_HOMOGENEOUS = (1<<0),
-	/** indicate that all workers have the same memory component */
-	STARPU_SCHED_COMPONENT_SINGLE_MEMORY_NODE = (1<<1)
+	STARPU_SCHED_COMPONENT_HOMOGENEOUS	  = (1 << 0), /**< indicate that all workers have the same starpu_worker_archtype */
+	STARPU_SCHED_COMPONENT_SINGLE_MEMORY_NODE = (1 << 1)  /**< indicate that all workers have the same memory component */
 };
 
 /**
@@ -69,14 +66,14 @@ struct starpu_sched_component
 	/** The tree containing the component*/
 	struct starpu_sched_tree *tree;
 	/** set of underlying workers */
-	struct starpu_bitmap *workers;
+	struct starpu_bitmap workers;
 	/**
 	   subset of starpu_sched_component::workers that is currently available in the context
 	   The push method should take this value into account, it is set with:
 	   component->workers UNION tree->workers UNION
 	   component->child[i]->workers_in_ctx iff exist x such as component->children[i]->parents[x] == component
 	*/
-	struct starpu_bitmap *workers_in_ctx;
+	struct starpu_bitmap workers_in_ctx;
 	/** private data */
 	void *data;
 	char *name;
@@ -138,7 +135,12 @@ struct starpu_sched_component
 	*/
 	int (*can_pull)(struct starpu_sched_component *component);
 
-	int (*notify)(struct starpu_sched_component* component, int message_ID, void* arg);
+	/**
+	   This function is called when starpu_do_schedule() is called by the application.
+	*/
+	void (*do_schedule)(struct starpu_sched_component *component);
+
+	int (*notify)(struct starpu_sched_component *component, int message_ID, void *arg);
 
 	/**
 	   heuristic to compute load of scheduler module. Basically the number of tasks divided by the sum
@@ -188,7 +190,7 @@ struct starpu_sched_tree
 	/**
 	   set of workers available in this context, this value is used to mask workers in modules
 	*/
-	struct starpu_bitmap *workers;
+	struct starpu_bitmap workers;
 	/**
 	   context id of the scheduler
 	*/
@@ -200,36 +202,48 @@ struct starpu_sched_tree
 	starpu_pthread_mutex_t lock;
 };
 
-void starpu_initialize_prio_center_policy(unsigned sched_ctx_id);
-
 /**
    @name Scheduling Tree API
    @{
 */
 
 /**
-   create a empty initialized starpu_sched_tree
+   create a empty initialized starpu_sched_tree.
+   See \ref ImplementAModularizedScheduler for more details.
 */
 struct starpu_sched_tree *starpu_sched_tree_create(unsigned sched_ctx_id) STARPU_ATTRIBUTE_MALLOC;
 /**
    destroy tree and free all non shared component in it.
+   See \ref ImplementAModularizedScheduler for more details.
 */
 void starpu_sched_tree_destroy(struct starpu_sched_tree *tree);
+/**
+   calls starpu_sched_tree_destroy, ready for use for starpu_sched_policy::deinit_sched field.
+   See \ref ImplementAModularizedScheduler for more details.
+ */
+void starpu_sched_tree_deinitialize(unsigned sched_ctx_id);
+/**
+   See \ref ImplementAModularizedScheduler for more details.
+ */
 struct starpu_sched_tree *starpu_sched_tree_get(unsigned sched_ctx_id);
 /**
    recursively set all starpu_sched_component::workers, do not take into account shared parts (except workers).
+	See \ref ImplementAModularizedScheduler for more details.
 */
 void starpu_sched_tree_update_workers(struct starpu_sched_tree *t);
 /**
    recursively set all starpu_sched_component::workers_in_ctx, do not take into account shared parts (except workers)
+	See \ref ImplementAModularizedScheduler for more details.
 */
 void starpu_sched_tree_update_workers_in_ctx(struct starpu_sched_tree *t);
 /**
-   compatibility with starpu_sched_policy interface
+   compatibility with starpu_sched_policy interface.
+   See \ref ImplementAModularizedScheduler for more details.
 */
 int starpu_sched_tree_push_task(struct starpu_task *task);
 /**
-   compatibility with starpu_sched_policy interface
+   compatibility with starpu_sched_policy interface.
+   See \ref ImplementAModularizedScheduler for more details.
 */
 struct starpu_task *starpu_sched_tree_pop_task(unsigned sched_ctx);
 
@@ -243,9 +257,9 @@ int starpu_sched_component_push_task(struct starpu_sched_component *from, struct
 */
 struct starpu_task *starpu_sched_component_pull_task(struct starpu_sched_component *from, struct starpu_sched_component *to);
 
-struct starpu_task* starpu_sched_component_pump_to(struct starpu_sched_component *component, struct starpu_sched_component *to, int* success);
-struct starpu_task* starpu_sched_component_pump_downstream(struct starpu_sched_component *component, int* success);
-int starpu_sched_component_send_can_push_to_parents(struct starpu_sched_component * component);
+struct starpu_task *starpu_sched_component_pump_to(struct starpu_sched_component *component, struct starpu_sched_component *to, int *success);
+struct starpu_task *starpu_sched_component_pump_downstream(struct starpu_sched_component *component, int *success);
+int starpu_sched_component_send_can_push_to_parents(struct starpu_sched_component *component);
 /**
    compatibility with starpu_sched_policy interface
 */
@@ -255,6 +269,11 @@ void starpu_sched_tree_add_workers(unsigned sched_ctx_id, int *workerids, unsign
 */
 void starpu_sched_tree_remove_workers(unsigned sched_ctx_id, int *workerids, unsigned nworkers);
 
+/**
+   Run the do_schedule method of the components. This is a helper for starpu_sched_policy::do_schedule.
+*/
+void starpu_sched_tree_do_schedule(unsigned sched_ctx_id);
+
 /**
    Attach component \p child to parent \p parent. Some component may accept only one child, others accept several (e.g. MCT)
 */
@@ -267,7 +286,7 @@ void starpu_sched_component_connect(struct starpu_sched_component *parent, struc
    @{
 */
 
-typedef struct starpu_sched_component * (*starpu_sched_component_create_t)(struct starpu_sched_tree *tree, void *data);
+typedef struct starpu_sched_component *(*starpu_sched_component_create_t)(struct starpu_sched_tree *tree, void *data);
 
 /**
    allocate and initialize component field with defaults values :
@@ -292,7 +311,7 @@ void starpu_sched_component_destroy(struct starpu_sched_component *component);
 */
 void starpu_sched_component_destroy_rec(struct starpu_sched_component *component);
 
-void starpu_sched_component_add_child(struct starpu_sched_component* component, struct starpu_sched_component * child);
+void starpu_sched_component_add_child(struct starpu_sched_component *component, struct starpu_sched_component *child);
 
 /**
    return true iff \p component can execute \p task, this function take into account the workers available in the scheduling context
@@ -367,39 +386,40 @@ void starpu_sched_component_worker_post_exec_hook(struct starpu_task *task, unsi
 
 /**
    @name Flow-control Fifo Component API
+   These can be used as methods of components. Note: they are not to be called directly, one should really call the methods of the components.
    @{
 */
 
 /**
    default function for the pull component method, just call pull of parents until one of them returns a task
 */
-struct starpu_task * starpu_sched_component_parents_pull_task(struct starpu_sched_component * component, struct starpu_sched_component * to);
+struct starpu_task *starpu_sched_component_parents_pull_task(struct starpu_sched_component *component, struct starpu_sched_component *to);
 
 /**
    default function for the can_push component method, just call can_push of parents until one of them returns non-zero
 */
-int starpu_sched_component_can_push(struct starpu_sched_component * component, struct starpu_sched_component * to);
+int starpu_sched_component_can_push(struct starpu_sched_component *component, struct starpu_sched_component *to);
 
 /**
 default function for the can_pull component method, just call can_pull of children until one of them returns non-zero
 */
-int starpu_sched_component_can_pull(struct starpu_sched_component * component);
+int starpu_sched_component_can_pull(struct starpu_sched_component *component);
 
 /**
    function for the can_pull component method, call can_pull of all children
 */
-int starpu_sched_component_can_pull_all(struct starpu_sched_component * component);
+int starpu_sched_component_can_pull_all(struct starpu_sched_component *component);
 
 /**
    default function for the estimated_load component method, just sum up the loads
    of the children of the component.
 */
-double starpu_sched_component_estimated_load(struct starpu_sched_component * component);
+double starpu_sched_component_estimated_load(struct starpu_sched_component *component);
 
 /**
    function that can be used for the estimated_end component method, compute the minimum completion time of the children.
 */
-double starpu_sched_component_estimated_end_min(struct starpu_sched_component * component);
+double starpu_sched_component_estimated_end_min(struct starpu_sched_component *component);
 
 /**
    function that can be used for the estimated_end component method, compute
@@ -408,18 +428,19 @@ double starpu_sched_component_estimated_end_min(struct starpu_sched_component *
    used instead of starpu_sched_component_estimated_end_min when the component
    contains a queue of tasks, which thus needs to be added to the estimations.
 */
-double starpu_sched_component_estimated_end_min_add(struct starpu_sched_component * component, double exp_len);
+double starpu_sched_component_estimated_end_min_add(struct starpu_sched_component *component, double exp_len);
 
 /**
    default function for the estimated_end component method, compute the average completion time of the children.
 */
-double starpu_sched_component_estimated_end_average(struct starpu_sched_component * component);
+double starpu_sched_component_estimated_end_average(struct starpu_sched_component *component);
 
 struct starpu_sched_component_fifo_data
 {
 	unsigned ntasks_threshold;
 	double exp_len_threshold;
 	int ready;
+	int exp;
 };
 
 /**
@@ -446,6 +467,7 @@ struct starpu_sched_component_prio_data
 	unsigned ntasks_threshold;
 	double exp_len_threshold;
 	int ready;
+	int exp;
 };
 struct starpu_sched_component *starpu_sched_component_prio_create(struct starpu_sched_tree *tree, struct starpu_sched_component_prio_data *prio_data) STARPU_ATTRIBUTE_MALLOC;
 int starpu_sched_component_is_prio(struct starpu_sched_component *component);
@@ -566,7 +588,7 @@ struct starpu_sched_component_heteroprio_data
 	unsigned batch;
 };
 
-struct starpu_sched_component * starpu_sched_component_heteroprio_create(struct starpu_sched_tree *tree, struct starpu_sched_component_heteroprio_data * params) STARPU_ATTRIBUTE_MALLOC;
+struct starpu_sched_component *starpu_sched_component_heteroprio_create(struct starpu_sched_tree *tree, struct starpu_sched_component_heteroprio_data *params) STARPU_ATTRIBUTE_MALLOC;
 int starpu_sched_component_is_heteroprio(struct starpu_sched_component *component);
 
 /** @} */
@@ -607,7 +629,7 @@ int starpu_sched_component_is_perfmodel_select(struct starpu_sched_component *co
    @{
 */
 
-struct starpu_sched_component * starpu_sched_component_stage_create(struct starpu_sched_tree *tree, void *arg) STARPU_ATTRIBUTE_MALLOC;
+struct starpu_sched_component *starpu_sched_component_stage_create(struct starpu_sched_tree *tree, void *arg) STARPU_ATTRIBUTE_MALLOC;
 int starpu_sched_component_is_stage(struct starpu_sched_component *component);
 
 /** @} */
@@ -617,7 +639,7 @@ int starpu_sched_component_is_stage(struct starpu_sched_component *component);
    @{
 */
 
-struct starpu_sched_component * starpu_sched_component_userchoice_create(struct starpu_sched_tree *tree, void *arg) STARPU_ATTRIBUTE_MALLOC;
+struct starpu_sched_component *starpu_sched_component_userchoice_create(struct starpu_sched_tree *tree, void *arg) STARPU_ATTRIBUTE_MALLOC;
 int starpu_sched_component_is_userchoice(struct starpu_sched_component *component);
 
 /** @} */
@@ -633,7 +655,8 @@ int starpu_sched_component_is_userchoice(struct starpu_sched_component *componen
 struct starpu_sched_component_composed_recipe;
 
 /**
-   return an empty recipe for a composed component, it should not be used without modification
+   return an empty recipe for a composed component, it should not be used without modification.
+   See \ref ImplementAModularizedScheduler for more details.
 */
 struct starpu_sched_component_composed_recipe *starpu_sched_component_composed_recipe_create(void) STARPU_ATTRIBUTE_MALLOC;
 
@@ -697,7 +720,6 @@ struct starpu_sched_component_specs
 	int mix_heterogeneous_workers;
 };
 
-
 /**
    build a scheduler for \p sched_ctx_id according to \p s and the hwloc topology of the machine.
 */
@@ -709,77 +731,89 @@ struct starpu_sched_tree *starpu_sched_component_make_scheduler(unsigned sched_c
    @{
 */
 
-#define STARPU_SCHED_SIMPLE_DECIDE_MASK		(3<<0)
+#define STARPU_SCHED_SIMPLE_DECIDE_MASK (3 << 0)
 
 /**
    Request to create downstream queues per worker, i.e. the scheduling decision-making component will choose exactly which workers tasks should got to.
 */
-#define STARPU_SCHED_SIMPLE_DECIDE_WORKERS	(1<<0)
+#define STARPU_SCHED_SIMPLE_DECIDE_WORKERS (1 << 0)
 
 /**
    Request to create downstream queues per memory nodes, i.e. the scheduling decision-making component will choose which memory node tasks will go to.
 */
-#define STARPU_SCHED_SIMPLE_DECIDE_MEMNODES	(2<<0)
+#define STARPU_SCHED_SIMPLE_DECIDE_MEMNODES (2 << 0)
 
 /**
    Request to create downstream queues per computation arch, i.e. the scheduling decision-making component will choose whether tasks go to CPUs, or CUDA, or OpenCL, etc.
 */
-#define STARPU_SCHED_SIMPLE_DECIDE_ARCHS	(3<<0)
+#define STARPU_SCHED_SIMPLE_DECIDE_ARCHS (3 << 0)
 
 /**
    Request to create the scheduling decision-making component even if there is only one available choice. This is useful for instance when the decision-making component will store tasks itself (and not use STARPU_SCHED_SIMPLE_FIFO_ABOVE) to decide in which order tasks should be passed below.
 */
-#define STARPU_SCHED_SIMPLE_DECIDE_ALWAYS	(1<<3)
+#define STARPU_SCHED_SIMPLE_DECIDE_ALWAYS (1 << 3)
 
 /**
    Request to add a perfmodel selector above the scheduling decision-making component. That way, only tasks with a calibrated performance model will be given to the component, other tasks will go to an eager branch that will distributed tasks so that their performance models will get calibrated.
    In other words, this is needed when using a component which needs performance models for tasks.
 */
-#define STARPU_SCHED_SIMPLE_PERFMODEL		(1<<4)
+#define STARPU_SCHED_SIMPLE_PERFMODEL (1 << 4)
 
 /**
    Request that a component be added just above workers, that chooses the best task implementation.
 */
-#define STARPU_SCHED_SIMPLE_IMPL		(1<<5)
+#define STARPU_SCHED_SIMPLE_IMPL (1 << 5)
 
 /**
    Request to create a fifo above the scheduling decision-making component, otherwise tasks will be pushed directly to the component.
 
    This is useful to store tasks if there is a fifo below which limits the number of tasks to be scheduld in advance. The scheduling decision-making component can also store tasks itself, in which case this flag is not useful.
 */
-#define STARPU_SCHED_SIMPLE_FIFO_ABOVE		(1<<6)
+#define STARPU_SCHED_SIMPLE_FIFO_ABOVE (1 << 6)
 
 /**
    Request that the fifo above be sorted by priorities
 */
-#define STARPU_SCHED_SIMPLE_FIFO_ABOVE_PRIO	(1<<7)
+#define STARPU_SCHED_SIMPLE_FIFO_ABOVE_PRIO (1 << 7)
 
 /**
    Request to create fifos below the scheduling decision-making component, otherwise tasks will be pulled directly from workers.
 
    This is useful to be able to schedule a (tunable) small number of tasks in advance only.
 */
-#define STARPU_SCHED_SIMPLE_FIFOS_BELOW		(1<<8)
+#define STARPU_SCHED_SIMPLE_FIFOS_BELOW (1 << 8)
 
 /**
    Request that the fifos below be sorted by priorities
 */
-#define STARPU_SCHED_SIMPLE_FIFOS_BELOW_PRIO	(1<<9)
+#define STARPU_SCHED_SIMPLE_FIFOS_BELOW_PRIO (1 << 9)
 
 /**
    Request that the fifos below be pulled rather ready tasks
 */
-#define STARPU_SCHED_SIMPLE_FIFOS_BELOW_READY	(1<<10)
+#define STARPU_SCHED_SIMPLE_FIFOS_BELOW_READY (1 << 10)
 
 /**
    Request that work between workers using the same fifo below be distributed using a work stealing component.
 */
-#define STARPU_SCHED_SIMPLE_WS_BELOW		(1<<11)
+#define STARPU_SCHED_SIMPLE_WS_BELOW (1 << 11)
 
 /**
    Request to not only choose between simple workers, but also choose between combined workers.
 */
-#define STARPU_SCHED_SIMPLE_COMBINED_WORKERS	(1<<12)
+#define STARPU_SCHED_SIMPLE_COMBINED_WORKERS (1 << 12)
+
+/**
+   Request that the fifos below keep track of expected duration, start and end time of theirs elements
+*/
+#define STARPU_SCHED_SIMPLE_FIFOS_BELOW_EXP (1 << 13)
+
+/**
+   Request to prepend a component before the decision component. This should be
+   used alone and followed by the component creation function pointer and its
+   data.
+*/
+#define STARPU_SCHED_SIMPLE_PRE_DECISION (1 << 14)
 
 /**
    Create a simple modular scheduler tree around a scheduling decision-making
@@ -787,6 +821,7 @@ struct starpu_sched_tree *starpu_sched_component_make_scheduler(unsigned sched_c
    is described by \p flags. The different STARPU_SCHED_SIMPL_DECIDE_* flags are
    mutually exclusive. \p data is passed to the \p create_decision_component
    function when creating the decision component.
+   See \ref ImplementAModularizedScheduler for more details.
 */
 void starpu_sched_component_initialize_simple_scheduler(starpu_sched_component_create_t create_decision_component, void *data, unsigned flags, unsigned sched_ctx_id);
 
@@ -806,17 +841,17 @@ void starpu_sched_component_initialize_simple_schedulers(unsigned sched_ctx_id,
 
 /** @} */
 
-#define STARPU_COMPONENT_MUTEX_LOCK(m) \
-do \
-{ \
-	const int _relaxed_state = starpu_worker_get_relax_state(); \
-	if (!_relaxed_state) \
-		starpu_worker_relax_on(); \
-	STARPU_PTHREAD_MUTEX_LOCK((m)); \
-	if (!_relaxed_state) \
-		starpu_worker_relax_off(); \
-} \
-while(0)
+#define STARPU_COMPONENT_MUTEX_LOCK(m)                                      \
+	do                                                                  \
+	{                                                                   \
+		const int _relaxed_state = starpu_worker_get_relax_state(); \
+		if (!_relaxed_state)                                        \
+			starpu_worker_relax_on();                           \
+		STARPU_PTHREAD_MUTEX_LOCK((m));                             \
+		if (!_relaxed_state)                                        \
+			starpu_worker_relax_off();                          \
+	}                                                                   \
+	while (0)
 
 #define STARPU_COMPONENT_MUTEX_TRYLOCK(m) STARPU_PTHREAD_MUTEX_TRYLOCK((m))
 
