@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2011       Télécom-SudParis
  * Copyright (C) 2013       Thibaut Lambert
  * Copyright (C) 2016       Uppsala University
@@ -23,8 +23,7 @@
 #include <starpu.h>
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 /**
@@ -110,6 +109,10 @@ struct starpu_sched_policy
 	   to be executed by the worker. This method therefore permits
 	   to keep the state of the scheduler coherent even when
 	   StarPU bypasses the scheduling strategy.
+
+	   Note: to get an estimation of the task duration, \p perf_workerid
+	   needs to be used rather than \p workerid, for the case of parallel
+	   tasks.
 	*/
 	void (*push_task_notify)(struct starpu_task *, int workerid, int perf_workerid, unsigned sched_ctx_id);
 
@@ -135,15 +138,6 @@ struct starpu_sched_policy
 	*/
 	struct starpu_task *(*pop_task)(unsigned sched_ctx_id);
 
-	/**
-	   Remove all available tasks from the scheduler (tasks are
-	   chained by the means of the field starpu_task::prev and
-	   starpu_task::next). The mutex associated to the worker is
-	   already taken when this method is called. This is currently
-	   not used and can be discarded.
-	*/
-	struct starpu_task *(*pop_every_task)(unsigned sched_ctx_id);
-
 	/**
 	   Optional field. This method is called when a task is
 	   submitted.
@@ -182,6 +176,11 @@ struct starpu_sched_policy
 	*/
 	void (*remove_workers)(unsigned sched_ctx_id, int *workerids, unsigned nworkers);
 
+	/** Whether this scheduling policy does data prefetching, and thus the
+	    core should not try to do it opportunistically.
+	*/
+	int prefetches;
+
 	/**
 	   Optional field. Name of the policy.
 	*/
@@ -198,8 +197,35 @@ struct starpu_sched_policy
 /**
    Return an <c>NULL</c>-terminated array of all the predefined
    scheduling policies.
+   See \ref TaskSchedulingPolicy for more details.
 */
-struct starpu_sched_policy **starpu_sched_get_predefined_policies();
+struct starpu_sched_policy **starpu_sched_get_predefined_policies(void);
+
+/**
+   Allow an external library to return a scheduling policy to be
+   loaded dynamically.
+   See \ref UsingaNewSchedulingPolicy for more details.
+ */
+struct starpu_sched_policy *starpu_get_sched_lib_policy(const char *name);
+
+/**
+   Allow an external library to return a list of scheduling policies to be
+   loaded dynamically.
+   See \ref UsingaNewSchedulingPolicy for more details.
+ */
+struct starpu_sched_policy **starpu_get_sched_lib_policies(void);
+
+/**
+   Return the scheduler policy of the default context.
+   See \ref TaskSchedulingPolicy for more details.
+*/
+struct starpu_sched_policy *starpu_sched_get_sched_policy_in_ctx(unsigned sched_ctx_id);
+
+/**
+   Return the scheduler policy of the given context.
+   See \ref TaskSchedulingPolicy for more details.
+*/
+struct starpu_sched_policy *starpu_sched_get_sched_policy(void);
 
 /**
    When there is no available task for a worker, StarPU blocks this
@@ -212,19 +238,25 @@ struct starpu_sched_policy **starpu_sched_get_predefined_policies();
 */
 void starpu_worker_get_sched_condition(int workerid, starpu_pthread_mutex_t **sched_mutex, starpu_pthread_cond_t **sched_cond);
 
+/**
+   Return the job identifier associated with the task.
+   See \ref TraceSchedTaskDetails for more details.
+*/
 unsigned long starpu_task_get_job_id(struct starpu_task *task);
 
 /**
    TODO: check if this is correct
    Return the current minimum priority level supported by the scheduling
-   policy
+   policy.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_sched_get_min_priority(void);
 
 /**
    TODO: check if this is correct
    Return the current maximum priority level supported by the
-   scheduling policy
+   scheduling policy.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_sched_get_max_priority(void);
 
@@ -237,6 +269,7 @@ int starpu_sched_get_max_priority(void);
    starpu_sched_get_min_priority(). This function should only be
    called from the initialization method of the scheduling policy, and
    should not be used directly from the application.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_sched_set_min_priority(int min_prio);
 
@@ -248,6 +281,7 @@ int starpu_sched_set_min_priority(int min_prio);
    starpu_sched_get_max_priority(). This function should only be
    called from the initialization method of the scheduling policy, and
    should not be used directly from the application.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_sched_set_max_priority(int max_prio);
 
@@ -255,6 +289,7 @@ int starpu_sched_set_max_priority(int max_prio);
    Check if the worker specified by workerid can execute the codelet.
    Schedulers need to call it before assigning a task to a worker,
    otherwise the task may fail to execute.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_worker_can_execute_task(unsigned workerid, struct starpu_task *task, unsigned nimpl);
 
@@ -267,6 +302,7 @@ int starpu_worker_can_execute_task(unsigned workerid, struct starpu_task *task,
    starpu_worker_can_execute_task() for each and every implementation.
    It can also be used with <c>impl_mask == NULL</c> to check for at
    least one implementation without determining which.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_worker_can_execute_task_impl(unsigned workerid, struct starpu_task *task, unsigned *impl_mask);
 
@@ -279,94 +315,124 @@ int starpu_worker_can_execute_task_impl(unsigned workerid, struct starpu_task *t
    each and every implementation. It can also be used with
    <c>impl_mask == NULL</c> to check for at least one implementation
    without determining which.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_worker_can_execute_task_first_impl(unsigned workerid, struct starpu_task *task, unsigned *nimpl);
 
 /**
    The scheduling policy may put tasks directly into a worker’s local
    queue so that it is not always necessary to create its own queue
-   when the local queue is sufficient. If \p back is not 0, \p task is
-   put at the back of the queue where the worker will pop tasks first.
-   Setting \p back to 0 therefore ensures a FIFO ordering.
+   when the local queue is sufficient. \p back is ignored: the task priority is
+   used to order tasks in this queue.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_push_local_task(int workerid, struct starpu_task *task, int back);
 
 /**
    Must be called by a scheduler to notify that the given
    task has just been pushed.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_push_task_end(struct starpu_task *task);
 
 /**
-   Whether \ref STARPU_PREFETCH was set
+   Whether \ref STARPU_PREFETCH was set.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_get_prefetch_flag(void);
 
 /**
    Prefetch data for a given p task on a given p node with a given
-   priority
+   priority.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_prefetch_task_input_on_node_prio(struct starpu_task *task, unsigned node, int prio);
 
 /**
-   Prefetch data for a given p task on a given p node
+   Prefetch data for a given p task on a given p node.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_prefetch_task_input_on_node(struct starpu_task *task, unsigned node);
 
 /**
    Prefetch data for a given p task on a given p node when the bus is
-   idle with a given priority
+   idle with a given priority.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_idle_prefetch_task_input_on_node_prio(struct starpu_task *task, unsigned node, int prio);
 
 /**
    Prefetch data for a given p task on a given p node when the bus is
-   idle
+   idle.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_idle_prefetch_task_input_on_node(struct starpu_task *task, unsigned node);
 
 /**
    Prefetch data for a given p task on a given p worker with a given
-   priority
+   priority.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_prefetch_task_input_for_prio(struct starpu_task *task, unsigned worker, int prio);
 
 /**
-   Prefetch data for a given p task on a given p worker
+   Prefetch data for a given p task on a given p worker.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_prefetch_task_input_for(struct starpu_task *task, unsigned worker);
 
 /**
    Prefetch data for a given p task on a given p worker when the bus
-   is idle with a given priority
+   is idle with a given priority.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_idle_prefetch_task_input_for_prio(struct starpu_task *task, unsigned worker, int prio);
 
 /**
    Prefetch data for a given p task on a given p worker when the bus
-   is idle
+   is idle.
+   See \ref SchedulingHelpers for more details.
 */
 int starpu_idle_prefetch_task_input_for(struct starpu_task *task, unsigned worker);
 
 /**
    Return the footprint for a given task, taking into account
    user-provided perfmodel footprint or size_base functions.
+   See \ref PerformanceModelExample for more details.
 */
 uint32_t starpu_task_footprint(struct starpu_perfmodel *model, struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned nimpl);
 
 /**
    Return the raw footprint for the data of a given task (without
    taking into account user-provided functions).
+   See \ref PerformanceModelExample for more details.
 */
 uint32_t starpu_task_data_footprint(struct starpu_task *task);
 
 /**
-   Return expected task duration in micro-seconds.
+   Return expected task duration in micro-seconds on a given architecture \p arch using given implementation \p nimpl.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_task_expected_length(struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned nimpl);
 
 /**
-   Return an estimated speedup factor relative to CPU speed
+   Same as starpu_task_expected_length() but for a precise worker.
+   See \ref SchedulingHelpers for more details.
+*/
+double starpu_task_worker_expected_length(struct starpu_task *task, unsigned workerid, unsigned sched_ctx_id, unsigned nimpl);
+
+/**
+   Return expected task duration in micro-seconds, averaged over the different workers driven by the scheduler \p sched_ctx_id
+   Note: this is not just the average of the durations using the number of
+   processing units as coefficients, but their efficiency at processing the
+   task, thus the harmonic average of the durations.
+   See \ref SchedulingHelpers for more details.
+*/
+double starpu_task_expected_length_average(struct starpu_task *task, unsigned sched_ctx_id);
+
+/**
+   Return an estimated speedup factor relative to CPU speed.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_worker_get_relative_speedup(struct starpu_perfmodel_arch *perf_arch);
 
@@ -374,28 +440,48 @@ double starpu_worker_get_relative_speedup(struct starpu_perfmodel_arch *perf_arc
    Return expected data transfer time in micro-seconds for the given \p
    memory_node. Prefer using starpu_task_expected_data_transfer_time_for() which is
    more precise.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_task_expected_data_transfer_time(unsigned memory_node, struct starpu_task *task);
 
 /**
    Return expected data transfer time in micro-seconds for the given
    \p worker.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_task_expected_data_transfer_time_for(struct starpu_task *task, unsigned worker);
 
 /**
    Predict the transfer time (in micro-seconds) to move \p handle to a
-   memory node
+   memory node.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_data_expected_transfer_time(starpu_data_handle_t handle, unsigned memory_node, enum starpu_data_access_mode mode);
 
 /**
-   Return expected energy consumption in J
+   Return expected energy use in J.
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_task_expected_energy(struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned nimpl);
 
 /**
-   Return expected conversion time in ms (multiformat interface only)
+   Same as starpu_task_expected_energy but for a precise worker.
+   See \ref SchedulingHelpers for more details.
+*/
+double starpu_task_worker_expected_energy(struct starpu_task *task, unsigned workerid, unsigned sched_ctx_id, unsigned nimpl);
+
+/**
+   Return expected task energy use in J, averaged over the different workers driven by the scheduler \p sched_ctx_id
+   Note: this is not just the average of the energy uses using the number of
+   processing units as coefficients, but their efficiency at processing the
+   task, thus the harmonic average of the energy uses.
+   See \ref SchedulingHelpers for more details.
+*/
+double starpu_task_expected_energy_average(struct starpu_task *task, unsigned sched_ctx_id);
+
+/**
+   Return expected conversion time in ms (multiformat interface only).
+   See \ref SchedulingHelpers for more details.
 */
 double starpu_task_expected_conversion_time(struct starpu_task *task, struct starpu_perfmodel_arch *arch, unsigned nimpl);
 
@@ -405,16 +491,27 @@ typedef void (*starpu_notify_ready_soon_func)(void *data, struct starpu_task *ta
    Register a callback to be called when it is determined when a task
    will be ready an estimated amount of time from now, because its
    last dependency has just started and we know how long it will take.
+   See \ref SchedulingHelpers for more details.
 */
 void starpu_task_notify_ready_soon_register(starpu_notify_ready_soon_func f, void *data);
 
 /**
    The scheduling policies indicates if the worker may pop tasks from
    the list of other workers or if there is a central list with task
-   for all the workers
+   for all the workers.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 void starpu_sched_ctx_worker_shares_tasks_lists(int workerid, int sched_ctx_id);
 
+/**
+   The scheduling policy should call this when it makes a scheduling decision
+   for a task. This will possibly stop execution at this point, and then the
+   programmer can inspect local variables etc. to determine why this scheduling
+   decision was done.
+
+   See \ref ::STARPU_TASK_BREAK_ON_SCHED
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
+ */
 void starpu_sched_task_break(struct starpu_task *task);
 
 /**
@@ -428,6 +525,7 @@ void starpu_sched_task_break(struct starpu_task *task);
    workerid has been woken up or its state_keep_awake flag has been
    set to \c 1, and \c 0 otherwise (if \p workerid was not in the
    STATE_SLEEPING or in the STATE_SCHEDULING).
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_wake_worker_relax(int workerid);
 
@@ -435,12 +533,14 @@ int starpu_wake_worker_relax(int workerid);
    Must be called to wake up a worker that is sleeping on the cond.
    Return 0 whenever the worker is not in a sleeping state or has the
    state_keep_awake flag on.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_wake_worker_no_relax(int workerid);
 
 /**
    Version of starpu_wake_worker_no_relax() which assumes that the
    sched mutex is locked
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_wake_worker_locked(int workerid);
 
@@ -448,6 +548,7 @@ int starpu_wake_worker_locked(int workerid);
    Light version of starpu_wake_worker_relax() which, when possible,
    speculatively set keep_awake on the target worker without waiting
    for the worker to enter the relax state.
+   See \ref DefiningANewBasicSchedulingPolicy for more details.
 */
 int starpu_wake_worker_relax_light(int workerid);
 
