@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2013,2015,2017                      CNRS
- * Copyright (C) 2009-2011,2014,2015,2017,2018            Université de Bordeaux
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2010                                     Mehdi Juhoor
+ * Copyright (C) 2009-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -42,15 +40,26 @@ int main(int argc, char **argv)
 
 	parse_args(argc, argv, nodes);
 
+	if (checkpoint_enabled)
+		starpu_mpi_checkpoint_init();
+
+	if (check)
+	{
+		fprintf(stderr,"can't check in distributed mode\n");
+		check = 0;
+	}
+
 	matrix_init(&bmat, rank, nodes, 0);
 
 	dw_cholesky(bmat, size/nblocks, rank, nodes, &timing, &flops);
 
-	starpu_cublas_shutdown();
-	starpu_mpi_shutdown();
-
 	matrix_free(&bmat, rank, nodes, 0);
 
+	starpu_cublas_shutdown();
+	if (checkpoint_enabled)
+		starpu_mpi_checkpoint_shutdown();
+	starpu_mpi_shutdown();
+
 	if (rank == 0)
 	{
 		FPRINTF(stdout, "Computation time (in ms): %2.2f\n", timing/1000);
