@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +20,7 @@
 #include <starpu.h>
 
 #ifdef __cplusplus
-extern "C"
-{
+extern "C" {
 #endif
 
 struct starpu_mpi_lb_conf
@@ -30,11 +29,12 @@ struct starpu_mpi_lb_conf
 	void (*get_data_unit_to_migrate)(starpu_data_handle_t **handle_unit, int *nhandles, int dst_node);
 };
 
-/* Inits the load balancer's environment with the load policy provided by the
- * user
- */
+/**
+   Initialize the load balancer's environment with the load policy provided by the
+   user
+*/
 void starpu_mpi_lb_init(const char *lb_policy_name, struct starpu_mpi_lb_conf *);
-void starpu_mpi_lb_shutdown();
+void starpu_mpi_lb_shutdown(void);
 
 #ifdef __cplusplus
 }
