@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,14 +27,14 @@
 
 #if defined(STARPU_USE_MPI_MPI)
 
-static int TAG_LOAD(int n)
+static starpu_mpi_tag_t TAG_LOAD(int n)
 {
-	return (n+1) << 24;
+	return ((starpu_mpi_tag_t) n+1) << 24;
 }
 
-static int TAG_MOV(int n)
+static starpu_mpi_tag_t TAG_MOV(int n)
 {
-	return (n+1) << 20;
+	return ((starpu_mpi_tag_t) n+1) << 20;
 }
 
 /* Hash table of local pieces of data that has been moved out of the local MPI
@@ -114,6 +114,7 @@ static void balance(starpu_data_handle_t load_data_cpy)
 		}
 	}
 
+	starpu_data_acquire_on_node(data_movements_handles[my_rank], STARPU_MAIN_RAM, STARPU_RW);
 	/* We found it */
 	if (less_loaded >= 0)
 	{
@@ -128,11 +129,11 @@ static void balance(starpu_data_handle_t load_data_cpy)
 			int nhandles = 0;
 			user_itf->get_data_unit_to_migrate(&handles, &nhandles, less_loaded);
 
-			data_movements_reallocate_tables(data_movements_handles[my_rank], nhandles);
+			data_movements_reallocate_tables(data_movements_handles[my_rank], STARPU_MAIN_RAM, nhandles);
 
 			if (nhandles)
 			{
-				int *tags = data_movements_get_tags_table(data_movements_handles[my_rank]);
+				starpu_mpi_tag_t *tags = data_movements_get_tags_table(data_movements_handles[my_rank]);
 				int *ranks = data_movements_get_ranks_table(data_movements_handles[my_rank]);
 
 				for (n = 0; n < nhandles; n++)
@@ -145,10 +146,11 @@ static void balance(starpu_data_handle_t load_data_cpy)
 			}
 		}
 		else
-			data_movements_reallocate_tables(data_movements_handles[my_rank], 0);
+			data_movements_reallocate_tables(data_movements_handles[my_rank], STARPU_MAIN_RAM, 0);
 	}
 	else
-		data_movements_reallocate_tables(data_movements_handles[my_rank], 0);
+		data_movements_reallocate_tables(data_movements_handles[my_rank], STARPU_MAIN_RAM, 0);
+	starpu_data_release_on_node(data_movements_handles[my_rank], STARPU_MAIN_RAM);
 }
 
 static void exchange_load_data_infos(starpu_data_handle_t load_data_cpy)
@@ -162,15 +164,18 @@ static void exchange_load_data_infos(starpu_data_handle_t load_data_cpy)
 	MPI_Status load_send_status[nneighbors];
 	MPI_Status load_recv_status[nneighbors];
 
-	int flag;
+	int flag, ret;
 
 	/* Send the local load data to neighbour nodes, and receive the remote load
 	 * data from neighbour nodes */
 	for (i = 0; i < nneighbors; i++)
 	{
 		//_STARPU_DEBUG("[node %d] sending and receiving with %i-th neighbor %i\n", my_rank, i, neighbor_ids[i]);
-		starpu_mpi_isend(load_data_cpy, &load_send_req[i], neighbor_ids[i], TAG_LOAD(my_rank), MPI_COMM_WORLD);
-		starpu_mpi_irecv(neighbor_load_data_handles[i], &load_recv_req[i], neighbor_ids[i], TAG_LOAD(neighbor_ids[i]), MPI_COMM_WORLD);
+		ret = starpu_mpi_isend(load_data_cpy, &load_send_req[i], neighbor_ids[i], TAG_LOAD(my_rank), MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_isend");
+
+		ret = starpu_mpi_irecv(neighbor_load_data_handles[i], &load_recv_req[i], neighbor_ids[i], TAG_LOAD(neighbor_ids[i]), MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_irecv");
 	}
 
 	/* Wait for completion of all send requests */
@@ -178,7 +183,10 @@ static void exchange_load_data_infos(starpu_data_handle_t load_data_cpy)
 	{
 		flag = 0;
 		while (!flag)
-			starpu_mpi_test(&load_send_req[i], &flag, &load_send_status[i]);
+		{
+			ret = starpu_mpi_test(&load_send_req[i], &flag, &load_send_status[i]);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_test");
+		}
 	}
 
 	/* Wait for completion of all receive requests */
@@ -186,7 +194,10 @@ static void exchange_load_data_infos(starpu_data_handle_t load_data_cpy)
 	{
 		flag = 0;
 		while (!flag)
-			starpu_mpi_test(&load_recv_req[i], &flag, &load_recv_status[i]);
+		{
+			ret = starpu_mpi_test(&load_recv_req[i], &flag, &load_recv_status[i]);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_test");
+		}
 	}
 }
 
@@ -201,7 +212,7 @@ static void exchange_data_movements_infos()
 	MPI_Status data_movements_send_status[world_size];
 	MPI_Status data_movements_recv_status[world_size];
 
-	int flag;
+	int flag, ret;
 
 	/* Send the new ranks of local data to all other nodes, and receive the new
 	 * ranks of all remote data from all other nodes */
@@ -210,8 +221,10 @@ static void exchange_data_movements_infos()
 		if (i != my_rank)
 		{
 			//_STARPU_DEBUG("[node %d] Send and receive data movement with %d\n", my_rank, i);
-			starpu_mpi_isend(data_movements_handles[my_rank], &data_movements_send_req[i], i, TAG_MOV(my_rank), MPI_COMM_WORLD);
-			starpu_mpi_irecv(data_movements_handles[i], &data_movements_recv_req[i], i, TAG_MOV(i), MPI_COMM_WORLD);
+			ret = starpu_mpi_isend(data_movements_handles[my_rank], &data_movements_send_req[i], i, TAG_MOV(my_rank), MPI_COMM_WORLD);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_isend");
+			ret = starpu_mpi_irecv(data_movements_handles[i], &data_movements_recv_req[i], i, TAG_MOV(i), MPI_COMM_WORLD);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_irecv");
 		}
 	}
 
@@ -223,7 +236,10 @@ static void exchange_data_movements_infos()
 			//fprintf(stderr,"Wait for sending data movement of %d to %d\n", my_rank, i);
 			flag = 0;
 			while (!flag)
-				starpu_mpi_test(&data_movements_send_req[i], &flag, &data_movements_send_status[i]);
+			{
+				ret = starpu_mpi_test(&data_movements_send_req[i], &flag, &data_movements_send_status[i]);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_test");
+			}
 		}
 	}
 
@@ -235,7 +251,10 @@ static void exchange_data_movements_infos()
 			//fprintf(stderr,"Wait for recieving data movement from %d on %d\n", i, my_rank);
 			flag = 0;
 			while (!flag)
-				starpu_mpi_test(&data_movements_recv_req[i], &flag, &data_movements_recv_status[i]);
+			{
+				ret = starpu_mpi_test(&data_movements_recv_req[i], &flag, &data_movements_recv_status[i]);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_test");
+			}
 		}
 	}
 }
@@ -291,7 +310,8 @@ static void update_data_ranks()
 				_STARPU_DEBUG("Call of starpu_mpi_get_data_on_node(%"PRIi64",%d) on node %d\n", starpu_mpi_data_get_tag(handle), dst_rank, my_rank);
 
 				/* Migrate the data handle */
-				starpu_mpi_get_data_on_node_detached(MPI_COMM_WORLD, handle, dst_rank, NULL, NULL);
+				int ret = starpu_mpi_get_data_on_node_detached(MPI_COMM_WORLD, handle, dst_rank, NULL, NULL);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_isend");
 
 				_STARPU_DEBUG("New rank (%d) of data %"PRIi64" upgraded on node %d\n", dst_rank, starpu_mpi_data_get_tag(handle), my_rank);
 				starpu_mpi_data_set_rank_comm(handle, dst_rank, MPI_COMM_WORLD);
@@ -559,12 +579,13 @@ static int deinit_heat()
 
 	unsigned int ndata_to_move_back = HASH_COUNT(mdh);
 
+	starpu_data_acquire_on_node(data_movements_handles[my_rank], STARPU_MAIN_RAM, STARPU_RW);
 	if (ndata_to_move_back)
 	{
 		_STARPU_DEBUG("Move back %u data on node %d ..\n", ndata_to_move_back, my_rank);
-		data_movements_reallocate_tables(data_movements_handles[my_rank], ndata_to_move_back);
+		data_movements_reallocate_tables(data_movements_handles[my_rank], STARPU_MAIN_RAM, ndata_to_move_back);
 
-		int *tags = data_movements_get_tags_table(data_movements_handles[my_rank]);
+		starpu_mpi_tag_t *tags = data_movements_get_tags_table(data_movements_handles[my_rank]);
 		int *ranks = data_movements_get_ranks_table(data_movements_handles[my_rank]);
 
 		int n = 0;
@@ -577,7 +598,8 @@ static int deinit_heat()
 		}
 	}
 	else
-		data_movements_reallocate_tables(data_movements_handles[my_rank], 0);
+		data_movements_reallocate_tables(data_movements_handles[my_rank], STARPU_MAIN_RAM, 0);
+	starpu_data_release_on_node(data_movements_handles[my_rank], STARPU_MAIN_RAM);
 
 	exchange_data_movements_infos();
 	move_back_data();
@@ -612,7 +634,9 @@ static int deinit_heat()
 	for (i = 0; i < world_size; i++)
 	{
 		starpu_mpi_cache_flush(MPI_COMM_WORLD, data_movements_handles[i]);
-		data_movements_reallocate_tables(data_movements_handles[i], 0);
+		starpu_data_acquire_on_node(data_movements_handles[i], STARPU_MAIN_RAM, STARPU_W);
+		data_movements_reallocate_tables(data_movements_handles[i], STARPU_MAIN_RAM, 0);
+		starpu_data_release_on_node(data_movements_handles[i], STARPU_MAIN_RAM);
 		starpu_data_unregister(data_movements_handles[i]);
 	}
 	free(data_movements_handles);
