@@ -17,6 +17,10 @@
 
 #include <stdlib.h>
 #include <limits.h>
+#include <common/config.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <starpu_mpi.h>
 #include <starpu_mpi_datatype.h>
 #include <starpu_mpi_private.h>
@@ -26,25 +30,29 @@
 #include <starpu_mpi_cache.h>
 #include <starpu_mpi_select_node.h>
 #include <starpu_mpi_init.h>
-#include <common/config.h>
 #include <common/thread.h>
 #include <datawizard/coherency.h>
 #include <core/task.h>
 #include <core/topology.h>
 
-#ifdef STARPU_USE_MPI_NMAD
-
-#include <nm_sendrecv_interface.h>
-#include <nm_mpi_nmad.h>
-#include "starpu_mpi_nmad_backend.h"
-
-static void _starpu_mpi_handle_request_termination(struct _starpu_mpi_req *req,nm_sr_event_t event);
-#ifdef STARPU_VERBOSE
-char *_starpu_mpi_request_type(enum _starpu_mpi_request_type request_type);
+#ifdef STARPU_USE_FXT
+#include <starpu_mpi_fxt.h>
 #endif
 
-static void _starpu_mpi_handle_pending_request(struct _starpu_mpi_req *req);
-static void _starpu_mpi_add_sync_point_in_fxt(void);
+#ifdef STARPU_USE_MPI_NMAD
+#include <nm_mpi_nmad.h>
+#include <nm_mcast_interface.h>
+#include <nm_sendrecv_interface.h>
+
+#include "starpu_mpi_nmad_coop.h"
+#include "starpu_mpi_nmad_backend.h"
+#include "starpu_mpi_nmad_unknown_datatype.h"
+
+void _starpu_mpi_handle_request_termination(struct _starpu_mpi_req *req);
+void _starpu_mpi_handle_pending_request(struct _starpu_mpi_req *req);
+static inline void _starpu_mpi_request_end(struct _starpu_mpi_req* req, int post_callback_sem);
+static inline void _starpu_mpi_request_try_end(struct _starpu_mpi_req* req, int post_callback_sem);
+
 
 /* Condition to wake up waiting for all current MPI requests to finish */
 static starpu_pthread_t progress_thread;
@@ -52,11 +60,13 @@ static starpu_pthread_cond_t progress_cond;
 static starpu_pthread_mutex_t progress_mutex;
 static volatile int running = 0;
 
-extern struct _starpu_mpi_req *_starpu_mpi_irecv_common(starpu_data_handle_t data_handle, int source, int data_tag, MPI_Comm comm, unsigned detached, unsigned sync, void (*callback)(void *), void *arg, int sequential_consistency, int is_internal_req, starpu_ssize_t count);
+static starpu_pthread_cond_t mpi_wait_for_all_running_cond;
+static int mpi_wait_for_all_running = 0;
+static starpu_pthread_mutex_t mpi_wait_for_all_running_mutex;
 
-/* Count requests posted by the application and not yet submitted to MPI, i.e pushed into the new_requests list */
-
-static volatile int pending_request = 0;
+/* Count running requests: this counter is incremented just before StarPU
+ * submits a MPI request, and decremented when a MPI request finishes. */
+static volatile int nb_pending_requests = 0;
 
 #define REQ_FINALIZED 0x1
 
@@ -65,6 +75,9 @@ static callback_lfstack_t callback_stack;
 
 static starpu_sem_t callback_sem;
 
+static int nmad_mcast_started = 0;
+
+
 /********************************************************/
 /*                                                      */
 /*  Send/Receive functionalities                        */
@@ -73,7 +86,8 @@ static starpu_sem_t callback_sem;
 
 void _starpu_mpi_req_willpost(struct _starpu_mpi_req *req STARPU_ATTRIBUTE_UNUSED)
 {
-	STARPU_ATOMIC_ADD( &pending_request, 1);
+	int new_nb = STARPU_ATOMIC_ADD(&nb_pending_requests, 1);
+	(void)new_nb;
 }
 
 /********************************************************/
@@ -82,13 +96,15 @@ void _starpu_mpi_req_willpost(struct _starpu_mpi_req *req STARPU_ATTRIBUTE_UNUSE
 /*                                                      */
 /********************************************************/
 
-static void _starpu_mpi_isend_data_func(struct _starpu_mpi_req *req)
+static void _starpu_mpi_isend_known_datatype(struct _starpu_mpi_req *req)
 {
 	_STARPU_MPI_LOG_IN();
 
+	STARPU_ASSERT_MSG(req->registered_datatype == 1, "Datatype is not registered, it cannot be sent through this way !");
+
 	_STARPU_MPI_DEBUG(30, "post NM isend request %p type %s tag %ld src %d data %p datasize %ld ptr %p datatype '%s' count %d registered_datatype %d sync %d\n", req, _starpu_mpi_request_type(req->request_type), req->node_tag.data_tag, req->node_tag.node.rank, req->data_handle, starpu_data_get_size(req->data_handle), req->ptr, req->datatype_name, (int)req->count, req->registered_datatype, req->sync);
 
-	_starpu_mpi_comm_amounts_inc(req->node_tag.node.comm, req->node_tag.node.rank, req->datatype, req->count);
+	_starpu_mpi_comm_amounts_inc(req->node_tag.node.comm, req->node, req->node_tag.node.rank, req->datatype, req->count);
 
 	_STARPU_MPI_TRACE_ISEND_SUBMIT_BEGIN(req->node_tag.node.rank, req->node_tag.data_tag, 0);
 
@@ -98,6 +114,9 @@ static void _starpu_mpi_isend_data_func(struct _starpu_mpi_req *req)
 	nm_sr_send_pack_data(req->backend->session, &(req->backend->data_request), &data);
 	nm_sr_send_set_priority(req->backend->session, &req->backend->data_request, req->prio);
 
+	// this trace event is the start of the communication link:
+	_STARPU_MPI_TRACE_ISEND_SUBMIT_END(_STARPU_MPI_FUT_POINT_TO_POINT_SEND, req, req->prio);
+
 	if (req->sync == 0)
 	{
 		req->ret = nm_sr_send_isend(req->backend->session, &(req->backend->data_request), req->backend->gate, req->node_tag.data_tag);
@@ -109,62 +128,30 @@ static void _starpu_mpi_isend_data_func(struct _starpu_mpi_req *req)
 		STARPU_ASSERT_MSG(req->ret == NM_ESUCCESS, "MPI_Issend returning %d", req->ret);
 	}
 
-	_STARPU_MPI_TRACE_ISEND_SUBMIT_END(req->node_tag.node.rank, req->node_tag.data_tag, starpu_data_get_size(req->data_handle), req->pre_sync_jobid);
-
 	_starpu_mpi_handle_pending_request(req);
 
 	_STARPU_MPI_LOG_OUT();
 }
 
-void _starpu_mpi_isend_size_func(struct _starpu_mpi_req *req)
+void _starpu_mpi_isend_func(struct _starpu_mpi_req *req)
 {
+	_STARPU_MPI_LOG_IN();
+
 	_starpu_mpi_datatype_allocate(req->data_handle, req);
 
 	if (req->registered_datatype == 1)
 	{
-		req->backend->waited = 1;
 		req->count = 1;
-		req->ptr = starpu_data_handle_to_pointer(req->data_handle, STARPU_MAIN_RAM);
+		req->ptr = starpu_data_handle_to_pointer(req->data_handle, req->node);
+
+		_starpu_mpi_isend_known_datatype(req);
 	}
 	else
 	{
-		starpu_ssize_t psize = -1;
-		int ret;
-		req->backend->waited =2;
-
-		// Do not pack the data, just try to find out the size
-		starpu_data_pack(req->data_handle, NULL, &psize);
-
-		if (psize != -1)
-		{
-			// We already know the size of the data, let's send it to overlap with the packing of the data
-			_STARPU_MPI_DEBUG(20, "Sending size %ld (%ld %s) to node %d (first call to pack)\n", psize, sizeof(req->count), "MPI_BYTE", req->node_tag.node.rank);
-			req->count = psize;
-			//ret = nm_sr_isend(nm_mpi_communicator_get_session(p_req->p_comm),nm_mpi_communicator_get_gate(p_comm,req->srcdst), req->mpi_tag,&req->count, sizeof(req->count), &req->backend->size_req);
-			ret = nm_sr_isend(req->backend->session,req->backend->gate, req->node_tag.data_tag,&req->count, sizeof(req->count), &req->backend->size_req);
-
-			//	ret = MPI_Isend(&req->count, sizeof(req->count), MPI_BYTE, req->srcdst, req->mpi_tag, req->comm, &req->backend->size_req);
-			STARPU_ASSERT_MSG(ret == NM_ESUCCESS, "when sending size, nm_sr_isend returning %d", ret);
-		}
-
-		// Pack the data
-		starpu_data_pack(req->data_handle, &req->ptr, &req->count);
-		if (psize == -1)
-		{
-			// We know the size now, let's send it
-			_STARPU_MPI_DEBUG(1, "Sending size %ld (%ld %s) with tag %ld to node %d (second call to pack)\n", req->count, sizeof(req->count), "MPI_BYTE", req->node_tag.data_tag, req->node_tag.node.rank);
-			ret = nm_sr_isend(req->backend->session,req->backend->gate, req->node_tag.data_tag,&req->count, sizeof(req->count), &req->backend->size_req);
-			STARPU_ASSERT_MSG(ret == NM_ESUCCESS, "when sending size, nm_sr_isend returning %d", ret);
-		}
-		else
-		{
-			// We check the size returned with the 2 calls to pack is the same
-			STARPU_ASSERT_MSG(req->count == psize, "Calls to pack_data returned different sizes %ld != %ld", req->count, psize);
-		}
-
-		// We can send the data now
+		_starpu_mpi_isend_unknown_datatype(req);
 	}
-	_starpu_mpi_isend_data_func(req);
+
+	_STARPU_MPI_LOG_OUT();
 }
 
 /********************************************************/
@@ -173,15 +160,16 @@ void _starpu_mpi_isend_size_func(struct _starpu_mpi_req *req)
 /*                                                      */
 /********************************************************/
 
-static void _starpu_mpi_irecv_data_func(struct _starpu_mpi_req *req)
+static void _starpu_mpi_irecv_known_datatype(struct _starpu_mpi_req *req)
 {
 	_STARPU_MPI_LOG_IN();
 
+	STARPU_ASSERT_MSG(req->registered_datatype == 1, "Datatype is not registered, it cannot be received through this way !");
+
 	_STARPU_MPI_DEBUG(20, "post NM irecv request %p type %s tag %ld src %d data %p ptr %p datatype '%s' count %d registered_datatype %d \n", req, _starpu_mpi_request_type(req->request_type), req->node_tag.data_tag, req->node_tag.node.rank, req->data_handle, req->ptr, req->datatype_name, (int)req->count, req->registered_datatype);
 
 	_STARPU_MPI_TRACE_IRECV_SUBMIT_BEGIN(req->node_tag.node.rank, req->node_tag.data_tag);
 
-	//req->ret = MPI_Irecv(req->ptr, req->count, req->datatype, req->srcdst, req->mpi_tag, req->comm, &req->request);
 	struct nm_data_s data;
 	nm_mpi_nmad_data_get(&data, (void*)req->ptr, req->datatype, req->count);
 	nm_sr_recv_init(req->backend->session, &(req->backend->data_request));
@@ -195,24 +183,7 @@ static void _starpu_mpi_irecv_data_func(struct _starpu_mpi_req *req)
 	_STARPU_MPI_LOG_OUT();
 }
 
-struct _starpu_mpi_irecv_size_callback
-{
-	starpu_data_handle_t handle;
-	struct _starpu_mpi_req *req;
-};
-
-static void _starpu_mpi_irecv_size_callback(void *arg)
-{
-	struct _starpu_mpi_irecv_size_callback *callback = (struct _starpu_mpi_irecv_size_callback *)arg;
-
-	starpu_data_unregister(callback->handle);
-	callback->req->ptr = (void *)starpu_malloc_on_node_flags(STARPU_MAIN_RAM, callback->req->count, 0);
-	STARPU_ASSERT_MSG(callback->req->ptr, "cannot allocate message of size %ld", callback->req->count);
-	_starpu_mpi_irecv_data_func(callback->req);
-	free(callback);
-}
-
-void _starpu_mpi_irecv_size_func(struct _starpu_mpi_req *req)
+void _starpu_mpi_irecv_func(struct _starpu_mpi_req *req)
 {
 	_STARPU_MPI_LOG_IN();
 
@@ -220,18 +191,15 @@ void _starpu_mpi_irecv_size_func(struct _starpu_mpi_req *req)
 	if (req->registered_datatype == 1)
 	{
 		req->count = 1;
-		req->ptr = starpu_data_handle_to_pointer(req->data_handle, STARPU_MAIN_RAM);
-		_starpu_mpi_irecv_data_func(req);
+		req->ptr = starpu_data_handle_to_pointer(req->data_handle, req->node);
+		_starpu_mpi_irecv_known_datatype(req);
 	}
 	else
 	{
-		struct _starpu_mpi_irecv_size_callback *callback = malloc(sizeof(struct _starpu_mpi_irecv_size_callback));
-		callback->req = req;
-		starpu_variable_data_register(&callback->handle, 0, (uintptr_t)&(callback->req->count), sizeof(callback->req->count));
-		_STARPU_MPI_DEBUG(4, "Receiving size with tag %ld from node %d\n", req->node_tag.data_tag, req->node_tag.node.rank);
-		_starpu_mpi_irecv_common(callback->handle, req->node_tag.node.rank, req->node_tag.data_tag, req->node_tag.node.comm, 1, 0, _starpu_mpi_irecv_size_callback, callback,1,0,0);
+		_starpu_mpi_irecv_unknown_datatype(req);
 	}
 
+	_STARPU_MPI_LOG_OUT();
 }
 
 /********************************************************/
@@ -240,12 +208,12 @@ void _starpu_mpi_irecv_size_func(struct _starpu_mpi_req *req)
 /*                                                      */
 /********************************************************/
 
-#define _starpu_mpi_req_status(PUBLIC_REQ,STATUS) do {			\
+#define _starpu_mpi_req_status(PUBLIC_REQ,STATUS) do { \
 	STATUS->MPI_SOURCE=PUBLIC_REQ->node_tag.node.rank; /**< field name mandatory by spec */ \
 	STATUS->MPI_TAG=PUBLIC_REQ->node_tag.data_tag;    /**< field name mandatory by spec */ \
 	STATUS->MPI_ERROR=PUBLIC_REQ->ret;  /**< field name mandatory by spec */ \
 	STATUS->size=PUBLIC_REQ->count;       /**< size of data received */ \
-	STATUS->cancelled=0;  /**< whether request was cancelled */	\
+	STATUS->cancelled=0;  /**< whether request was cancelled */ \
 } while(0)
 
 int _starpu_mpi_wait(starpu_mpi_req *public_req, MPI_Status *status)
@@ -265,8 +233,9 @@ int _starpu_mpi_wait(starpu_mpi_req *public_req, MPI_Status *status)
 	if (status!=MPI_STATUS_IGNORE)
 		_starpu_mpi_req_status(req,status);
 
-	_starpu_mpi_request_destroy(req);
+	_starpu_mpi_request_try_end(req, 1);
 	*public_req = NULL;
+
 	_STARPU_MPI_LOG_OUT();
 	return MPI_SUCCESS;
 }
@@ -300,7 +269,7 @@ int _starpu_mpi_test(starpu_mpi_req *public_req, int *flag, MPI_Status *status)
 
 	if(*flag)
 	{
-		_starpu_mpi_request_destroy(req);
+		_starpu_mpi_request_try_end(req, 1);
 		*public_req = NULL;
 	}
 	_STARPU_MPI_LOG_OUT();
@@ -316,156 +285,276 @@ int _starpu_mpi_test(starpu_mpi_req *public_req, int *flag, MPI_Status *status)
 int _starpu_mpi_barrier(MPI_Comm comm)
 {
 	_STARPU_MPI_LOG_IN();
-	int ret;
-	//	STARPU_ASSERT_MSG(!barrier_running, "Concurrent starpu_mpi_barrier is not implemented, even on different communicators");
-	ret = MPI_Barrier(comm);
 
+	int ret = MPI_Barrier(comm);
 	STARPU_ASSERT_MSG(ret == MPI_SUCCESS, "MPI_Barrier returning %d", ret);
 
 	_STARPU_MPI_LOG_OUT();
 	return ret;
 }
 
+int _starpu_mpi_wait_for_all(MPI_Comm comm)
+{
+	(void) comm;
+	_STARPU_MPI_LOG_IN();
+
+	STARPU_PTHREAD_MUTEX_LOCK(&mpi_wait_for_all_running_mutex);
+	STARPU_MPI_ASSERT_MSG(!mpi_wait_for_all_running, "Concurrent starpu_mpi_wait_for_all is not implemented, even on different communicators");
+	mpi_wait_for_all_running = 1;
+	do
+	{
+		while (nb_pending_requests)
+			STARPU_PTHREAD_COND_WAIT(&mpi_wait_for_all_running_cond, &mpi_wait_for_all_running_mutex);
+		STARPU_PTHREAD_MUTEX_UNLOCK(&mpi_wait_for_all_running_mutex);
+
+		starpu_task_wait_for_all();
+
+		STARPU_PTHREAD_MUTEX_LOCK(&mpi_wait_for_all_running_mutex);
+	} while (nb_pending_requests);
+	mpi_wait_for_all_running = 0;
+	STARPU_PTHREAD_MUTEX_UNLOCK(&mpi_wait_for_all_running_mutex);
+
+	_STARPU_MPI_LOG_OUT();
+	return 0;
+}
+
 /********************************************************/
 /*                                                      */
 /*  Progression                                         */
 /*                                                      */
 /********************************************************/
 
-#ifdef STARPU_VERBOSE
-char *_starpu_mpi_request_type(enum _starpu_mpi_request_type request_type)
+/* Completely finalize a request: destroy it and decrement the number of pending requests */
+static inline void _starpu_mpi_request_end(struct _starpu_mpi_req* req, int post_callback_sem)
 {
-	switch (request_type)
+	/* Destroying a request and decrementing the number of pending requests
+	 * should be done together, so let's wrap these two things in a
+	 * function. This means instead of calling _starpu_mpi_request_destroy(),
+	 * you should call this function. */
+
+	/* If request went through _starpu_mpi_handle_received_data(), finalized has to be true: */
+	assert((req->backend->has_received_data && req->backend->finalized) || !req->backend->has_received_data);
+
+	_starpu_mpi_request_destroy(req);
+
+	int pending_remaining = STARPU_ATOMIC_ADD(&nb_pending_requests, -1);
+	assert(pending_remaining >= 0);
+	if (!pending_remaining)
 	{
-		case SEND_REQ: return "SEND_REQ";
-		case RECV_REQ: return "RECV_REQ";
-		case WAIT_REQ: return "WAIT_REQ";
-		case TEST_REQ: return "TEST_REQ";
-		case BARRIER_REQ: return "BARRIER_REQ";
-		default: return "unknown request type";
+		STARPU_PTHREAD_COND_BROADCAST(&mpi_wait_for_all_running_cond);
+		if (post_callback_sem && !running)
+		{
+			starpu_sem_post(&callback_sem);
+		}
 	}
 }
-#endif
 
-static void _starpu_mpi_handle_request_termination(struct _starpu_mpi_req *req,nm_sr_event_t event)
+/* Check if the caller has to completely finalize a request and try to do it */
+static inline void _starpu_mpi_request_try_end(struct _starpu_mpi_req* req, int post_callback_sem)
+{
+	_starpu_spin_lock(&req->backend->finalized_to_destroy_lock);
+	if (!req->backend->has_received_data || req->backend->finalized)
+	{
+		_starpu_spin_unlock(&req->backend->finalized_to_destroy_lock);
+		_starpu_mpi_request_end(req, post_callback_sem);
+	}
+	else
+	{
+		/* Request isn't finalized yet (NewMadeleine still needs it), since
+		 * this function should have destroyed the request, tell
+		 * _starpu_mpi_handle_request_termination() to destroy it when
+		 * NewMadeleine won't need it anymore. */
+		req->backend->to_destroy = 1;
+		_starpu_spin_unlock(&req->backend->finalized_to_destroy_lock);
+	}
+}
+
+/* Do required actions when a request is completed (but maybe not finalized!) */
+static inline void _starpu_mpi_handle_post_actions(struct _starpu_mpi_req* req)
+{
+	if (req->callback)
+	{
+		/* Callbacks are executed outside of this function, later by the
+		 * progression thread.
+		 * Indeed, this current function is executed by a NewMadeleine handler,
+		 * and possibly inside of a PIOman ltask. In such context, some locking
+		 * or system calls can be forbidden to avoid any deadlock, thus
+		 * callbacks are deported outside of this handler. */
+		struct callback_lfstack_cell_s* c = padico_malloc(sizeof(struct callback_lfstack_cell_s));
+		c->req = req;
+		callback_lfstack_push(&callback_stack, c);
+
+		/* The main thread can exit without waiting
+		* the end of the detached request. Callback thread
+		* must then be kept alive if they have a callback.*/
+		starpu_sem_post(&callback_sem);
+	}
+	else if(!req->detached)
+	{
+		/* tell anyone potentially waiting on the request that it is
+		 * terminated now (should be done after the callback)*/
+		req->completed = 1;
+		piom_cond_signal(&req->backend->req_cond, REQ_FINALIZED);
+	}
+}
+
+/* Function called when data arrived, but NewMadeleine still holds a reference
+ * on it (to make progress a broadcast for instance). Application can thus read
+ * the data, but not yet write it. */
+void _starpu_mpi_handle_received_data(struct _starpu_mpi_req* req)
+{
+	_STARPU_MPI_LOG_IN();
+
+	assert(req->request_type == RECV_REQ);
+	assert(!_starpu_mpi_recv_wait_finalize);
+	assert(!req->backend->has_received_data);
+	assert(!req->backend->finalized);
+
+	req->backend->has_received_data = 1;
+
+	if (req->registered_datatype == 0)
+	{
+		/* Without peek_data, we can't unpack data for StarPU's use and keep
+		 * the buffer alive for NewMadeleine, so calling
+		 * _starpu_mpi_handle_received_data() makes no sense. */
+		assert(starpu_data_get_interface_ops(req->data_handle)->peek_data);
+		starpu_data_peek_node(req->data_handle, req->node, req->ptr, req->count);
+	}
+
+	// Release write acquire on the handle: can unlock tasks waiting to read the handle:
+	starpu_data_release_to(req->data_handle, STARPU_R);
+
+	_starpu_mpi_handle_post_actions(req);
+
+	_STARPU_MPI_LOG_OUT();
+}
+
+/* Function called when nmad completely finished a request */
+void _starpu_mpi_handle_request_termination(struct _starpu_mpi_req* req)
 {
 	_STARPU_MPI_LOG_IN();
 
 	_STARPU_MPI_DEBUG(2, "complete MPI request %p type %s tag %ld src %d data %p ptr %p datatype '%s' count %d registered_datatype %d \n",
 			  req, _starpu_mpi_request_type(req->request_type), req->node_tag.data_tag, req->node_tag.node.rank, req->data_handle, req->ptr, req->datatype_name, (int)req->count, req->registered_datatype);
 
+	assert(!req->backend->finalized);
+
 	if (req->request_type == RECV_REQ || req->request_type == SEND_REQ)
 	{
 		if (req->registered_datatype == 0)
 		{
-			if(req->backend->waited == 1)
-			        nm_mpi_nmad_data_release(req->datatype);
-			if (req->request_type == SEND_REQ)
-			{
-				req->backend->waited--;
-				// We need to make sure the communication for sending the size
-				// has completed, as MPI can re-order messages, let's count
-				// recerived message.
-				// FIXME concurent access.
-				STARPU_ASSERT_MSG(event == NM_SR_EVENT_FINALIZED, "Callback with event %d", event);
-				if(req->backend->waited>0)
-					return;
-
-			}
 			if (req->request_type == RECV_REQ)
-				// req->ptr is freed by starpu_data_unpack
-				starpu_data_unpack(req->data_handle, req->ptr, req->count);
+			{
+				if (starpu_data_get_interface_ops(req->data_handle)->peek_data)
+				{
+					if (!req->backend->has_received_data)
+					{
+						starpu_data_peek_node(req->data_handle, req->node, req->ptr, req->count);
+					}
+					starpu_free_on_node_flags(req->node, (uintptr_t) req->ptr, req->count, 0);
+				}
+				else
+				{
+					// req->ptr is freed by starpu_data_unpack
+					starpu_data_unpack_node(req->data_handle, req->node, req->ptr, req->count);
+				}
+			}
 			else
-				starpu_free_on_node_flags(STARPU_MAIN_RAM, (uintptr_t) req->ptr, req->count, 0);
+				starpu_free_on_node_flags(req->node, (uintptr_t) req->ptr, req->count, 0);
 		}
-		else
+		else if (req->backend->posted) // with coop, only one request is really used to do the broadcast, so only posted request really allocates memory for the data:
 		{
-		        nm_mpi_nmad_data_release(req->datatype);
+			nm_mpi_nmad_data_release(req->datatype);
 			_starpu_mpi_datatype_free(req->data_handle, &req->datatype);
 		}
 	}
-	_STARPU_MPI_TRACE_TERMINATED(req, req->node_tag.node.rank, req->node_tag.data_tag);
+
+	// for recv requests, this event is the end of the communication link:
+	_STARPU_MPI_TRACE_TERMINATED(req);
+
 	_starpu_mpi_release_req_data(req);
 
-	/* Execute the specified callback, if any */
-	if (req->callback)
+	if (req->backend->has_received_data)
 	{
-		struct callback_lfstack_cell_s* c = padico_malloc(sizeof(struct callback_lfstack_cell_s));
-		c->req = req;
-		/* The main thread can exit without waiting
-		* the end of the detached request. Callback thread
-		* must then be kept alive if they have a callback.*/
+		assert(req->request_type == RECV_REQ);
 
-		callback_lfstack_push(&callback_stack, c);
-		starpu_sem_post(&callback_sem);
-	}
-	else
-	{
-		if(req->detached)
+		/* Callback, test or wait were unlocked by
+		 * _starpu_mpi_handle_received_data(), maybe they were already
+		 * executed and since the request wasn't finalized yet, they didn't
+		 * destroy the request, and we have to do it now: */
+		_starpu_spin_lock(&req->backend->finalized_to_destroy_lock);
+		req->backend->finalized = 1;
+		if (req->backend->to_destroy || req->detached)
 		{
-			_starpu_mpi_request_destroy(req);
-			// a detached request wont be wait/test (and freed inside).
+			_starpu_spin_unlock(&req->backend->finalized_to_destroy_lock);
+			_starpu_mpi_request_end(req, 1);
 		}
 		else
 		{
-			/* tell anyone potentially waiting on the request that it is
-			 * terminated now (should be done after the callback)*/
-			req->completed = 1;
-			piom_cond_signal(&req->backend->req_cond, REQ_FINALIZED);
+			_starpu_spin_unlock(&req->backend->finalized_to_destroy_lock);
 		}
-		int pending_remaining = STARPU_ATOMIC_ADD(&pending_request, -1);
-		if (!running && !pending_remaining)
-			starpu_sem_post(&callback_sem);
 	}
+	else if (!req->callback && req->detached)
+	{
+		/* This request has no callback and is detached: we have to end it now: */
+		_starpu_mpi_request_end(req, 1);
+	}
+	else
+	{
+		_starpu_mpi_handle_post_actions(req);
+	}
+
 	_STARPU_MPI_LOG_OUT();
 }
 
-void _starpu_mpi_handle_request_termination_callback(nm_sr_event_t event, const nm_sr_event_info_t*event_info, void*ref)
+void _starpu_mpi_handle_request_termination_callback(nm_sr_event_t event STARPU_ATTRIBUTE_UNUSED, const nm_sr_event_info_t* event_info STARPU_ATTRIBUTE_UNUSED, void* ref)
 {
-	_starpu_mpi_handle_request_termination(ref,event);
-}
+	assert(ref != NULL);
 
-static void _starpu_mpi_handle_pending_request(struct _starpu_mpi_req *req)
-{
-	if(req->request_type == SEND_REQ && req->backend->waited>1)
+	struct _starpu_mpi_req* req = (struct _starpu_mpi_req*) ref;
+	req->backend->posted = 1; // a network event was triggered for this request, so it was really posted
+
+	if (event & NM_SR_EVENT_FINALIZED)
 	{
-		nm_sr_request_set_ref(&(req->backend->size_req), req);
-		nm_sr_request_monitor(req->backend->session, &(req->backend->size_req), NM_SR_EVENT_FINALIZED,_starpu_mpi_handle_request_termination_callback);
+		_starpu_mpi_handle_request_termination(req);
 	}
-	/* the if must be before, because the first callback can directly free
-	* a detached request (the second callback free if req->backend->waited>1). */
-	nm_sr_request_set_ref(&(req->backend->data_request), req);
-
-	nm_sr_request_monitor(req->backend->session, &(req->backend->data_request), NM_SR_EVENT_FINALIZED,_starpu_mpi_handle_request_termination_callback);
-}
-
-void _starpu_mpi_coop_sends_build_tree(struct _starpu_mpi_coop_sends *coop_sends)
-{
-	/* TODO: turn them into redirects & forwards */
-}
-
-void _starpu_mpi_submit_coop_sends(struct _starpu_mpi_coop_sends *coop_sends, int submit_control, int submit_data)
-{
-	unsigned i, n = coop_sends->n;
-
-	/* Note: coop_sends might disappear very very soon after last request is submitted */
-	for (i = 0; i < n; i++)
+	else if (event & NM_SR_EVENT_RECV_COMPLETED && req->request_type == RECV_REQ && !_starpu_mpi_recv_wait_finalize && req->sequential_consistency)
 	{
-		if (coop_sends->reqs_array[i]->request_type == SEND_REQ && submit_data)
-		{
-			_STARPU_MPI_DEBUG(0, "cooperative sends %p sending to %d\n", coop_sends, coop_sends->reqs_array[i]->node_tag.node.rank);
-			_starpu_mpi_submit_ready_request(coop_sends->reqs_array[i]);
-		}
-		/* TODO: handle redirect requests */
+		/* About required sequential consistency:
+		 * "If it is 0, user can launch tasks writing in the handle, which will
+		 * mix data manipulated by nmad and data manipulated by tasks, this
+		 * could break some expected behaviours." (sthibault) */
+
+		/* Unknown datatype case is in starpu_mpi_nmad_unknown_datatype.c */
+		assert(req->registered_datatype == 1);
+
+		_starpu_mpi_handle_received_data(req);
 	}
 }
 
+void _starpu_mpi_handle_pending_request(struct _starpu_mpi_req *req)
+{
+	assert(req != NULL);
+	nm_sr_request_set_ref(&req->backend->data_request, req);
+	int ret = nm_sr_request_monitor(req->backend->session, &req->backend->data_request,
+									NM_SR_EVENT_FINALIZED | NM_SR_EVENT_RECV_COMPLETED,
+									_starpu_mpi_handle_request_termination_callback);
+	assert(ret == NM_ESUCCESS);
+}
+
 void _starpu_mpi_submit_ready_request(void *arg)
 {
 	_STARPU_MPI_LOG_IN();
 	struct _starpu_mpi_req *req = arg;
 	STARPU_ASSERT_MSG(req, "Invalid request");
 
+	if (req->reserved_size)
+	{
+		/* The core will have really allocated the reception buffer now, release our reservation */
+		starpu_memory_deallocate(req->node, req->reserved_size);
+		req->reserved_size = 0;
+	}
+
 	/* submit the request to MPI directly from submitter */
 	_STARPU_MPI_DEBUG(2, "Handling new request %p type %s tag %ld src %d data %p ptr %p datatype '%s' count %d registered_datatype %d \n",
 			  req, _starpu_mpi_request_type(req->request_type), req->node_tag.data_tag, req->node_tag.node.rank, req->data_handle, req->ptr, req->datatype_name, (int)req->count, req->registered_datatype);
@@ -481,7 +570,9 @@ static void *_starpu_mpi_progress_thread_func(void *arg)
 #ifndef STARPU_SIMGRID
 	if (!_starpu_mpi_nobind && starpu_bind_thread_on(_starpu_mpi_thread_cpuid, 0, "MPI") < 0)
 	{
-		_STARPU_DISP("No core was available for the MPI thread. You should use STARPU_RESERVE_NCPU to leave one core available for MPI, or specify one core less in STARPU_NCPU\n");
+		char hostname[65];
+		gethostname(hostname, sizeof(hostname));
+		_STARPU_DISP("[%s] No core was available for the MPI thread. You should use STARPU_RESERVE_NCPU to leave one core available for MPI, or specify one core less in STARPU_NCPU\n", hostname);
 	}
 #endif
 
@@ -511,16 +602,24 @@ static void *_starpu_mpi_progress_thread_func(void *arg)
 	_starpu_mpi_cache_init(argc_argv->comm);
 	_starpu_mpi_select_node_init();
 	_starpu_mpi_datatype_init();
+	_starpu_mpi_tags_init();
 
 #ifdef STARPU_USE_FXT
-	if (_starpu_fxt_wait_initialisation())
+	_starpu_mpi_fxt_init(argc_argv);
+#endif
+
+	if (_starpu_mpi_use_coop_sends)
 	{
-		/* We need to record our ID in the trace before the main thread makes any MPI call */
-		_STARPU_MPI_TRACE_START(argc_argv->rank, argc_argv->world_size);
-		starpu_profiling_set_id(argc_argv->rank);
-		_starpu_mpi_add_sync_point_in_fxt();
+		if (argc_argv->world_size > 2)
+		{
+			_starpu_mpi_nmad_coop_init();
+			nmad_mcast_started = 1; // to shutdown mcast
+		}
+		else
+		{
+			_starpu_mpi_use_coop_sends = 0;
+		}
 	}
-#endif //STARPU_USE_FXT
 
 	/* notify the main thread that the progression thread is ready */
 	STARPU_PTHREAD_MUTEX_LOCK(&progress_mutex);
@@ -533,49 +632,62 @@ static void *_starpu_mpi_progress_thread_func(void *arg)
 		struct callback_lfstack_cell_s* c = callback_lfstack_pop(&callback_stack);
 		int err=0;
 
-		if(running || pending_request>0)
+		if(running || nb_pending_requests>0)
 		{
 			/* shall we block ? */
 			err = starpu_sem_wait(&callback_sem);
-			//running pending_request can change while waiting
+			//running nb_pending_requests can change while waiting
 		}
 		if(c==NULL)
 		{
 			c = callback_lfstack_pop(&callback_stack);
 			if (c == NULL)
 			{
-				if(running && pending_request>0)
+				if(running && nb_pending_requests>0)
 				{
 					STARPU_ASSERT_MSG(c!=NULL, "Callback thread awakened without callback ready with error %d.",err);
 				}
 				else
 				{
-					if (pending_request==0)
+					if (nb_pending_requests==0)
 						break;
 				}
 				continue;
 			}
 		}
 
-
 		c->req->callback(c->req->callback_arg);
 		if (c->req->detached)
 		{
-			_starpu_mpi_request_destroy(c->req);
+			_starpu_mpi_request_try_end(c->req, 0);
 		}
 		else
 		{
 			c->req->completed=1;
 			piom_cond_signal(&(c->req->backend->req_cond), REQ_FINALIZED);
 		}
-		STARPU_ATOMIC_ADD( &pending_request, -1);
-		/* we signal that the request is completed.*/
 
 		free(c);
-
 	}
+
+
+	/** Now, shutting down MPI **/
+
+
 	STARPU_ASSERT_MSG(callback_lfstack_pop(&callback_stack)==NULL, "List of callback not empty.");
-	STARPU_ASSERT_MSG(pending_request==0, "Request still pending.");
+	STARPU_ASSERT_MSG(nb_pending_requests==0, "Request still pending.");
+
+	/* We cannot rely on _starpu_mpi_use_coop_sends to shutdown mcast:
+	 * coops can be disabled with starpu_mpi_coop_sends_set_use() after
+	 * initialiazation of mcast. */
+	if (nmad_mcast_started)
+	{
+		_starpu_mpi_nmad_coop_shutdown();
+	}
+
+#ifdef STARPU_USE_FXT
+	_starpu_mpi_fxt_shutdown();
+#endif
 
 	if (argc_argv->initialize_mpi)
 	{
@@ -598,44 +710,13 @@ static void *_starpu_mpi_progress_thread_func(void *arg)
 // static int hookid = - 1;
 // #endif /* STARPU_MPI_ACTIVITY */
 
-static void _starpu_mpi_add_sync_point_in_fxt(void)
-{
-#ifdef STARPU_USE_FXT
-	int rank;
-	int worldsize;
-	int ret;
-
-	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
-	starpu_mpi_comm_size(MPI_COMM_WORLD, &worldsize);
-
-	ret = MPI_Barrier(MPI_COMM_WORLD);
-	STARPU_MPI_ASSERT_MSG(ret == MPI_SUCCESS, "MPI_Barrier returning %s", _starpu_mpi_get_mpi_error_code(ret));
-
-	/* We generate a "unique" key so that we can make sure that different
-	 * FxT traces come from the same MPI run. */
-	int random_number;
-
-	/* XXX perhaps we don't want to generate a new seed if the application
-	 * specified some reproductible behaviour ? */
-	if (rank == 0)
-	{
-		srand(time(NULL));
-		random_number = rand();
-	}
-
-	ret = MPI_Bcast(&random_number, 1, MPI_INT, 0, MPI_COMM_WORLD);
-	STARPU_MPI_ASSERT_MSG(ret == MPI_SUCCESS, "MPI_Bcast returning %s", _starpu_mpi_get_mpi_error_code(ret));
-
-	_STARPU_MPI_TRACE_BARRIER(rank, worldsize, random_number);
-
-	_STARPU_MPI_DEBUG(3, "unique key %x\n", random_number);
-#endif
-}
-
 int _starpu_mpi_progress_init(struct _starpu_mpi_argc_argv *argc_argv)
 {
-        STARPU_PTHREAD_MUTEX_INIT(&progress_mutex, NULL);
-        STARPU_PTHREAD_COND_INIT(&progress_cond, NULL);
+	STARPU_PTHREAD_MUTEX_INIT(&progress_mutex, NULL);
+	STARPU_PTHREAD_COND_INIT(&progress_cond, NULL);
+
+	STARPU_PTHREAD_MUTEX_INIT(&mpi_wait_for_all_running_mutex, NULL);
+	STARPU_PTHREAD_COND_INIT(&mpi_wait_for_all_running_cond, NULL);
 
 	starpu_sem_init(&callback_sem, 0, 0);
 	running = 0;
@@ -646,13 +727,13 @@ int _starpu_mpi_progress_init(struct _starpu_mpi_argc_argv *argc_argv)
 	 * required for piom_ltask_set_bound_thread_indexes() */
 	_starpu_mpi_do_initialize(argc_argv);
 
-	callback_lfstack_init(&callback_stack);
-
 	if (!_starpu_mpi_nobind && _starpu_mpi_thread_cpuid < 0)
 	{
 		_starpu_mpi_thread_cpuid = starpu_get_next_bindid(STARPU_THREAD_ACTIVE, NULL, 0);
 	}
 
+	callback_lfstack_init(&callback_stack);
+
 	/* Tell pioman to use a bound thread for communication progression:
 	 * share the same core as StarPU's MPI thread, the MPI thread has very low activity with NMAD backend */
 #ifdef HAVE_PIOM_LTASK_SET_BOUND_THREAD_OS_INDEXES
@@ -701,32 +782,37 @@ int _starpu_mpi_progress_init(struct _starpu_mpi_argc_argv *argc_argv)
 
 	if(polling_point_prog)
 	{
-		starpu_progression_hook_register((unsigned (*)(void *))&piom_ltask_schedule, (void *)&polling_point_prog);
+		starpu_progression_hook_register((void *)&piom_ltask_schedule, (void *)&polling_point_prog);
 	}
 
 	if(polling_point_idle)
 	{
-		starpu_idle_hook_register((unsigned (*)(void *))&piom_ltask_schedule, (void *)&polling_point_idle);
+		starpu_idle_hook_register((void *)&piom_ltask_schedule, (void *)&polling_point_idle);
 	}
 
 	/* Launch thread used for nmad callbacks */
 	STARPU_PTHREAD_CREATE(&progress_thread, NULL, _starpu_mpi_progress_thread_func, argc_argv);
 
-        STARPU_PTHREAD_MUTEX_LOCK(&progress_mutex);
-        while (!running)
-                STARPU_PTHREAD_COND_WAIT(&progress_cond, &progress_mutex);
-        STARPU_PTHREAD_MUTEX_UNLOCK(&progress_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&progress_mutex);
+	while (!running)
+		STARPU_PTHREAD_COND_WAIT(&progress_cond, &progress_mutex);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&progress_mutex);
 
-        return 0;
+	return 0;
 }
 
 void _starpu_mpi_progress_shutdown(void **value)
 {
+	if (!running)
+	{
+		_STARPU_ERROR("The progress thread was not launched. Was StarPU successfully initialized?\n");
+	}
+
 	/* kill the progression thread */
-        STARPU_PTHREAD_MUTEX_LOCK(&progress_mutex);
-        running = 0;
-        STARPU_PTHREAD_COND_BROADCAST(&progress_cond);
-        STARPU_PTHREAD_MUTEX_UNLOCK(&progress_mutex);
+	STARPU_PTHREAD_MUTEX_LOCK(&progress_mutex);
+	running = 0;
+	STARPU_PTHREAD_COND_BROADCAST(&progress_cond);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&progress_mutex);
 
 	starpu_sem_post(&callback_sem);
 
@@ -734,8 +820,11 @@ void _starpu_mpi_progress_shutdown(void **value)
 
 	callback_lfstack_destroy(&callback_stack);
 
-        STARPU_PTHREAD_MUTEX_DESTROY(&progress_mutex);
-        STARPU_PTHREAD_COND_DESTROY(&progress_cond);
+	STARPU_PTHREAD_MUTEX_DESTROY(&progress_mutex);
+	STARPU_PTHREAD_COND_DESTROY(&progress_cond);
+
+	STARPU_PTHREAD_MUTEX_DESTROY(&mpi_wait_for_all_running_mutex);
+	STARPU_PTHREAD_COND_DESTROY(&mpi_wait_for_all_running_cond);
 }
 
 static int64_t _starpu_mpi_tag_max = INT64_MAX;
