@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,9 @@
 #include <starpu_mpi_task_insert.h>
 #include <starpu_mpi_select_node.h>
 #include <util/starpu_task_insert_utils.h>
+#include <datawizard/coherency.h>
+#include <core/task.h>
+#include <core/workers.h>
 
 #ifdef HAVE_MPI_COMM_F2C
 static
@@ -50,13 +53,17 @@ int _fstarpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_no
 		if (arg_type==STARPU_EXECUTE_ON_NODE)
 		{
 			arg_i++;
-			*xrank = *(int *)arglist[arg_i];
-			if (node_selected == 0)
+			int rank = *(int *)arglist[arg_i];
+			if (rank != -1)
 			{
-				_STARPU_MPI_DEBUG(100, "Executing on node %d\n", *xrank);
-				*do_execute = 1;
-				node_selected = 1;
-				inconsistent_execute = 0;
+				*xrank = rank;
+				if (node_selected == 0)
+				{
+					_STARPU_MPI_DEBUG(100, "Executing on node %d\n", *xrank);
+					*do_execute = 1;
+					node_selected = 1;
+					inconsistent_execute = 0;
+				}
 			}
 		}
 		else if (arg_type==STARPU_EXECUTE_ON_DATA)
@@ -74,7 +81,7 @@ int _fstarpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_no
 				inconsistent_execute = 0;
 			}
 		}
-		else if (arg_type_nocommute & STARPU_R || arg_type_nocommute & STARPU_W || arg_type_nocommute & STARPU_RW || arg_type & STARPU_SCRATCH || arg_type & STARPU_REDUX)
+		else if (arg_type_nocommute & STARPU_R || arg_type_nocommute & STARPU_W || arg_type_nocommute & STARPU_RW || arg_type & STARPU_SCRATCH || arg_type & STARPU_REDUX || arg_type & STARPU_MPI_REDUX)
 		{
 			arg_i++;
 			starpu_data_handle_t data = arglist[arg_i];
@@ -225,6 +232,16 @@ int _fstarpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_no
 			arg_i++;
 			/* void* */
 		}
+		else if (arg_type==STARPU_EPILOGUE_CALLBACK)
+		{
+			arg_i++;
+			/* _starpu_callback_func_t */
+		}
+		else if (arg_type==STARPU_EPILOGUE_CALLBACK_ARG)
+		{
+			arg_i++;
+			/* void* */
+		}
 		else if (arg_type==STARPU_PRIORITY)
 		{
 			arg_i++;
@@ -256,32 +273,32 @@ int _fstarpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_no
 			/* unsigned* */
 		}
 		else if (arg_type==STARPU_PROLOGUE_CALLBACK)
-                {
+		{
 			arg_i++;
 			/* _starpu_callback_func_t */
 		}
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG)
-                {
-			arg_i++;
-			/* void* */
-                }
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG_NFREE)
-                {
-			arg_i++;
-			/* void* */
-                }
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP)
-                {
-			arg_i++;
-			/* _starpu_callback_func_t */
-                }
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG)
-                {
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG)
+		{
 			arg_i++;
 			/* void* */
 		}
-                else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG_NFREE)
-                {
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_ARG_NFREE)
+		{
+			arg_i++;
+			/* void* */
+		}
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP)
+		{
+			arg_i++;
+			/* _starpu_callback_func_t */
+		}
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG)
+		{
+			arg_i++;
+			/* void* */
+		}
+		else if (arg_type==STARPU_PROLOGUE_CALLBACK_POP_ARG_NFREE)
+		{
 			arg_i++;
 			/* void* */
 		}
@@ -330,11 +347,35 @@ int _fstarpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_no
 			arg_i++;
 			/* int* */
 		}
+		else if (arg_type==STARPU_TRANSACTION)
+		{
+			arg_i++;
+			/* struct starpu_transaction * */
+		}
 		else if (arg_type==STARPU_HANDLES_SEQUENTIAL_CONSISTENCY)
 		{
 			arg_i++;
 			/* char* */
 		}
+#ifdef STARPU_BUBBLE
+		else if (arg_type==STARPU_BUBBLE_FUNC)
+		{
+			STARPU_ASSERT_MSG(0, "Bubbles + MPI not supported yet\n");
+			arg_i++;
+		}
+		else if (arg_type==STARPU_BUBBLE_FUNC_ARG)
+		{
+			arg_i++;
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC)
+		{
+			arg_i++;
+		}
+		else if (arg_type==STARPU_BUBBLE_GEN_DAG_FUNC_ARG)
+		{
+			arg_i++;
+		}
+#endif
 		else if (arg_type==STARPU_TASK_END_DEP)
 		{
 			arg_i++;
@@ -367,6 +408,16 @@ int _fstarpu_mpi_task_decode_v(struct starpu_codelet *codelet, int me, int nb_no
 			arg_i++;
 			/* void * */
 		}
+		else if (arg_type==STARPU_TASK_FILE)
+		{
+			arg_i++;
+			/* char* */
+		}
+		else if (arg_type==STARPU_TASK_LINE)
+		{
+			arg_i++;
+			/* int */
+		}
 		else
 		{
 			STARPU_ABORT_MSG("Unrecognized argument %d, did you perhaps forget to end arguments with 0?\n", arg_type);
@@ -421,6 +472,14 @@ int _fstarpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, str
 	/* Send and receive data as requested */
 	for(i=0 ; i<nb_data ; i++)
 	{
+                if (descrs[i].handle && descrs[i].handle->mpi_data)
+		{
+			char *redux_map = starpu_mpi_data_get_redux_map(descrs[i].handle);
+			if (redux_map != NULL && descrs[i].mode & STARPU_R && descrs[i].mode & ~ STARPU_REDUX && descrs[i].mode & ~ STARPU_MPI_REDUX)
+			{
+				_starpu_mpi_redux_wrapup_data(descrs[i].handle);
+			}
+		}
 		_starpu_mpi_exchange_data_before_execution(descrs[i].handle, descrs[i].mode, me, xrank, do_execute, prio, comm);
 	}
 
@@ -435,13 +494,8 @@ int _fstarpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, str
 		*descrs_p = descrs;
 	else
 		free(descrs);
-	_STARPU_TRACE_TASK_MPI_PRE_END();
 
-	if (do_execute == 0)
-	{
-		return 1;
-	}
-	else
+	if (do_execute == 1)
 	{
 		_STARPU_MPI_DEBUG(100, "Execution of the codelet %p (%s)\n", codelet, codelet?codelet->name:NULL);
 
@@ -452,8 +506,32 @@ int _fstarpu_mpi_task_build_v(MPI_Comm comm, struct starpu_codelet *codelet, str
 		(*task)->prologue_callback_pop_arg_free = 1;
 
 		_fstarpu_task_insert_create(codelet, *task, arglist);
-		return 0;
+
+		if ((*task)->cl)
+		{
+			/* we suppose the current context is not going to change between now and the execution of the task */
+			(*task)->sched_ctx = _starpu_sched_ctx_get_current_context();
+			/* Check the type of worker(s) required by the task exist */
+			if (STARPU_UNLIKELY(!_starpu_worker_exists(*task)))
+			{
+				_STARPU_MPI_DEBUG(0, "There is no worker to execute the codelet %p (%s)\n", codelet, codelet?codelet->name:NULL);
+				return -ENODEV;
+			}
+
+			/* In case we require that a task should be explicitely
+			 * executed on a specific worker, we make sure that the worker
+			 * is able to execute this task.  */
+			if (STARPU_UNLIKELY((*task)->execute_on_a_specific_worker && !starpu_combined_worker_can_execute_task((*task)->workerid, *task, 0)))
+			{
+				_STARPU_MPI_DEBUG(0, "The specified worker %d cannot execute the codelet %p (%s)\n", (*task)->workerid, codelet, codelet?codelet->name:NULL);
+				return -ENODEV;
+			}
+		}
 	}
+
+	_STARPU_TRACE_TASK_MPI_PRE_END();
+
+	return do_execute;
 }
 
 static
@@ -471,7 +549,7 @@ int _fstarpu_mpi_task_insert_v(MPI_Comm comm, struct starpu_codelet *codelet, vo
 	if (ret < 0)
 		return ret;
 
-	if (ret == 0)
+	if (ret == 1)
 	{
 		do_execute = 1;
 		ret = starpu_task_submit(task);
@@ -486,9 +564,18 @@ int _fstarpu_mpi_task_insert_v(MPI_Comm comm, struct starpu_codelet *codelet, vo
 
 			task->destroy = 0;
 			starpu_task_destroy(task);
+			free(descrs);
+			return -ENODEV;
 		}
 	}
-	return _starpu_mpi_task_postbuild_v(comm, xrank, do_execute, descrs, nb_data, prio);
+
+	int val = _starpu_mpi_task_postbuild_v(comm, xrank, do_execute, descrs, nb_data, prio);
+	free(descrs);
+
+	if (ret == 1)
+		_starpu_mpi_pre_submit_hook_call(task);
+
+	return val;
 }
 
 void fstarpu_mpi_task_insert(void **arglist)
@@ -519,8 +606,7 @@ struct starpu_task *fstarpu_mpi_task_build(void **arglist)
 	int ret;
 
 	ret = _fstarpu_mpi_task_build_v(MPI_Comm_f2c(comm), codelet, &task, NULL, NULL, NULL, NULL, arglist+2);
-	STARPU_ASSERT(ret >= 0);
-	return (ret > 0) ? NULL : task;
+	return (ret == 1 || ret == -ENODEV) ? task : NULL;
 }
 
 void fstarpu_mpi_task_post_build(void **arglist)
@@ -545,6 +631,7 @@ void fstarpu_mpi_task_post_build(void **arglist)
 	STARPU_ASSERT(ret >= 0);
 
 	ret = _starpu_mpi_task_postbuild_v(MPI_Comm_f2c(comm), xrank, do_execute, descrs, nb_data, prio);
+	free(descrs);
 	STARPU_ASSERT(ret >= 0);
 }
 
