@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,10 +21,17 @@ int main(int argc, char **argv)
 {
 	int ret, rank, size;
 	int mpi_init;
+	struct starpu_conf conf;
 
 	MPI_INIT_THREAD(&argc, &argv, MPI_THREAD_SERIALIZED, &mpi_init);
 
-	ret = starpu_mpi_init_conf(&argc, &argv, mpi_init, MPI_COMM_WORLD, NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(&argc, &argv, mpi_init, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
@@ -52,14 +59,17 @@ int main(int argc, char **argv)
 			for(i=0 ; i<2 ; i++)
 				starpu_variable_data_register(&handle[i], STARPU_MAIN_RAM, (uintptr_t)&var[i], sizeof(var[i]));
 
-			starpu_mpi_recv(handle[0], n, 42, MPI_COMM_WORLD, &status[0]);
+			ret = starpu_mpi_recv(handle[0], n, 42, MPI_COMM_WORLD, &status[0]);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_recv");
 			starpu_data_acquire(handle[0], STARPU_R);
 			STARPU_ASSERT_MSG(var[0] == n, "Received incorrect value <%d> from node <%d>\n", var[0], n);
 			FPRINTF_MPI(stderr, "received <%d> from node %d\n", var[0], n);
 			starpu_data_release(handle[0]);
 
-			starpu_mpi_recv(handle[0], n, 42, MPI_COMM_WORLD, &status[1]);
-			starpu_mpi_recv(handle[1], n, 44, MPI_COMM_WORLD, &status[2]);
+			ret = starpu_mpi_recv(handle[0], n, 42, MPI_COMM_WORLD, &status[1]);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_recv");
+			ret = starpu_mpi_recv(handle[1], n, 44, MPI_COMM_WORLD, &status[2]);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_recv");
 			for(i=0 ; i<2 ; i++)
 				starpu_data_acquire(handle[i], STARPU_R);
 			STARPU_ASSERT_MSG(var[0] == n*2, "Received incorrect value <%d> from node <%d>\n", var[0], n);
@@ -82,9 +92,12 @@ int main(int argc, char **argv)
 		var[2] = var[0] * 4;
 		for(i=0 ; i<3 ; i++)
 			starpu_variable_data_register(&handle[i], STARPU_MAIN_RAM, (uintptr_t)&var[i], sizeof(var[i]));
-		starpu_mpi_send(handle[0], 0, 42, MPI_COMM_WORLD);
-		starpu_mpi_send(handle[1], 0, 42, MPI_COMM_WORLD);
-		starpu_mpi_send(handle[2], 0, 44, MPI_COMM_WORLD);
+		ret = starpu_mpi_send(handle[0], 0, 42, MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_send");
+		ret = starpu_mpi_send(handle[1], 0, 42, MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_send");
+		ret = starpu_mpi_send(handle[2], 0, 44, MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_send");
 		for(i=0 ; i<3 ; i++)
 			starpu_data_unregister(handle[i]);
 	}
