@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -34,46 +34,65 @@ int exchange(int rank, starpu_data_handle_t *handles, check_func func)
 
 	if (rank%2)
 	{
-		starpu_mpi_issend(handles[0], &req[0], other_rank, 0, MPI_COMM_WORLD);
-		starpu_mpi_isend(handles[NB-1], &req[NB-1], other_rank, NB-1, MPI_COMM_WORLD);
-		starpu_mpi_issend(handles[NB-2], &req[NB-2], other_rank, NB-2, MPI_COMM_WORLD);
+		ret = starpu_mpi_issend(handles[0], &req[0], other_rank, 0, MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_issend");
+		ret = starpu_mpi_issend(handles[NB-2], &req[NB-2], other_rank, NB-2, MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_issend");
+		ret = starpu_mpi_isend(handles[NB-1], &req[NB-1], other_rank, NB-1, MPI_COMM_WORLD);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_isend");
 
 		for(i=1 ; i<NB-2 ; i++)
 		{
 			if (i%2)
 			{
 				FPRINTF_MPI(stderr, "iSsending value %d\n", i);
-				starpu_mpi_issend(handles[i], &req[i], other_rank, i, MPI_COMM_WORLD);
+				ret = starpu_mpi_issend(handles[i], &req[i], other_rank, i, MPI_COMM_WORLD);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_issend");
 			}
 			else
 			{
 				FPRINTF_MPI(stderr, "isending value %d\n", i);
-				starpu_mpi_isend(handles[i], &req[i], other_rank, i, MPI_COMM_WORLD);
+				ret = starpu_mpi_isend(handles[i], &req[i], other_rank, i, MPI_COMM_WORLD);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_isend");
 			}
 		}
 		for(i=0 ; i<NB ; i++)
 		{
-			starpu_mpi_wait(&req[i], MPI_STATUS_IGNORE);
+			ret = starpu_mpi_wait(&req[i], MPI_STATUS_IGNORE);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_wait");
 		}
 	}
 	else
 	{
-		starpu_mpi_irecv(handles[0], &req[0], other_rank, 0, MPI_COMM_WORLD);
-		STARPU_ASSERT(req[0] != NULL);
-		starpu_mpi_irecv(handles[1], &req[1], other_rank, 1, MPI_COMM_WORLD);
-		STARPU_ASSERT(req[1] != NULL);
-
-		// We sleep to make sure that the data for the tag 8 and the tag 9 will be received before the recv are posted
-		starpu_sleep(2);
-		for(i=2 ; i<NB ; i++)
+		for(i=0 ; i<2 ; i++)
 		{
-			starpu_mpi_irecv(handles[i], &req[i], other_rank, i, MPI_COMM_WORLD);
+			ret = starpu_mpi_irecv(handles[i], &req[i], other_rank, i, MPI_COMM_WORLD);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_irecv");
 			STARPU_ASSERT(req[i] != NULL);
 		}
 
-		for(i=0 ; i<NB ; i++)
+		// We first wait for the first 2 requests to make sure
+		// the following data are internally received before
+		// their recv is posted
+		for(i=0 ; i<2 ; i++)
 		{
-			starpu_mpi_wait(&req[i], MPI_STATUS_IGNORE);
+			ret = starpu_mpi_wait(&req[i], MPI_STATUS_IGNORE);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_wait");
+			if (func)
+				func(handles[i], i, rank, &ret);
+		}
+
+		for(i=2 ; i<NB ; i++)
+		{
+			ret = starpu_mpi_irecv(handles[i], &req[i], other_rank, i, MPI_COMM_WORLD);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_irecv");
+			STARPU_ASSERT(req[i] != NULL);
+		}
+
+		for(i=2 ; i<NB ; i++)
+		{
+			ret = starpu_mpi_wait(&req[i], MPI_STATUS_IGNORE);
+			STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_wait");
 			if (func)
 				func(handles[i], i, rank, &ret);
 		}
@@ -87,14 +106,14 @@ void check_variable(starpu_data_handle_t handle, int i, int rank, int *error)
 
 	starpu_data_acquire_on_node(handle, starpu_worker_get_local_memory_node(), STARPU_R);
 	int *rvalue = (int *)starpu_data_get_local_ptr(handle);
-	if (*rvalue != i*other_rank)
+	if (*rvalue != (i+1)*(other_rank+1))
 	{
-		FPRINTF_MPI(stderr, "Incorrect received value: %d != %d\n", *rvalue, i*other_rank);
+		FPRINTF_MPI(stderr, "Incorrect received value: %d != %d\n", *rvalue, (i+1)*(other_rank+1));
 		*error = 1;
 	}
 	else
 	{
-		FPRINTF_MPI(stderr, "Correct received value: %d == %d\n", *rvalue, i*other_rank);
+		FPRINTF_MPI(stderr, "Correct received value: %d == %d\n", *rvalue, (i+1)*(other_rank+1));
 	}
 	starpu_data_release(handle);
 }
@@ -104,15 +123,22 @@ int exchange_variable(int rank)
 	int ret, i;
 	starpu_data_handle_t tab_handle[NB];
 	int value[NB];
+	struct starpu_conf conf;
 
-	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, NULL);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	FPRINTF_MPI(stderr, "Exchanging variable data\n");
 
 	for(i=0 ; i<NB ; i++)
 	{
-		value[i]=i*rank;
+		value[i]=(i+1)*(rank+1);
 		starpu_variable_data_register(&tab_handle[i], STARPU_MAIN_RAM, (uintptr_t)&value[i], sizeof(int));
 		starpu_mpi_data_register(tab_handle[i], i, rank);
 	}
@@ -129,13 +155,18 @@ int exchange_void(int rank)
 {
 	int ret, i;
 	starpu_data_handle_t tab_handle[NB];
+	struct starpu_conf conf;
 
 	// This test is not run with valgrind as valgrind falsely detects error when exchanging NULL pointers
 	STARPU_SKIP_IF_VALGRIND_RETURN_ZERO;
 
-	ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-	ret = starpu_mpi_init(NULL, NULL, 0);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
 
 	FPRINTF_MPI(stderr, "Exchanging void data\n");
@@ -163,12 +194,12 @@ void check_complex(starpu_data_handle_t handle, int i, int rank, int *error)
 
 	if ((*real != ((i*other_rank)+12)) || (*imaginary != ((i*other_rank)+45)))
 	{
-		FPRINTF_MPI(stderr, "Incorrect received value: %f != %d || %f != %d\n", *real, ((i*other_rank)+12), *imaginary, ((i*other_rank)+45));
+		FPRINTF_MPI(stderr, "Incorrect received value: %f != %f || %f != %f\n", *real, (double)((i*other_rank)+12), *imaginary, (double)((i*other_rank)+45));
 		*error = 1;
 	}
 	else
 	{
-		FPRINTF_MPI(stderr, "Correct received value: %f == %d || %f == %d\n", *real, ((i*other_rank)+12), *imaginary, ((i*other_rank)+45));
+		FPRINTF_MPI(stderr, "Correct received value: %f == %f && %f == %f\n", *real, (double)((i*other_rank)+12), *imaginary, (double)((i*other_rank)+45));
 	}
 }
 
@@ -178,10 +209,15 @@ int exchange_complex(int rank)
 	starpu_data_handle_t handle[NB];
 	double real[NB];
 	double imaginary[NB];
+	struct starpu_conf conf;
 
-	ret = starpu_init(NULL);
-	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-	ret = starpu_mpi_init(NULL, NULL, 0);
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, &conf);
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
 
 	FPRINTF_MPI(stderr, "Exchanging complex data\n");
@@ -208,8 +244,8 @@ int main(int argc, char **argv)
 	int rank, size;
 
 	MPI_INIT_THREAD_real(&argc, &argv, MPI_THREAD_SERIALIZED);
-	starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
-	starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
+	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+	MPI_Comm_size(MPI_COMM_WORLD, &size);
 
 	if (size%2 != 0)
 	{
