@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2015-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -39,9 +39,16 @@ int do_test(int rank, int sdetached, int rdetached)
 	int ret, i;
 	int val[2];
 	starpu_data_handle_t data[2];
+	struct starpu_conf conf;
 
-        ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, NULL);
-        STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
+
+	ret = starpu_mpi_init_conf(NULL, NULL, 0, MPI_COMM_WORLD, &conf);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	if (rank == 1)
 	{
@@ -63,9 +70,15 @@ int do_test(int rank, int sdetached, int rdetached)
 		for(i=1 ; i>=0 ; i--)
 		{
 			if (sdetached)
-				starpu_mpi_isend_detached(data[i], 0, starpu_data_get_tag(data[i]), MPI_COMM_WORLD, NULL, NULL);
+			{
+				ret = starpu_mpi_isend_detached(data[i], 0, starpu_data_get_tag(data[i]), MPI_COMM_WORLD, NULL, NULL);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_isend_detached");
+			}
 			else
-				starpu_mpi_send(data[i], 0, starpu_data_get_tag(data[i]), MPI_COMM_WORLD);
+			{
+				ret = starpu_mpi_send(data[i], 0, starpu_data_get_tag(data[i]), MPI_COMM_WORLD);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_send");
+			}
 		}
 	}
 	else if (rank == 0)
@@ -77,9 +90,15 @@ int do_test(int rank, int sdetached, int rdetached)
 		for(i=0 ; i<2 ; i++)
 		{
 			if (rdetached)
-				starpu_mpi_irecv_detached(data[i], 1, starpu_data_get_tag(data[i]), MPI_COMM_WORLD, callback, &received);
+			{
+				ret = starpu_mpi_irecv_detached(data[i], 1, starpu_data_get_tag(data[i]), MPI_COMM_WORLD, callback, &received);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_irecv_detached");
+			}
 			else
-				starpu_mpi_recv(data[i], 1, starpu_data_get_tag(data[i]), MPI_COMM_WORLD, MPI_STATUS_IGNORE);
+			{
+				ret = starpu_mpi_recv(data[i], 1, starpu_data_get_tag(data[i]), MPI_COMM_WORLD, MPI_STATUS_IGNORE);
+				STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_recv");
+			}
 		}
 
 		if (rdetached)
@@ -122,15 +141,15 @@ int main(int argc, char **argv)
 	int sdetached, rdetached;
 
 	MPI_INIT_THREAD_real(&argc, &argv, MPI_THREAD_SERIALIZED);
-        starpu_mpi_comm_rank(MPI_COMM_WORLD, &rank);
-        starpu_mpi_comm_size(MPI_COMM_WORLD, &size);
+	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+	MPI_Comm_size(MPI_COMM_WORLD, &size);
 
-        if (size < 2)
-        {
-		FPRINTF_MPI(stderr, "We need at least 2 processes.\n");
-                MPI_Finalize();
-                return rank == 0 ? STARPU_TEST_SKIPPED : 0;
-        }
+	if (size < 2)
+	{
+		FPRINTF(stderr, "We need at least 2 processes.\n");
+		MPI_Finalize();
+		return rank == 0 ? STARPU_TEST_SKIPPED : 0;
+	}
 
 	for(sdetached=0 ; sdetached<=1 ; sdetached++)
 	{
@@ -140,6 +159,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-        MPI_Finalize();
+	MPI_Finalize();
 	return rank == 0 ? ret : 0;
 }
