@@ -46,6 +46,9 @@
  *   * Initializes a list (initially empty)
  *   void		FOO_list_init(struct FOO_list*);
  *
+ *   * Initializes a list (initially empty), assuming that the content of FOO_list was already zeroed
+ *   void		FOO_list_init0(struct FOO_list*);
+ *
  *   * Suppresses a liste
  *   void		FOO_list_delete(struct FOO_list*);
  *
@@ -220,7 +223,9 @@
   /** @internal */LIST_INLINE struct ENAME *ENAME##_list_back(const struct ENAME##_list *l) \
     { return l->_tail; } \
   /** @internal */LIST_INLINE void ENAME##_list_init(struct ENAME##_list *l) \
-    { l->_head=NULL; l->_tail=l->_head; } \
+    { l->_head=NULL; l->_tail=NULL; } \
+  /** @internal */LIST_INLINE void ENAME##_list_init0(struct ENAME##_list *l STARPU_ATTRIBUTE_UNUSED) \
+    { } \
   /** @internal */LIST_INLINE struct ENAME##_list *ENAME##_list_new(void) \
     { struct ENAME##_list *l; _STARPU_MALLOC(l, sizeof(struct ENAME##_list)); \
       ENAME##_list_init(l); return l; } \
@@ -419,6 +424,26 @@ LIST_INLINE TYPE *ENAME##_multilist_end_##MEMBER(struct ENAME##_multilist_##MEMB
 LIST_INLINE TYPE *ENAME##_multilist_next_##MEMBER(TYPE *e) { \
 	return ENAME##_of_multilist_##MEMBER(e->MEMBER.next); \
 } \
+/* Return the first element of the list.  */ \
+LIST_INLINE TYPE *ENAME##_multilist_front_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  return ENAME##_of_multilist_##MEMBER(head->next); \
+} \
+/* Return the last element of the list.  */ \
+LIST_INLINE TYPE *ENAME##_multilist_back_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  return ENAME##_of_multilist_##MEMBER(head->prev); \
+} \
+\
+/* Return the first element of the list and erase it. */ \
+LIST_INLINE TYPE *ENAME##_multilist_pop_front_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  TYPE *e = ENAME##_multilist_front_##MEMBER(head); \
+      ENAME##_multilist_erase_##MEMBER(head, e); return e; \
+} \
+/* Return the last element of the list and erase it. */ \
+LIST_INLINE TYPE *ENAME##_multilist_pop_back_##MEMBER(struct ENAME##_multilist_##MEMBER *head) { \
+  TYPE *e = ENAME##_multilist_back_##MEMBER(head); \
+      ENAME##_multilist_erase_##MEMBER(head, e); return e; \
+} \
+\
 \
  /* Move a list from its head to another head. Passing newhead == NULL allows to detach the list from any head. */ \
 LIST_INLINE void ENAME##_multilist_move_##MEMBER(struct ENAME##_multilist_##MEMBER *head, struct ENAME##_multilist_##MEMBER *newhead) { \
