@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  * Copyright (C) 2013       Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
@@ -37,7 +37,6 @@
 #include <datawizard/copy_driver.h>
 #include <datawizard/data_request.h>
 #include <datawizard/memory_manager.h>
-#include <starpu_parameters.h>
 #include <common/uthash.h>
 
 #ifdef STARPU_HAVE_WINDOWS
@@ -55,7 +54,7 @@
 #define MAX_OPEN_FILES 64
 #define TEMP_HIERARCHY_DEPTH 2
 
-#if !defined(HAVE_COPY_FILE_RANGE) && defined( __NR_copy_file_range)
+#if !defined(HAVE_COPY_FILE_RANGE) && defined(__linux__) && defined(__NR_copy_file_range)
 static starpu_ssize_t copy_file_range(int fd_in, loff_t *off_in, int fd_out,
 				      loff_t *off_out, size_t len, unsigned int flags)
 {
@@ -70,8 +69,8 @@ static unsigned starpu_unistd_opened_files;
 LIST_TYPE(starpu_unistd_work_copy,
 	int fd_src;
 	int fd_dst;
-	loff_t off_src;
-	loff_t off_dst;
+	starpu_loff_t off_src;
+	starpu_loff_t off_dst;
 	struct starpu_unistd_global_obj * obj_src;
 	struct starpu_unistd_global_obj * obj_dst;
 	size_t len;
@@ -105,24 +104,24 @@ struct starpu_unistd_base
 #endif
 #if defined(HAVE_LIBAIO_H)
 	io_context_t ctx;
-        struct starpu_unistd_aiocb_link * hashtable;
-        starpu_pthread_mutex_t mutex;
+	struct starpu_unistd_aiocb_link * hashtable;
+	starpu_pthread_mutex_t mutex;
 #endif
 };
 
 #if defined(HAVE_LIBAIO_H)
 struct starpu_unistd_aiocb_link
 {
-        UT_hash_handle hh;
-        void * starpu_aiocb;
-        void * aiocb;
+	UT_hash_handle hh;
+	void * starpu_aiocb;
+	void * aiocb;
 };
 struct starpu_unistd_aiocb
 {
-        int finished;
+	int finished;
 	struct iocb iocb;
 	struct starpu_unistd_global_obj *obj;
-        struct starpu_unistd_base *base;
+	struct starpu_unistd_base *base;
 	size_t len;
 };
 #elif defined(HAVE_AIO_H)
@@ -285,10 +284,20 @@ int starpu_unistd_global_read(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, voi
 	struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
 	starpu_ssize_t nb;
 	int fd = tmp->descriptor;
+	starpu_ssize_t bytes_to_write = size;
 
 #ifdef HAVE_PREAD
 	if (fd >= 0)
-		nb = pread(fd, buf, size, offset);
+	{
+		while (bytes_to_write > 0)
+		{
+			nb = pread(fd, buf, bytes_to_write, offset);
+			STARPU_ASSERT_MSG(nb >= 0, "Starpu Disk unistd pread failed: size %lu got errno %d", (unsigned long) size, errno);
+			bytes_to_write -= nb;
+			buf = (char*) buf + nb;
+			offset += nb;
+		}
+	}
 	else
 #endif
 	{
@@ -300,7 +309,14 @@ int starpu_unistd_global_read(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, voi
 		int res = lseek(fd, offset, SEEK_SET);
 		STARPU_ASSERT_MSG(res >= 0, "Starpu Disk unistd lseek for read failed: offset %lu got errno %d", (unsigned long) offset, errno);
 
-		nb = read(fd, buf, size);
+		while (bytes_to_write > 0)
+		{
+			nb = read(fd, buf, bytes_to_write);
+			STARPU_ASSERT_MSG(nb >= 0, "Starpu Disk unistd read failed: offset %lu got errno %d", (unsigned long) offset, errno);
+			bytes_to_write -= nb;
+			buf = (char*) buf + nb;
+			offset += nb;
+		}
 
 		if (tmp->descriptor >= 0)
 			STARPU_PTHREAD_MUTEX_UNLOCK(&tmp->mutex);
@@ -309,86 +325,84 @@ int starpu_unistd_global_read(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, voi
 
 	}
 
-	STARPU_ASSERT_MSG(nb >= 0, "Starpu Disk unistd read failed: size %lu got errno %d", (unsigned long) size, errno);
-
-	return nb;
+	return 0;
 }
 
 #if defined(HAVE_LIBAIO_H)
 void *starpu_unistd_global_async_read(void *base, void *obj, void *buf, off_t offset, size_t size)
 {
-        struct starpu_unistd_base * fileBase = (struct starpu_unistd_base *) base;
-        struct starpu_unistd_global_obj *tmp = obj;
+	struct starpu_unistd_base * fileBase = (struct starpu_unistd_base *) base;
+	struct starpu_unistd_global_obj *tmp = obj;
 	struct starpu_unistd_wait * event;
 	_STARPU_CALLOC(event, 1,sizeof(*event));
 	event->type = STARPU_UNISTD_AIOCB;
-        struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-        struct iocb *iocb = &starpu_aiocb->iocb;
-        starpu_aiocb->obj = obj;
-        int fd = tmp->descriptor;
+	struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
+	struct iocb *iocb = &starpu_aiocb->iocb;
+	starpu_aiocb->obj = obj;
+	int fd = tmp->descriptor;
 	int err;
 
-        if (fd < 0)
-                fd = _starpu_unistd_reopen(obj);
+	if (fd < 0)
+		fd = _starpu_unistd_reopen(obj);
 
 	starpu_aiocb->len = size;
-        starpu_aiocb->finished = 0;
-        starpu_aiocb->base = fileBase;
+	starpu_aiocb->finished = 0;
+	starpu_aiocb->base = fileBase;
 	io_prep_pread(iocb, fd, buf, size, offset);
 	if ((err = io_submit(fileBase->ctx, 1, &iocb)) < 0)
 	{
 		_STARPU_DISP("Warning: io_submit returned %d (%s)\n", err, strerror(err));
-                if (tmp->descriptor < 0)
-                        _starpu_unistd_reclose(fd);
-                iocb = NULL;
-        }
+		if (tmp->descriptor < 0)
+			_starpu_unistd_reclose(fd);
+		iocb = NULL;
+	}
 
-        struct starpu_unistd_aiocb_link *l;
-        _STARPU_MALLOC(l, sizeof(*l));
-        l->aiocb = iocb;
-        l->starpu_aiocb = starpu_aiocb;
-        STARPU_PTHREAD_MUTEX_LOCK(&fileBase->mutex);
-        HASH_ADD_PTR(fileBase->hashtable, aiocb, l);
-        STARPU_PTHREAD_MUTEX_UNLOCK(&fileBase->mutex);
+	struct starpu_unistd_aiocb_link *l;
+	_STARPU_MALLOC(l, sizeof(*l));
+	l->aiocb = iocb;
+	l->starpu_aiocb = starpu_aiocb;
+	STARPU_PTHREAD_MUTEX_LOCK(&fileBase->mutex);
+	HASH_ADD_PTR(fileBase->hashtable, aiocb, l);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&fileBase->mutex);
 
-        return event;
+	return event;
 }
 #elif defined(HAVE_AIO_H)
 void *starpu_unistd_global_async_read(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, void *buf, off_t offset, size_t size)
 {
-        struct starpu_unistd_global_obj *tmp = obj;
+	struct starpu_unistd_global_obj *tmp = obj;
 	struct starpu_unistd_wait * event;
 	_STARPU_CALLOC(event, 1,sizeof(*event));
 	event->type = STARPU_UNISTD_AIOCB;
-        struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-        struct aiocb *aiocb = &starpu_aiocb->aiocb;
-        starpu_aiocb->obj = obj;
-        int fd = tmp->descriptor;
+	struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
+	struct aiocb *aiocb = &starpu_aiocb->aiocb;
+	starpu_aiocb->obj = obj;
+	int fd = tmp->descriptor;
 
-        if (fd < 0)
-                fd = _starpu_unistd_reopen(obj);
+	if (fd < 0)
+		fd = _starpu_unistd_reopen(obj);
 
-        aiocb->aio_fildes = fd;
-        aiocb->aio_offset = offset;
-        aiocb->aio_nbytes = size;
-        aiocb->aio_buf = buf;
-        aiocb->aio_reqprio = 0;
-        aiocb->aio_lio_opcode = LIO_NOP;
+	aiocb->aio_fildes = fd;
+	aiocb->aio_offset = offset;
+	aiocb->aio_nbytes = size;
+	aiocb->aio_buf = buf;
+	aiocb->aio_reqprio = 0;
+	aiocb->aio_lio_opcode = LIO_NOP;
 
-        if (aio_read(aiocb) < 0)
-        {
+	if (aio_read(aiocb) < 0)
+	{
 		_STARPU_DISP("Warning: aio_read returned %d (%s)\n", errno, strerror(errno));
-                if (tmp->descriptor < 0)
-                        _starpu_unistd_reclose(fd);
-        }
+		if (tmp->descriptor < 0)
+			_starpu_unistd_reclose(fd);
+	}
 
-        return event;
+	return event;
 }
 #endif
 
 int starpu_unistd_global_full_read(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, void **ptr, size_t *size, unsigned dst_node)
 {
-        struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
+	struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
 	int fd = tmp->descriptor;
 
 	if (fd < 0)
@@ -414,12 +428,22 @@ int starpu_unistd_global_full_read(void *base STARPU_ATTRIBUTE_UNUSED, void *obj
 int starpu_unistd_global_write(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, const void *buf, off_t offset, size_t size)
 {
 	struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
-	int res;
+	starpu_ssize_t res;
 	int fd = tmp->descriptor;
+	starpu_ssize_t bytes_to_write = size;
 
 #ifdef HAVE_PWRITE
 	if (fd >= 0)
-		res = pwrite(fd, buf, size, offset);
+	{
+		while (bytes_to_write > 0)
+		{
+			res = pwrite(fd, buf, bytes_to_write, offset);
+			STARPU_ASSERT_MSG(res >= 0, "Starpu Disk unistd pwrite failed: offset %lu got errno %d", (unsigned long) offset, errno);
+			bytes_to_write -= res;
+			buf = (char*) buf + res;
+			offset += res;
+		}
+	}
 	else
 #endif
 	{
@@ -431,7 +455,14 @@ int starpu_unistd_global_write(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, co
 		res = lseek(fd, offset, SEEK_SET);
 		STARPU_ASSERT_MSG(res >= 0, "Starpu Disk unistd lseek for write failed: offset %lu got errno %d", (unsigned long) offset, errno);
 
-		res = write(fd, buf, size);
+		while (bytes_to_write > 0)
+		{
+			res = write(fd, buf, bytes_to_write);
+			STARPU_ASSERT_MSG(res >= 0, "Starpu Disk unistd write failed: offset %lu got errno %d", (unsigned long) offset, errno);
+			bytes_to_write -= res;
+			buf = (char*) buf + res;
+			offset += res;
+		}
 
 		if (tmp->descriptor >= 0)
 			STARPU_PTHREAD_MUTEX_UNLOCK(&tmp->mutex);
@@ -439,90 +470,89 @@ int starpu_unistd_global_write(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, co
 			_starpu_unistd_reclose(fd);
 	}
 
-	STARPU_ASSERT_MSG(res >= 0, "Starpu Disk unistd write failed: size %lu got errno %d", (unsigned long) size, errno);
 	return 0;
 }
 
 #if defined(HAVE_LIBAIO_H)
 void *starpu_unistd_global_async_write(void *base, void *obj, void *buf, off_t offset, size_t size)
 {
-        struct starpu_unistd_base * fileBase = (struct starpu_unistd_base *) base;
-        struct starpu_unistd_global_obj *tmp = obj;
+	struct starpu_unistd_base * fileBase = (struct starpu_unistd_base *) base;
+	struct starpu_unistd_global_obj *tmp = obj;
 	struct starpu_unistd_wait * event;
 	_STARPU_CALLOC(event, 1,sizeof(*event));
 	event->type = STARPU_UNISTD_AIOCB;
-        struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-        struct iocb *iocb = &starpu_aiocb->iocb;
-        starpu_aiocb->obj = obj;
-        int fd = tmp->descriptor;
+	struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
+	struct iocb *iocb = &starpu_aiocb->iocb;
+	starpu_aiocb->obj = obj;
+	int fd = tmp->descriptor;
 	int err;
 
-        if (fd < 0)
-                fd = _starpu_unistd_reopen(obj);
+	if (fd < 0)
+		fd = _starpu_unistd_reopen(obj);
 
 	starpu_aiocb->len = size;
-        starpu_aiocb->finished = 0;
-        starpu_aiocb->base = fileBase;
+	starpu_aiocb->finished = 0;
+	starpu_aiocb->base = fileBase;
 	io_prep_pwrite(iocb, fd, buf, size, offset);
 	if ((err = io_submit(fileBase->ctx, 1, &iocb)) < 0)
-        {
+	{
 		_STARPU_DISP("Warning: io_submit returned %d (%s)\n", err, strerror(err));
-                if (tmp->descriptor < 0)
-                        _starpu_unistd_reclose(fd);
-                iocb = NULL;
-        }
+		if (tmp->descriptor < 0)
+			_starpu_unistd_reclose(fd);
+		iocb = NULL;
+	}
 
-        struct starpu_unistd_aiocb_link *l;
-        _STARPU_MALLOC(l, sizeof(*l));
-        l->aiocb = iocb;
-        l->starpu_aiocb = starpu_aiocb;
-        STARPU_PTHREAD_MUTEX_LOCK(&fileBase->mutex);
-        HASH_ADD_PTR(fileBase->hashtable, aiocb, l);
-        STARPU_PTHREAD_MUTEX_UNLOCK(&fileBase->mutex);
+	struct starpu_unistd_aiocb_link *l;
+	_STARPU_MALLOC(l, sizeof(*l));
+	l->aiocb = iocb;
+	l->starpu_aiocb = starpu_aiocb;
+	STARPU_PTHREAD_MUTEX_LOCK(&fileBase->mutex);
+	HASH_ADD_PTR(fileBase->hashtable, aiocb, l);
+	STARPU_PTHREAD_MUTEX_UNLOCK(&fileBase->mutex);
 
-        return event;
+	return event;
 }
 #elif defined(HAVE_AIO_H)
 void *starpu_unistd_global_async_write(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, void *buf, off_t offset, size_t size)
 {
-        struct starpu_unistd_global_obj *tmp = obj;
+	struct starpu_unistd_global_obj *tmp = obj;
 	struct starpu_unistd_wait * event;
 	_STARPU_CALLOC(event, 1,sizeof(*event));
 	event->type = STARPU_UNISTD_AIOCB;
-        struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-        struct aiocb *aiocb = &starpu_aiocb->aiocb;
-        starpu_aiocb->obj = obj;
-        int fd = tmp->descriptor;
+	struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
+	struct aiocb *aiocb = &starpu_aiocb->aiocb;
+	starpu_aiocb->obj = obj;
+	int fd = tmp->descriptor;
 
-        if (fd < 0)
-                fd = _starpu_unistd_reopen(obj);
+	if (fd < 0)
+		fd = _starpu_unistd_reopen(obj);
 
-        aiocb->aio_fildes = fd;
-        aiocb->aio_offset = offset;
-        aiocb->aio_nbytes = size;
-        aiocb->aio_buf = buf;
-        aiocb->aio_reqprio = 0;
-        aiocb->aio_lio_opcode = LIO_NOP;
+	aiocb->aio_fildes = fd;
+	aiocb->aio_offset = offset;
+	aiocb->aio_nbytes = size;
+	aiocb->aio_buf = buf;
+	aiocb->aio_reqprio = 0;
+	aiocb->aio_lio_opcode = LIO_NOP;
 
-        if (aio_write(aiocb) < 0)
-        {
+	if (aio_write(aiocb) < 0)
+	{
 		_STARPU_DISP("Warning: aio_write returned %d (%s)\n", errno, strerror(errno));
-                if (tmp->descriptor < 0)
-                        _starpu_unistd_reclose(fd);
-                aiocb = NULL;
-        }
+		if (tmp->descriptor < 0)
+			_starpu_unistd_reclose(fd);
+		aiocb = NULL;
+	}
 
-        return event;
+	return event;
 }
 #endif
 
 int starpu_unistd_global_full_write(void *base STARPU_ATTRIBUTE_UNUSED, void *obj, void *ptr, size_t size)
 {
-        struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
+	struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
 
-        /* update file size to realise the next good full_read */
-        if(size != tmp->size)
-        {
+	/* update file size to realise the next good full_read */
+	if(size != tmp->size)
+	{
 		int fd = tmp->descriptor;
 
 		if (fd < 0)
@@ -532,7 +562,7 @@ int starpu_unistd_global_full_write(void *base STARPU_ATTRIBUTE_UNUSED, void *ob
 			_starpu_unistd_reclose(fd);
 		STARPU_ASSERT(val == 0);
 		tmp->size = size;
-        }
+	}
 
 	return starpu_unistd_global_write(base, obj, ptr, 0, size);
 }
@@ -540,7 +570,7 @@ int starpu_unistd_global_full_write(void *base STARPU_ATTRIBUTE_UNUSED, void *ob
 #if defined(HAVE_AIO_H)
 void * starpu_unistd_global_async_full_read (void * base, void * obj, void ** ptr, size_t * size, unsigned dst_node)
 {
-        struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
+	struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
 	int fd = tmp->descriptor;
 
 	if (fd < 0)
@@ -554,6 +584,14 @@ void * starpu_unistd_global_async_full_read (void * base, void * obj, void ** pt
 
 	*size = st.st_size;
 #endif
+#ifdef STARPU_LINUX_SYS
+	/* on Linux, read() (and similar system calls) will transfer at most 0x7ffff000 bytes, see read(2) */
+	/* FIXME: make starpu_unistd_global_test_request and starpu_unistd_global_wait_request
+	 * resubmit an updated request whenever the request completion is truncated */
+	if (*size > 0x7ffff000)
+		return NULL;
+#endif
+
 	if (tmp->descriptor < 0)
 		_starpu_unistd_reclose(fd);
 
@@ -564,11 +602,19 @@ void * starpu_unistd_global_async_full_read (void * base, void * obj, void ** pt
 
 void * starpu_unistd_global_async_full_write (void * base, void * obj, void * ptr, size_t size)
 {
-        struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
+	struct starpu_unistd_global_obj *tmp = (struct starpu_unistd_global_obj *) obj;
 
-        /* update file size to realise the next good full_read */
-        if(size != tmp->size)
-        {
+#ifdef STARPU_LINUX_SYS
+	/* on Linux, write() (and similar system calls) will transfer at most 0x7ffff000 bytes, see write(2) */
+	/* FIXME: make starpu_unistd_global_test_request and starpu_unistd_global_wait_request
+	 * resubmit an updated request whenever the request completion is truncated */
+	if (size > 0x7ffff000)
+		return NULL;
+#endif
+
+	/* update file size to realise the next good full_read */
+	if(size != tmp->size)
+	{
 		int fd = tmp->descriptor;
 
 		if (fd < 0)
@@ -578,7 +624,7 @@ void * starpu_unistd_global_async_full_write (void * base, void * obj, void * pt
 			_starpu_unistd_reclose(fd);
 		STARPU_ASSERT(val == 0);
 		tmp->size = size;
-        }
+	}
 
 	return starpu_unistd_global_async_write(base, obj, ptr, 0, size);
 }
@@ -593,7 +639,7 @@ static void * starpu_unistd_internal_thread(void * arg)
 	{
 		STARPU_PTHREAD_MUTEX_LOCK(&internal_copy_thread->mutex);
 		if (internal_copy_thread->run && starpu_unistd_work_copy_list_empty(&internal_copy_thread->list))
-                        STARPU_PTHREAD_COND_WAIT(&internal_copy_thread->cond, &internal_copy_thread->mutex);
+			STARPU_PTHREAD_COND_WAIT(&internal_copy_thread->cond, &internal_copy_thread->mutex);
 		STARPU_PTHREAD_MUTEX_UNLOCK(&internal_copy_thread->mutex);
 
 		if (!starpu_unistd_work_copy_list_empty(&internal_copy_thread->list))
@@ -622,7 +668,7 @@ static void * starpu_unistd_internal_thread(void * arg)
 				ret = pwrite(work->fd_dst, buf, work->len, work->off_dst);
 				STARPU_ASSERT_MSG(ret >= 0, "Writing failed (errno %d)", errno);
 				STARPU_ASSERT_MSG((size_t) ret == work->len, "Writing failed (value %ld instead of %ld)", (long)ret, (long)work->len);
-				starpu_free(buf);
+				starpu_free_noflag(buf, work->len);
 			}
 			else
 			{
@@ -668,10 +714,10 @@ void *starpu_unistd_global_plug(void *parameter, starpu_ssize_t size STARPU_ATTR
 	}
 
 #if defined(HAVE_LIBAIO_H)
-        STARPU_PTHREAD_MUTEX_INIT(&base->mutex, NULL);
-        base->hashtable = NULL;
-        unsigned nb_event = MAX_PENDING_REQUESTS_PER_NODE + MAX_PENDING_PREFETCH_REQUESTS_PER_NODE + MAX_PENDING_IDLE_REQUESTS_PER_NODE;
-        memset(&base->ctx, 0, sizeof(base->ctx));
+	STARPU_PTHREAD_MUTEX_INIT(&base->mutex, NULL);
+	base->hashtable = NULL;
+	unsigned nb_event = MAX_PENDING_REQUESTS_PER_NODE + MAX_PENDING_PREFETCH_REQUESTS_PER_NODE + MAX_PENDING_IDLE_REQUESTS_PER_NODE;
+	memset(&base->ctx, 0, sizeof(base->ctx));
 	int ret = io_setup(nb_event, &base->ctx);
 	STARPU_ASSERT(ret == 0);
 #endif
@@ -713,8 +759,8 @@ void starpu_unistd_global_unplug(void *base)
 {
 	struct starpu_unistd_base * fileBase = (struct starpu_unistd_base *) base;
 #if defined(HAVE_LIBAIO_H)
-        STARPU_PTHREAD_MUTEX_DESTROY(&fileBase->mutex);
-        io_destroy(fileBase->ctx);
+	STARPU_PTHREAD_MUTEX_DESTROY(&fileBase->mutex);
+	io_destroy(fileBase->ctx);
 #endif
 	if (fileBase->created)
 		rmdir(fileBase->path);
@@ -817,7 +863,7 @@ int _starpu_get_unistd_global_bandwidth_between_disk_and_main_ram(unsigned node,
 	starpu_free_flags(buf, MEM_SIZE, 0);
 
 	_starpu_save_bandwidth_and_latency_disk((NITER/timing_slowness)*STARPU_DISK_SIZE_MIN, (NITER/timing_slowness)*STARPU_DISK_SIZE_MIN,
-			timing_latency/NITER, timing_latency/NITER, node, fileBase->path);
+						timing_latency/NITER, timing_latency/NITER, node, fileBase->path);
 	return 1;
 }
 
@@ -830,14 +876,14 @@ void starpu_unistd_global_wait_request(void *async_channel)
 		{
 #if defined(HAVE_LIBAIO_H)
 			struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-			struct io_event event;
+			struct io_event ev;
 
 			int values = -1;
 			int myerrno = EAGAIN;
 			while(!starpu_aiocb->finished || (values <= 0 && (myerrno == EAGAIN || myerrno == EINTR)))
 			{
 				/* Wait the answer of the request timeout IS NULL */
-				values = io_getevents(starpu_aiocb->base->ctx, 1, 1, &event, NULL);
+				values = io_getevents(starpu_aiocb->base->ctx, 1, 1, &ev, NULL);
 				if (values < 0)
 					myerrno = -values;
 				if (values > 0)
@@ -846,28 +892,33 @@ void starpu_unistd_global_wait_request(void *async_channel)
 					STARPU_PTHREAD_MUTEX_LOCK(&starpu_aiocb->base->mutex);
 
 					struct starpu_unistd_aiocb_link *l = NULL;
-					HASH_FIND_PTR(starpu_aiocb->base->hashtable, &event.obj, l);
+					HASH_FIND_PTR(starpu_aiocb->base->hashtable, &ev.obj, l);
 					STARPU_ASSERT(l != NULL);
 
 					HASH_DEL(starpu_aiocb->base->hashtable, l);
 					STARPU_PTHREAD_MUTEX_UNLOCK(&starpu_aiocb->base->mutex);
-					((struct starpu_unistd_aiocb *) l->starpu_aiocb)->finished = 1;
+					struct starpu_unistd_aiocb *aiocb = l->starpu_aiocb;
+					STARPU_ASSERT_MSG(ev.res == aiocb->len, "Aio request was truncated");
+					aiocb->finished = 1;
 					free(l);
 				}
 			}
 #elif defined(HAVE_AIO_H)
 			struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-			const struct aiocb *aiocb = &starpu_aiocb->aiocb;
+			struct aiocb *aiocb = &starpu_aiocb->aiocb;
 			int values = -1;
 			int ret, myerrno = EAGAIN;
+			starpu_ssize_t size;
 			while(values < 0 && (myerrno == EAGAIN || myerrno == EINTR))
 			{
 				/* Wait the answer of the request TIMESTAMP IS NULL */
-				values = aio_suspend(&aiocb, 1, NULL);
+				values = aio_suspend((const struct aiocb **) &aiocb, 1, NULL);
 				myerrno = errno;
 			}
 			ret = aio_error(aiocb);
 			STARPU_ASSERT_MSG(!ret, "aio_error returned %d", ret);
+			size = aio_return(aiocb);
+			STARPU_ASSERT(size == (starpu_ssize_t) aiocb->aio_nbytes);
 #endif
 			break;
 		}
@@ -895,7 +946,7 @@ int starpu_unistd_global_test_request(void *async_channel)
 		{
 #if defined(HAVE_LIBAIO_H)
 			struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-			struct io_event event;
+			struct io_event ev;
 			struct timespec ts;
 			int ret;
 
@@ -905,7 +956,7 @@ int starpu_unistd_global_test_request(void *async_channel)
 			memset(&ts, 0, sizeof(ts));
 
 			/* Test the answer of the request */
-			ret = io_getevents(starpu_aiocb->base->ctx, 0, 1, &event, &ts);
+			ret = io_getevents(starpu_aiocb->base->ctx, 0, 1, &ev, &ts);
 
 			if (ret == 1)
 			{
@@ -913,12 +964,14 @@ int starpu_unistd_global_test_request(void *async_channel)
 				STARPU_PTHREAD_MUTEX_LOCK(&starpu_aiocb->base->mutex);
 
 				struct starpu_unistd_aiocb_link *l = NULL;
-				HASH_FIND_PTR(starpu_aiocb->base->hashtable, &event.obj, l);
+				HASH_FIND_PTR(starpu_aiocb->base->hashtable, &ev.obj, l);
 				STARPU_ASSERT(l != NULL);
 
 				HASH_DEL(starpu_aiocb->base->hashtable, l);
 				STARPU_PTHREAD_MUTEX_UNLOCK(&starpu_aiocb->base->mutex);
-				((struct starpu_unistd_aiocb *) l->starpu_aiocb)->finished = 1;
+				struct starpu_unistd_aiocb *aiocb = l->starpu_aiocb;
+				STARPU_ASSERT_MSG(ev.res == aiocb->len, "Aio request was truncated");
+				aiocb->finished = 1;
 				free(l);
 
 				if (starpu_aiocb->finished)
@@ -928,22 +981,27 @@ int starpu_unistd_global_test_request(void *async_channel)
 			return 0;
 #elif defined(HAVE_AIO_H)
 			struct starpu_unistd_aiocb *starpu_aiocb = &event->event.event_aiocb;
-			const struct aiocb *aiocb = &starpu_aiocb->aiocb;
+			struct aiocb *aiocb = &starpu_aiocb->aiocb;
 			int ret;
 
 #if defined(__GLIBC__) && (__GLIBC__ < 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ < 22))
 			/* glibc's aio_error was not threadsafe before glibc 2.22 */
 			struct timespec ts = { .tv_sec = 0, .tv_nsec = 0 };
-			ret = aio_suspend(&aiocb, 1, &ts);
+			ret = aio_suspend((const struct aiocb **) &aiocb, 1, &ts);
 			if (ret < 0 && (errno == EAGAIN || errno == EINTR))
 				return 0;
 			STARPU_ASSERT_MSG(!ret, "aio_suspend returned %d %d\n", ret, errno);
 #endif
+			starpu_ssize_t size;
 			/* Test the answer of the request */
 			ret = aio_error(aiocb);
 			if (ret == 0)
+			{
 				/* request is finished */
+				size = aio_return(aiocb);
+				STARPU_ASSERT_MSG(size == (starpu_ssize_t) aiocb->aio_nbytes, "AIO op got %ld bytes instead of %ld bytes\n", (long) size, (long) aiocb->aio_nbytes);
 				return 1;
+			}
 			if (ret == EINTR || ret == EINPROGRESS || ret == EAGAIN)
 				return 0;
 			/* an error occured */
@@ -985,7 +1043,6 @@ void starpu_unistd_global_free_request(void *async_channel)
 			struct aiocb *aiocb = &starpu_aiocb->aiocb;
 			if (starpu_aiocb->obj->descriptor < 0)
 				_starpu_unistd_reclose(aiocb->aio_fildes);
-			aio_return(aiocb);
 			free(event);
 #endif
 			break;
@@ -1057,7 +1114,7 @@ void *  starpu_unistd_global_copy(void *base_src, void* obj_src, off_t offset_sr
 
 	STARPU_PTHREAD_MUTEX_LOCK(&thread->mutex);
 	starpu_unistd_work_copy_list_push_front(&thread->list, work);
-        STARPU_PTHREAD_COND_BROADCAST(&thread->cond);
+	STARPU_PTHREAD_COND_BROADCAST(&thread->cond);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&thread->mutex);
 
 	return event;
