@@ -1,10 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2017                                Inria
- * Copyright (C) 2008-2018                                Université de Bordeaux
- * Copyright (C) 2010-2019                                CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,12 +24,16 @@
 #include <common/config.h>
 #include <common/utils.h>
 #include <common/graph.h>
+#include <datawizard/memory_nodes.h>
 #include <profiling/profiling.h>
 #include <profiling/bound.h>
 #include <core/debug.h>
 #include <limits.h>
+#include <core/workers.h>
 
 static int max_memory_use;
+static int task_progress;
+static unsigned long njobs_finished;
 static unsigned long njobs, maxnjobs;
 
 #ifdef STARPU_DEBUG
@@ -40,41 +42,55 @@ static struct _starpu_job_multilist_all_submitted all_jobs_list;
 static starpu_pthread_mutex_t all_jobs_list_mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
 #endif
 
+void _starpu_job_crash();
+
 void _starpu_job_init(void)
 {
-	max_memory_use = starpu_get_env_number_default("STARPU_MAX_MEMORY_USE", 0);
+	max_memory_use = starpu_getenv_number_default("STARPU_MAX_MEMORY_USE", 0);
+	task_progress = starpu_getenv_number_default("STARPU_TASK_PROGRESS", 0);
 #ifdef STARPU_DEBUG
 	_starpu_job_multilist_head_init_all_submitted(&all_jobs_list);
 #endif
+	_starpu_crash_add_hook(&_starpu_job_crash);
 }
 
-void _starpu_job_fini(void)
+void _starpu_job_memory_use(int check)
 {
 	if (max_memory_use)
 	{
 		_STARPU_DISP("Memory used for %lu tasks: %lu MiB\n", maxnjobs, (unsigned long) (maxnjobs * (sizeof(struct starpu_task) + sizeof(struct _starpu_job))) >> 20);
-		STARPU_ASSERT_MSG(njobs == 0, "Some tasks have not been cleaned, did you forget to call starpu_task_destroy or starpu_task_clean?");
+		if (check)
+			STARPU_ASSERT_MSG(njobs == 0, "Some tasks have not been cleaned, did you forget to call starpu_task_destroy or starpu_task_clean?");
 	}
 }
 
+void _starpu_job_crash()
+{
+	_starpu_job_memory_use(0);
+}
+
+void _starpu_job_fini(void)
+{
+	_starpu_job_memory_use(1);
+}
+
 void _starpu_exclude_task_from_dag(struct starpu_task *task)
 {
 	struct _starpu_job *j = _starpu_get_job_associated_to_task(task);
 
 	j->exclude_from_dag = 1;
+	_STARPU_TRACE_TASK_EXCLUDE_FROM_DAG(j);
 }
 
 /* create an internal struct _starpu_job structure to encapsulate the task */
 struct _starpu_job* STARPU_ATTRIBUTE_MALLOC _starpu_job_create(struct starpu_task *task)
 {
 	struct _starpu_job *job;
-        _STARPU_LOG_IN();
-
-	_STARPU_MALLOC(job, sizeof(*job));
+	_STARPU_LOG_IN();
 
 	/* As most of the fields must be initialized at NULL, let's put 0
 	 * everywhere */
-	memset(job, 0, sizeof(*job));
+	_STARPU_CALLOC(job, 1, sizeof(*job));
 
 	if (task->dyn_handles)
 	{
@@ -84,9 +100,15 @@ struct _starpu_job* STARPU_ATTRIBUTE_MALLOC _starpu_job_create(struct starpu_tas
 
 	job->task = task;
 
-#ifndef STARPU_USE_FXT
-	if (_starpu_bound_recording || _starpu_task_break_on_push != -1 || _starpu_task_break_on_sched != -1 || _starpu_task_break_on_pop != -1 || _starpu_task_break_on_exec != -1 || STARPU_AYU_EVENT)
+	if (
+#if defined(STARPU_DEBUG)
+	    1
+#elif defined(STARPU_USE_FXT)
+	    fut_active
+#else
+	    _starpu_bound_recording || _starpu_task_break_on_push != -1 || _starpu_task_break_on_sched != -1 || _starpu_task_break_on_pop != -1 || _starpu_task_break_on_exec != -1 || STARPU_AYU_EVENT
 #endif
+	   )
 	{
 		job->job_id = _starpu_fxt_get_job_id();
 		STARPU_AYU_ADDTASK(job->job_id, task);
@@ -99,21 +121,57 @@ struct _starpu_job* STARPU_ATTRIBUTE_MALLOC _starpu_job_create(struct starpu_tas
 			maxnjobs = jobs;
 	}
 
-	_starpu_cg_list_init(&job->job_successors);
+	_starpu_cg_list_init0(&job->job_successors);
 
-	STARPU_PTHREAD_MUTEX_INIT(&job->sync_mutex, NULL);
-	STARPU_PTHREAD_COND_INIT(&job->sync_cond, NULL);
+	STARPU_PTHREAD_MUTEX_INIT0(&job->sync_mutex, NULL);
+	STARPU_PTHREAD_COND_INIT0(&job->sync_cond, NULL);
 
 	/* By default we have sequential tasks */
 	job->task_size = 1;
 
+	job->workerid = -1;
+
 	if (task->use_tag)
 		_starpu_tag_declare(task->tag_id, job);
 
 	if (_starpu_graph_record)
 		_starpu_graph_add_job(job);
 
-        _STARPU_LOG_OUT();
+	_STARPU_LOG_OUT();
+	return job;
+}
+
+struct _starpu_job* _starpu_get_job_associated_to_task_slow(struct starpu_task *task, struct _starpu_job *job)
+{
+	if (job == _STARPU_JOB_UNSET)
+	{
+		job = STARPU_VAL_COMPARE_AND_SWAP_PTR(&task->starpu_private, _STARPU_JOB_UNSET, _STARPU_JOB_SETTING);
+		if (job != _STARPU_JOB_UNSET && job != _STARPU_JOB_SETTING)
+		{
+			/* Actually available in the meanwhile */
+			STARPU_RMB();
+			return job;
+		}
+
+		if (job == _STARPU_JOB_UNSET)
+		{
+			/* Ok, we have to do it */
+			job = _starpu_job_create(task);
+			STARPU_WMB();
+			task->starpu_private = job;
+			return job;
+		}
+	}
+
+	/* Saw _STARPU_JOB_SETTING, somebody is doing it, wait for it.
+	 * This is rare enough that busy-reading is fine enough. */
+	while ((job = *(struct _starpu_job *volatile*) &task->starpu_private) == _STARPU_JOB_SETTING)
+	{
+		STARPU_UYIELD();
+		STARPU_SYNCHRONIZE();
+	}
+
+	STARPU_RMB();
 	return job;
 }
 
@@ -167,7 +225,7 @@ void _starpu_wait_job(struct _starpu_job *j)
 {
 	STARPU_ASSERT(j->task);
 	STARPU_ASSERT(!j->task->detach);
-        _STARPU_LOG_IN();
+	_STARPU_LOG_IN();
 
 	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
 
@@ -183,7 +241,7 @@ void _starpu_wait_job(struct _starpu_job *j)
 	}
 
 	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
-        _STARPU_LOG_OUT();
+	_STARPU_LOG_OUT();
 }
 
 #ifdef STARPU_OPENMP
@@ -224,6 +282,7 @@ void _starpu_job_prepare_for_continuation_ext(struct _starpu_job *j, unsigned co
 	j->continuation_callback_on_sleep = continuation_callback_on_sleep;
 	j->continuation_callback_on_sleep_arg = continuation_callback_on_sleep_arg;
 	j->job_successors.ndeps = 0;
+	j->job_successors.ndeps_completed = 0;
 }
 /* Prepare a currently running job for accepting a new set of
  * dependencies in anticipation of becoming a continuation. */
@@ -260,6 +319,16 @@ void _starpu_handle_job_submission(struct _starpu_job *j)
 void starpu_task_end_dep_release(struct starpu_task *t)
 {
 	struct _starpu_job *j = _starpu_get_job_associated_to_task(t);
+
+#ifdef STARPU_USE_FXT
+	struct starpu_task *current = starpu_task_get_current();
+	if (current)
+	{
+		struct _starpu_job *jcurrent = _starpu_get_job_associated_to_task(current);
+		_STARPU_TRACE_TASK_END_DEP(jcurrent, j);
+	}
+#endif
+
 	_starpu_handle_job_termination(j);
 }
 
@@ -282,10 +351,18 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 		if (nb != 0) return;
 	}
 
+	if (task_progress)
+	{
+		unsigned long jobs = STARPU_ATOMIC_ADDL(&njobs_finished, 1);
+
+		fprintf(stderr,"\r%lu tasks finished (last %lu %p on %d)...", jobs, j->job_id, j->task, starpu_worker_get_id());
+	}
+
 	struct starpu_task *task = j->task;
 	struct starpu_task *end_rdep = NULL;
 	unsigned sched_ctx = task->sched_ctx;
 	double flops = task->flops;
+
 	const unsigned continuation =
 #ifdef STARPU_OPENMP
 		j->continuation
@@ -293,6 +370,38 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 		0
 #endif
 		;
+
+	if (!continuation)
+	{
+		void (*epilogue_callback)(void *) = task->epilogue_callback_func;
+		/* the epilogue callback is executed before the dependencies release*/
+		if (epilogue_callback)
+		{
+			enum _starpu_worker_status old_status = _starpu_get_local_worker_status();
+
+			/* so that we can check whether we are doing blocking calls
+			 * within the callback */
+			if (!(old_status & STATUS_CALLBACK))
+				_starpu_add_local_worker_status(STATUS_INDEX_CALLBACK, NULL);
+
+			/* Perhaps we have nested callbacks (eg. with chains of empty
+			 * tasks). So we store the current task and we will restore it
+			 * later. */
+			struct starpu_task *current_task = starpu_task_get_current();
+
+			_starpu_set_current_task(task);
+
+			_STARPU_TRACE_START_CALLBACK(j);
+			epilogue_callback(task->epilogue_callback_arg);
+			_STARPU_TRACE_END_CALLBACK(j);
+
+			_starpu_set_current_task(current_task);
+
+			if (!(old_status & STATUS_CALLBACK))
+				_starpu_clear_local_worker_status(STATUS_INDEX_CALLBACK, NULL);
+		}
+	}
+
 #ifdef STARPU_DEBUG
 	STARPU_PTHREAD_MUTEX_LOCK(&all_jobs_list_mutex);
 	_starpu_job_multilist_erase_all_submitted(&all_jobs_list, j);
@@ -311,6 +420,10 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	{
 		task->status = STARPU_TASK_FINISHED;
 
+		/* already prepare for next run */
+		struct _starpu_cg_list *job_successors = &j->job_successors;
+		job_successors->ndeps_completed = 0;
+
 		/* We must have set the j->terminated flag early, so that it is
 		 * possible to express task dependencies within the callback
 		 * function. A value of 1 means that the codelet was executed but that
@@ -326,7 +439,11 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 #endif //STARPU_USE_SC_HYPERVISOR
 
 	/* We release handle reference count */
-	if (task->cl && !continuation)
+	if (task->cl && !continuation
+#ifdef STARPU_BUBBLE
+	    && !j->is_bubble
+#endif
+	    )
 	{
 		unsigned i;
 		unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(task);
@@ -348,21 +465,14 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 				_starpu_spin_unlock(&handle->header_lock);
 		}
 	}
-	/* If this is a continuation, we do not release task dependencies now.
-	 * Task dependencies will be released only when the continued task
-	 * fully completes */
-	if (!continuation)
-	{
-		/* Tell other tasks that we don't exist any more, thus no need for
-		 * implicit dependencies any more.  */
-		_starpu_release_task_enforce_sequential_consistency(j);
-	}
 
+	/* Check nowhere before releasing the sequential consistency (which may
+	 * unregister the handle and free its switch_cl, and thus task->cl here.  */
+	unsigned nowhere = !task->cl || task->cl->where == STARPU_NOWHERE || task->where == STARPU_NOWHERE;
 	/* If the job was executed on a combined worker there is no need for the
 	 * scheduler to process it : the task structure doesn't contain any valuable
 	 * data as it's not linked to an actual worker */
 	/* control task should not execute post_exec_hook */
-	unsigned nowhere = !task->cl || task->cl->where == STARPU_NOWHERE || task->where == STARPU_NOWHERE;
 	if(j->task_size == 1 && !nowhere && !j->internal
 #ifdef STARPU_OPENMP
 	/* If this is a continuation, we do not execute the post_exec_hook. The
@@ -387,6 +497,23 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	if (_starpu_graph_record)
 		_starpu_graph_drop_job(j);
 
+	/* Get callback pointer for codelet before notifying dependencies, in
+	   case dependencies free the codelet (see starpu_data_unregister for
+	   instance) */
+	void (*callback)(void *) = task->callback_func;
+	if (!callback && task->cl)
+		callback = task->cl->callback_func;
+
+	/* If this is a continuation, we do not release task dependencies now.
+	 * Task dependencies will be released only when the continued task
+	 * fully completes */
+	if (!continuation)
+	{
+		/* Tell other tasks that we don't exist any more, thus no need for
+		 * implicit dependencies any more.  */
+		_starpu_release_task_enforce_sequential_consistency(j);
+	}
+
 	/* Task does not have a cl, but has explicit data dependencies, we need
 	 * to tell them that we will not exist any more before notifying the
 	 * tasks waiting for us
@@ -404,36 +531,36 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 			_starpu_spin_unlock(&handle->header_lock);
 	}
 
-	_STARPU_TRACE_TASK_NAME(j);
-
-	/* If this is a continuation, we do not notify task/tag dependencies
-	 * now. Task/tag dependencies will be notified only when the continued
-	 * task fully completes */
 	if (!continuation)
 	{
+		/* If this is a continuation, we do not notify task/tag dependencies
+		 * now. Task/tag dependencies will be notified only when the continued
+		 * task fully completes */
 		/* in case there are dependencies, wake up the proper tasks */
 		if (end_rdep)
 			starpu_task_end_dep_release(end_rdep);
 		_starpu_notify_dependencies(j);
-	}
 
-	/* If this is a continuation, we do not execute the callback
-	 * now. The callback will be executed only when the continued
-	 * task fully completes */
-	if (!continuation)
-	{
+		/* If this is a continuation, we do not execute the callback
+		 * now. The callback will be executed only when the continued
+		 * task fully completes */
 		/* the callback is executed after the dependencies so that we may remove the tag
 		 * of the task itself */
-		if (task->callback_func)
+		if (callback)
 		{
+			struct timespec *time = NULL;
 			int profiling = starpu_profiling_status_get();
 			if (profiling && task->profiling_info)
-				_starpu_clock_gettime(&task->profiling_info->callback_start_time);
+			{
+				time = &task->profiling_info->callback_start_time;
+				_starpu_clock_gettime(time);
+			}
+			enum _starpu_worker_status old_status = _starpu_get_local_worker_status();
 
 			/* so that we can check whether we are doing blocking calls
 			 * within the callback */
-			_starpu_set_local_worker_status(STATUS_CALLBACK);
-
+			if (!(old_status & STATUS_CALLBACK))
+				_starpu_add_local_worker_status(STATUS_INDEX_CALLBACK, time);
 
 			/* Perhaps we have nested callbacks (eg. with chains of empty
 			 * tasks). So we store the current task and we will restore it
@@ -443,15 +570,19 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 			_starpu_set_current_task(task);
 
 			_STARPU_TRACE_START_CALLBACK(j);
-			task->callback_func(task->callback_arg);
+			callback(task->callback_arg);
 			_STARPU_TRACE_END_CALLBACK(j);
 
 			_starpu_set_current_task(current_task);
 
-			_starpu_set_local_worker_status(STATUS_UNKNOWN);
-
 			if (profiling && task->profiling_info)
-				_starpu_clock_gettime(&task->profiling_info->callback_end_time);
+			{
+				time = &task->profiling_info->callback_end_time;
+				_starpu_clock_gettime(time);
+			}
+
+			if (!(old_status & STATUS_CALLBACK))
+				_starpu_clear_local_worker_status(STATUS_INDEX_CALLBACK, time);
 		}
 	}
 
@@ -460,16 +591,16 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	 */
 	_STARPU_TRACE_TASK_DONE(j);
 
+	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+
 	/* NB: we do not save those values before the callback, in case the
 	 * application changes some parameters eventually (eg. a task may not
 	 * be generated if the application is terminated). */
 	unsigned destroy = task->destroy;
 	unsigned detach = task->detach;
 	unsigned regenerate = task->regenerate;
+	unsigned synchronous = task->synchronous;
 
-	/* we do not desallocate the job structure if some is going to
-	 * wait after the task */
-	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
 	if (!continuation)
 	{
 #ifdef STARPU_OPENMP
@@ -484,10 +615,13 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 		 * also the callback were executed. */
 		j->terminated = 2;
 	}
+	task->prefetched = 0;
 	STARPU_PTHREAD_COND_BROADCAST(&j->sync_cond);
 	STARPU_AYU_REMOVETASK(j->job_id);
 	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
+	/* we do not desallocate the job structure if some is going to
+	 * wait after the task */
 	if (detach && !continuation)
 	{
 		/* no one is going to synchronize with that task so we release
@@ -501,9 +635,9 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 	/* A continuation is not much different from a regenerated task. */
 	if (regenerate || continuation)
 	{
-		STARPU_ASSERT_MSG((detach && !destroy && !task->synchronous)
+		STARPU_ASSERT_MSG((detach && !destroy && !synchronous)
 				|| continuation
-				, "Regenerated task must be detached (was %u), and not have detroy=1 (was %u) or synchronous=1 (was %u)", detach, destroy, task->synchronous);
+				, "Regenerated task must be detached (was %u), and not have detroy=1 (was %u) or synchronous=1 (was %u)", detach, destroy, synchronous);
 		STARPU_AYU_ADDTASK(j->job_id, j->exclude_from_dag?NULL:task);
 
 		{
@@ -519,7 +653,7 @@ void _starpu_handle_job_termination(struct _starpu_job *j)
 			{
 				/* We reuse the same job structure */
 				task->status = STARPU_TASK_BLOCKED;
-				int ret = _starpu_submit_job(j);
+				int ret = _starpu_submit_job(j, 0);
 				STARPU_ASSERT(!ret);
 			}
 #ifdef STARPU_OPENMP
@@ -570,7 +704,7 @@ static unsigned _starpu_not_all_tag_deps_are_fulfilled(struct _starpu_job *j)
 	if (tag_successors->ndeps != tag_successors->ndeps_completed)
 	{
 		tag->state = STARPU_BLOCKED;
-                j->task->status = STARPU_TASK_BLOCKED_ON_TAG;
+		j->task->status = STARPU_TASK_BLOCKED_ON_TAG;
 		ret = 1;
 	}
 	else
@@ -599,20 +733,150 @@ static unsigned _starpu_not_all_task_deps_are_fulfilled(struct _starpu_job *j)
 	if (!j->submitted || (job_successors->ndeps != job_successors->ndeps_completed))
 	{
 		STARPU_ASSERT(j->task->status == STARPU_TASK_BLOCKED || j->task->status == STARPU_TASK_BLOCKED_ON_TAG);
-                j->task->status = STARPU_TASK_BLOCKED_ON_TASK;
+		j->task->status = STARPU_TASK_BLOCKED_ON_TASK;
 		ret = 1;
 	}
 	else
 	{
 		/* existing deps (if any) are fulfilled */
-		/* already prepare for next run */
-		job_successors->ndeps_completed = 0;
 		ret = 0;
 	}
 
 	return ret;
 }
 
+#ifdef STARPU_BUBBLE
+int _starpu_bubble_unpartition_data_if_needed(struct _starpu_job *j)
+{
+	//_STARPU_DEBUG("[%s(%p)]\n", starpu_task_get_name(j->task), j->task);
+	int unpartition_needed = 0;
+	unsigned nbuffers = STARPU_TASK_GET_NBUFFERS(j->task);
+	unsigned nhandle = 0;
+	unsigned i;
+	struct starpu_task *control_task = NULL;
+
+	for (i = 0; i < nbuffers; i++)
+	{
+		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(j->task, i);
+		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(j->task, i);
+
+		STARPU_PTHREAD_MUTEX_LOCK(&handle->unpartition_mutex);
+
+		/**
+		 * Version A
+		 *
+		 * We create a control task with the required data
+		 * dependencies that will be automatically/magically
+		 * handled by _starpu_data_partition_access_submit()
+		 * called in _starpu_task_submit_head().
+		 */
+		if (handle->nplans > 0)
+		{
+			if (unpartition_needed == 0)
+			{
+				control_task = starpu_task_create();
+				control_task->name = "ucontrol";
+				_starpu_task_declare_deps_array(j->task, 1, &control_task, 0);
+
+				unpartition_needed = 1;
+			}
+
+			//STARPU_TASK_SET_HANDLE(control_task, handle, nhandle);
+			control_task->handles[nhandle] = handle;
+			//STARPU_TASK_SET_MODE(control_task, mode, nhandle);
+			control_task->modes[nhandle] = mode;
+			nhandle ++;
+		}
+		/**
+		 * Version B
+		 *
+		 * We find a way to call directly
+		 * _starpu_data_partition_access_submit() here, and we
+		 * (re-)plug the current task onto the last task
+		 * generated by
+		 * _starpu_data_partition_access_submit().
+		 */
+		else
+		{
+			//_starpu_data_partition_access_submit(handle, (mode & STARPU_W) != 0);
+			// + replug on the current task
+		}
+		STARPU_PTHREAD_MUTEX_UNLOCK(&handle->unpartition_mutex);
+	}
+
+	// No data has been partitioned, let's keep going
+	if (unpartition_needed == 0)
+	{
+		return 0;
+	}
+
+	// Add the dependency on the unpartition tasks
+	STARPU_PTHREAD_MUTEX_LOCK(&j->sync_mutex);
+	j->task->status = STARPU_TASK_BLOCKED_ON_TASK;
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+
+	STARPU_ASSERT(control_task);
+	int ret = starpu_task_submit(control_task);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit(control_task)");
+
+	return 1;
+}
+
+static int _starpu_turn_task_into_bubble(struct _starpu_job *j)
+{
+	if (j->already_turned_into_bubble)
+	{
+		/*
+		 * We have first checked all dependencies of the bubble,
+		 * and secondly checked in a second stage the additional
+		 * partition/unpartition dependencies
+		 */
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		return 0;
+	}
+	j->already_turned_into_bubble = 1;
+	//_STARPU_DEBUG("[%s(%p)]\n", starpu_task_get_name(j->task), j->task);
+
+	if (j->is_bubble == 1)
+	{
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		return 0;
+	}
+	else if (j->task->cl == NULL)
+	{
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		return 0;
+	}
+	else
+	{
+		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+		return _starpu_bubble_unpartition_data_if_needed(j);
+	}
+}
+
+void _starpu_bubble_execute(struct _starpu_job *j)
+{
+	_STARPU_TRACE_BUBBLE(j);
+	_STARPU_TRACE_TASK_NAME_LINE_COLOR(j);
+	_STARPU_TRACE_START_CODELET_BODY(j, 0, NULL, 0);
+	STARPU_ASSERT_MSG(j->task->bubble_gen_dag_func!=NULL || (j->task->cl && j->task->cl->bubble_gen_dag_func!=NULL),
+			  "task->bubble_gen_dag_func MUST be defined\n");
+
+#ifdef STARPU_VERBOSE
+	struct timespec tp;
+	clock_gettime(CLOCK_MONOTONIC, &tp);
+	unsigned long long timestamp = 1000000000ULL*tp.tv_sec + tp.tv_nsec;
+	_STARPU_DEBUG("{%llu} [%s(%p)] Running bubble\n", timestamp, starpu_task_get_name(j->task), j->task);
+#endif
+	if (j->task->bubble_gen_dag_func)
+		j->task->bubble_gen_dag_func(j->task, j->task->bubble_gen_dag_func_arg);
+	else
+		j->task->cl->bubble_gen_dag_func(j->task, j->task->bubble_gen_dag_func_arg);
+	j->task->where = STARPU_NOWHERE;
+	_STARPU_TRACE_END_CODELET_BODY(j, 0, NULL, 0);
+}
+#endif
+
 /*
  *	In order, we enforce tag, task and data dependencies. The task is
  *	passed to the scheduler only once all these constraints are fulfilled.
@@ -623,7 +887,7 @@ static unsigned _starpu_not_all_task_deps_are_fulfilled(struct _starpu_job *j)
 unsigned _starpu_enforce_deps_and_schedule(struct _starpu_job *j)
 {
 	unsigned ret;
-        _STARPU_LOG_IN();
+	_STARPU_LOG_IN();
 
 	/* enfore tag dependencies */
 	if (_starpu_not_all_tag_deps_are_fulfilled(j))
@@ -633,21 +897,51 @@ unsigned _starpu_enforce_deps_and_schedule(struct _starpu_job *j)
 		return 0;
 	}
 
-	/* enfore task dependencies */
+	/* enforce task dependencies */
 	if (_starpu_not_all_task_deps_are_fulfilled(j))
 	{
 		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 		_STARPU_LOG_OUT_TAG("not_all_task_deps_are_fulfilled");
 		return 0;
 	}
-	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 
-	/* enforce data dependencies */
-	if (_starpu_submit_job_enforce_data_deps(j))
+#ifdef STARPU_BUBBLE
+	/* Wait for all dependencies at the correct level to be
+	 * fulfilled before adding missing partition/unpartition
+	 *
+	 * If partition/unpartition are submitted we will enter the if
+	 * case and come back later when these new dependencies are
+	 * fulfilled
+	 */
+	if (_starpu_turn_task_into_bubble(j))
 	{
-		_STARPU_LOG_OUT_TAG("enforce_data_deps");
+		_STARPU_LOG_OUT_TAG("bubble");
 		return 0;
 	}
+#else
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+#endif
+
+#ifdef STARPU_BUBBLE
+	if (j->is_bubble == 1)
+	{
+		_starpu_bubble_execute(j);
+	}
+	else
+#endif
+	{
+		/* respect data concurrent access */
+		if (_starpu_concurrent_data_access(j))
+		{
+			_STARPU_LOG_OUT_TAG("concurrent_data_access");
+			return 0;
+		}
+	}
+
+#ifdef STARPU_BUBBLE
+	if (j->task->bubble_parent != 0)
+		_STARPU_TRACE_BUBBLE_TASK_DEPS(j->task->bubble_parent, j);
+#endif
 
 	ret = _starpu_push_task(j);
 
@@ -666,11 +960,33 @@ unsigned _starpu_enforce_deps_starting_from_task(struct _starpu_job *j)
 		STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
 		return 0;
 	}
-	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
-
-	/* enforce data dependencies */
-	if (_starpu_submit_job_enforce_data_deps(j))
+#ifdef STARPU_BUBBLE
+	if (_starpu_turn_task_into_bubble(j))
+	{
+		_STARPU_LOG_OUT_TAG("bubble");
 		return 0;
+	}
+#else
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+#endif
+
+#ifdef STARPU_BUBBLE
+	if (j->is_bubble == 1)
+	{
+		_starpu_bubble_execute(j);
+	}
+	else
+#endif
+	{
+		/* respect data concurrent access */
+		if (_starpu_concurrent_data_access(j))
+			return 0;
+	}
+
+#ifdef STARPU_BUBBLE
+	if (j->task->bubble_parent != 0)
+		_STARPU_TRACE_BUBBLE_TASK_DEPS(j->task->bubble_parent, j);
+#endif
 
 	ret = _starpu_push_task(j);
 
@@ -682,7 +998,7 @@ unsigned _starpu_enforce_deps_starting_from_task(struct _starpu_job *j)
 unsigned _starpu_reenforce_task_deps_and_schedule(struct _starpu_job *j)
 {
 	unsigned ret;
-        _STARPU_LOG_IN();
+	_STARPU_LOG_IN();
 	STARPU_ASSERT(j->discontinuous);
 
 	/* enfore task dependencies */
@@ -700,6 +1016,25 @@ unsigned _starpu_reenforce_task_deps_and_schedule(struct _starpu_job *j)
 }
 #endif
 
+unsigned _starpu_take_deps_and_schedule(struct _starpu_job *j)
+{
+	unsigned ret;
+	STARPU_PTHREAD_MUTEX_UNLOCK(&j->sync_mutex);
+
+	/* Take references */
+	_starpu_submit_job_take_data_deps(j);
+
+#ifdef STARPU_BUBBLE
+	if (j->task->bubble_parent != 0)
+		_STARPU_TRACE_BUBBLE_TASK_DEPS(j->task->bubble_parent, j);
+#endif
+
+	/* And immediately push task */
+	ret = _starpu_push_task(j);
+
+	return ret;
+}
+
 /* This is called when a tag or task dependency is to be released.  */
 void _starpu_enforce_deps_notify_job_ready_soon(struct _starpu_job *j, _starpu_notify_job_start_data *data, int tag)
 {
@@ -707,10 +1042,10 @@ void _starpu_enforce_deps_notify_job_ready_soon(struct _starpu_job *j, _starpu_n
 		/* It's not even submitted actually */
 		return;
 	struct _starpu_cg_list *job_successors = &j->job_successors;
-        /* tag is 1 when we got woken up by a tag dependency about to be
-         * released, and thus we have to check the exact numbner of
-         * dependencies.  Otherwise it's a task dependency which is about to be
-         * released.  */
+	/* tag is 1 when we got woken up by a tag dependency about to be
+	 * released, and thus we have to check the exact numbner of
+	 * dependencies.  Otherwise it's a task dependency which is about to be
+	 * released.  */
 	if (job_successors->ndeps != job_successors->ndeps_completed + 1 - tag)
 		/* There are still other dependencies */
 		return;
@@ -742,14 +1077,14 @@ struct starpu_task *_starpu_pop_local_task(struct _starpu_worker *worker)
 		}
 	}
 
-	if (!starpu_task_list_empty(&worker->local_tasks))
-		task = starpu_task_list_pop_front(&worker->local_tasks);
+	if (!starpu_task_prio_list_empty(&worker->local_tasks))
+		task = starpu_task_prio_list_pop_front_highest(&worker->local_tasks);
 
 	_starpu_pop_task_end(task);
 	return task;
 }
 
-int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *task, int prio)
+int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *task)
 {
 	/* Check that the worker is able to execute the task ! */
 	STARPU_ASSERT(task && task->cl);
@@ -792,13 +1127,7 @@ int _starpu_push_local_task(struct _starpu_worker *worker, struct starpu_task *t
 	}
 	else
 	{
-#ifdef STARPU_DEVEL
-#warning FIXME use a prio_list
-#endif
-		if (prio)
-			starpu_task_list_push_front(&worker->local_tasks, task);
-		else
-			starpu_task_list_push_back(&worker->local_tasks, task);
+		starpu_task_prio_list_push_back(&worker->local_tasks, task);
 	}
 
 	starpu_wake_worker_locked(worker->workerid);
