@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2012-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,11 +14,11 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <starpu.h>
-#include <core/simgrid.h>
 #include <common/config.h>
 
 #ifdef STARPU_SIMGRID
+#include <starpu.h>
+#include <core/simgrid.h>
 #if SIMGRID_VERSION >= 32190
 #include <simgrid/simix.hpp>
 #else
@@ -26,6 +26,27 @@
 #endif
 #include <smpi/smpi.h>
 
+#ifdef STARPU_HAVE_S4U_ON_TIME_ADVANCE_CB
+#include <simgrid/s4u/Engine.hpp>
+#endif
+
+#ifdef STARPU_HAVE_S4U_ON_TIME_ADVANCE_CB
+starpu_pthread_mutex_t _starpu_simgrid_time_advance_mutex;
+starpu_pthread_cond_t _starpu_simgrid_time_advance_cond;
+#endif
+
+void _starpu_simgrid_cpp_init(void)
+{
+#ifdef STARPU_HAVE_S4U_ON_TIME_ADVANCE_CB
+	STARPU_PTHREAD_MUTEX_INIT(&_starpu_simgrid_time_advance_mutex, NULL);
+	STARPU_PTHREAD_COND_INIT(&_starpu_simgrid_time_advance_cond, NULL);
+	simgrid::s4u::Engine::on_time_advance_cb([](double)
+	{
+		STARPU_PTHREAD_COND_BROADCAST(&_starpu_simgrid_time_advance_cond);
+	});
+#endif
+}
+
 /* thread_create function which implements inheritence of MPI privatization */
 /* See https://github.com/simgrid/simgrid/issues/139 */
 
@@ -84,15 +105,17 @@ static int _starpu_simgrid_xbt_thread_create_wrapper(int argc STARPU_ATTRIBUTE_U
 }
 #endif
 
-void _starpu_simgrid_xbt_thread_create(const char *name, void_f_pvoid_t code, void *param)
+void _starpu_simgrid_xbt_thread_create(const char *name, starpu_pthread_attr_t *attr, void_f_pvoid_t code, void *param)
 {
 #if SIMGRID_VERSION >= 32501
 	starpu_pthread_t t;
 	thread_data_t *res = (thread_data_t *) malloc(sizeof(thread_data_t));
 	res->userparam = param;
 	res->code = code;
-	starpu_pthread_create_on(name, &t, NULL, _starpu_simgrid_xbt_thread_create_wrapper, res, sg_host_self());
+	starpu_pthread_create_on(name, &t, attr, _starpu_simgrid_xbt_thread_create_wrapper, res, sg_host_self());
 #else
+	if (attr && attr->stacksize)
+		_starpu_simgrid_set_stack_size(attr->stacksize);
 #if SIMGRID_VERSION >= 32190 || defined(HAVE_SIMCALL_PROCESS_CREATE) || defined(simcall_process_create)
 #ifdef HAVE_SMX_ACTOR_T
 	smx_actor_t process STARPU_ATTRIBUTE_UNUSED;
@@ -138,6 +161,8 @@ void _starpu_simgrid_xbt_thread_create(const char *name, void_f_pvoid_t code, vo
 #else
 	STARPU_ABORT_MSG("Can't run StarPU-Simgrid-MPI with a Simgrid version which does not provide simcall_process_create and does not fix https://github.com/simgrid/simgrid/issues/139 , sorry.");
 #endif
+	if (attr && attr->stacksize)
+		_starpu_simgrid_set_stack_size(_starpu_default_stack_size);
 #endif
 }
 
