@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,8 +15,14 @@
  */
 
 #include <starpu.h>
+#ifdef BUILDING_STARPU
+#include <datawizard/memory_nodes.h>
+#endif
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
+static int map_block(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int unmap_block(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
+static int update_map_block(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
 
 static const struct starpu_data_copy_methods block_copy_data_methods_s =
 {
@@ -24,9 +30,8 @@ static const struct starpu_data_copy_methods block_copy_data_methods_s =
 };
 
 
-static void register_block_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_block_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static void *block_to_pointer(void *data_interface, unsigned node);
-static int block_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static starpu_ssize_t allocate_block_buffer_on_node(void *data_interface_, unsigned dst_node);
 static void free_block_buffer_on_node(void *data_interface, unsigned node);
 static size_t block_interface_get_size(starpu_data_handle_t handle);
@@ -34,6 +39,7 @@ static uint32_t footprint_block_interface_crc32(starpu_data_handle_t handle);
 static int block_compare(void *data_interface_a, void *data_interface_b);
 static void display_block_interface(starpu_data_handle_t handle, FILE *f);
 static int pack_block_handle(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count);
+static int peek_block_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static int unpack_block_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count);
 static starpu_ssize_t describe(void *data_interface, char *buf, size_t size);
 
@@ -42,8 +48,10 @@ struct starpu_data_interface_ops starpu_interface_block_ops =
 	.register_data_handle = register_block_handle,
 	.allocate_data_on_node = allocate_block_buffer_on_node,
 	.to_pointer = block_to_pointer,
-	.pointer_is_inside = block_pointer_is_inside,
 	.free_data_on_node = free_block_buffer_on_node,
+	.map_data = map_block,
+	.unmap_data = unmap_block,
+	.update_map = update_map_block,
 	.copy_methods = &block_copy_data_methods_s,
 	.get_size = block_interface_get_size,
 	.footprint = footprint_block_interface_crc32,
@@ -52,9 +60,13 @@ struct starpu_data_interface_ops starpu_interface_block_ops =
 	.interface_size = sizeof(struct starpu_block_interface),
 	.display = display_block_interface,
 	.pack_data = pack_block_handle,
+	.peek_data = peek_block_handle,
 	.unpack_data = unpack_block_handle,
 	.describe = describe,
-	.name = "STARPU_BLOCK_INTERFACE"
+	.name = "STARPU_BLOCK_INTERFACE",
+	.pack_meta = NULL,
+	.unpack_meta = NULL,
+	.free_meta = NULL
 };
 
 static void *block_to_pointer(void *data_interface, unsigned node)
@@ -65,26 +77,11 @@ static void *block_to_pointer(void *data_interface, unsigned node)
 	return (void*) block_interface->ptr;
 }
 
-static int block_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
-{
-	(void) node;
-	struct starpu_block_interface *block_interface = data_interface;
-	uint32_t ldy = block_interface->ldy;
-	uint32_t ldz = block_interface->ldz;
-	uint32_t nx = block_interface->nx;
-	uint32_t ny = block_interface->ny;
-	uint32_t nz = block_interface->nz;
-	size_t elemsize = block_interface->elemsize;
-
-	return (char*) ptr >= (char*) block_interface->ptr &&
-		(char*) ptr < (char*) block_interface->ptr + (nz-1)*ldz*elemsize + (ny-1)*ldy*elemsize + nx*elemsize;
-}
-
-static void register_block_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_block_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_block_interface *block_interface = (struct starpu_block_interface *) data_interface;
 
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_block_interface *local_interface = (struct starpu_block_interface *)
@@ -93,16 +90,16 @@ static void register_block_handle(starpu_data_handle_t handle, unsigned home_nod
 		if (node == home_node)
 		{
 			local_interface->ptr = block_interface->ptr;
-                        local_interface->dev_handle = block_interface->dev_handle;
-                        local_interface->offset = block_interface->offset;
+			local_interface->dev_handle = block_interface->dev_handle;
+			local_interface->offset = block_interface->offset;
 			local_interface->ldy  = block_interface->ldy;
 			local_interface->ldz  = block_interface->ldz;
 		}
 		else
 		{
 			local_interface->ptr = 0;
-                        local_interface->dev_handle = 0;
-                        local_interface->offset = 0;
+			local_interface->dev_handle = 0;
+			local_interface->offset = 0;
 			local_interface->ldy  = 0;
 			local_interface->ldz  = 0;
 		}
@@ -120,12 +117,14 @@ void starpu_block_data_register(starpu_data_handle_t *handleptr, int home_node,
 				uintptr_t ptr, uint32_t ldy, uint32_t ldz, uint32_t nx,
 				uint32_t ny, uint32_t nz, size_t elemsize)
 {
+	STARPU_ASSERT_MSG(ldy >= nx, "ldy = %u should not be less than nx = %u.", ldy, nx);
+	STARPU_ASSERT_MSG(ldz/ldy >= ny, "ldz/ldy = %u/%u = %u should not be less than ny = %u.", ldz, ldy, ldz/ldy, ny);
 	struct starpu_block_interface block_interface =
 	{
 		.id = STARPU_BLOCK_INTERFACE_ID,
 		.ptr = ptr,
-                .dev_handle = ptr,
-                .offset = 0,
+		.dev_handle = ptr,
+		.offset = 0,
 		.ldy = ldy,
 		.ldz = ldz,
 		.nx = nx,
@@ -249,7 +248,7 @@ static int pack_block_handle(starpu_data_handle_t handle, unsigned node, void **
 	return 0;
 }
 
-static int unpack_block_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+static int peek_block_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
 {
 	STARPU_ASSERT(starpu_data_test_if_allocated_on_node(handle, node));
 
@@ -295,6 +294,12 @@ static int unpack_block_handle(starpu_data_handle_t handle, unsigned node, void
 		}
 	}
 
+	return 0;
+}
+
+static int unpack_block_handle(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	peek_block_handle(handle, node, ptr, count);
 	starpu_free_on_node_flags(node, (uintptr_t)ptr, count, 0);
 
 	return 0;
@@ -463,6 +468,51 @@ static void free_block_buffer_on_node(void *data_interface, unsigned node)
 	size_t elemsize = block_interface->elemsize;
 
 	starpu_free_on_node(node, block_interface->dev_handle, nx*ny*nz*elemsize);
+	block_interface->ptr = 0;
+	block_interface->dev_handle = 0;
+}
+
+static int map_block(void *src_interface, unsigned src_node,
+		     void *dst_interface, unsigned dst_node)
+{
+	struct starpu_block_interface *src_block = src_interface;
+	struct starpu_block_interface *dst_block = dst_interface;
+	int ret;
+	uintptr_t mapped;
+	/* map area ldz*(nz-1)+ldy*(ny-1)+nx */
+	mapped = starpu_interface_map(src_block->dev_handle, src_block->offset, src_node, dst_node, (src_block->ldz*(src_block->nz-1)+src_block->ldy*(src_block->ny-1)+src_block->nx)*src_block->elemsize, &ret);
+	if (mapped)
+	{
+		dst_block->dev_handle = mapped;
+		dst_block->offset = 0;
+		if (starpu_node_get_kind(dst_node) != STARPU_OPENCL_RAM)
+			dst_block->ptr = mapped;
+		dst_block->ldy = src_block->ldy;
+		dst_block->ldz = src_block->ldz;
+		return 0;
+	}
+	return ret;
+}
+
+static int unmap_block(void *src_interface, unsigned src_node,
+		       void *dst_interface, unsigned dst_node)
+{
+	struct starpu_block_interface *src_block = src_interface;
+	struct starpu_block_interface *dst_block = dst_interface;
+
+	int ret = starpu_interface_unmap(src_block->dev_handle, src_block->offset, src_node, dst_block->dev_handle, dst_node, (src_block->ldz*(src_block->nz-1)+src_block->ldy*(src_block->ny-1)+src_block->nx)*src_block->elemsize);
+	dst_block->dev_handle = 0;
+
+	return ret;
+}
+
+static int update_map_block(void *src_interface, unsigned src_node,
+			    void *dst_interface, unsigned dst_node)
+{
+	struct starpu_block_interface *src_block = src_interface;
+	struct starpu_block_interface *dst_block = dst_interface;
+
+	return starpu_interface_update_map(src_block->dev_handle, src_block->offset, src_node, dst_block->dev_handle, dst_block->offset, dst_node, (src_block->ldz*(src_block->nz-1)+src_block->ldy*(src_block->ny-1)+src_block->nx)*src_block->elemsize);
 }
 
 static int copy_any_to_any(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data)
