@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2009-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,7 @@
 #include <datawizard/memstats.h>
 #include <datawizard/malloc.h>
 #include <core/dependencies/data_concurrency.h>
-#include <common/uthash.h>
+#include <common/knobs.h>
 #include <common/starpu_spinlock.h>
 #include <core/task.h>
 #include <core/workers.h>
@@ -30,85 +30,45 @@
 #include <util/openmp_runtime_support.h>
 #endif
 
-/* Entry in the `registered_handles' hash table.  */
-struct handle_entry
-{
-	UT_hash_handle hh;
-	void *pointer;
-	starpu_data_handle_t handle;
-};
+static struct starpu_data_interface_ops **_id_to_ops_array;
+static unsigned _id_to_ops_array_size;
 
 /* Hash table mapping host pointers to data handles.  */
-static int nregistered, maxnregistered;
-static struct handle_entry *registered_handles;
-static struct _starpu_spinlock    registered_handles_lock;
+static int32_t nregistered, maxnregistered;
 static int _data_interface_number = STARPU_MAX_INTERFACE_ID;
 starpu_arbiter_t _starpu_global_arbiter;
+static int max_memory_use;
 
 static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned coherent, unsigned nowait);
 
+void _starpu_data_interface_fini(void);
+
 void _starpu_data_interface_init(void)
 {
-	_starpu_spin_init(&registered_handles_lock);
+	max_memory_use = starpu_getenv_number_default("STARPU_MAX_MEMORY_USE", 0);
 
 	/* Just for testing purpose */
-	if (starpu_get_env_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
+	if (starpu_getenv_number_default("STARPU_GLOBAL_ARBITER", 0) > 0)
 		_starpu_global_arbiter = starpu_arbiter_create();
+
+	_starpu_crash_add_hook(&_starpu_data_interface_fini);
+}
+
+void _starpu_data_interface_fini(void)
+{
+	if (max_memory_use)
+		_STARPU_DISP("Memory used for %d data handles: %lu MiB\n", maxnregistered, (unsigned long) (maxnregistered * sizeof(struct _starpu_data_state)) >> 20);
 }
 
 void _starpu_data_interface_shutdown()
 {
-	struct handle_entry *entry=NULL, *tmp=NULL;
+	free(_id_to_ops_array);
+	_id_to_ops_array = NULL;
+	_id_to_ops_array_size = 0;
 
-	if (registered_handles)
-	{
-		_STARPU_DISP("[warning] The application has not unregistered all data handles.\n");
-	}
-
-	_starpu_spin_destroy(&registered_handles_lock);
-
-	HASH_ITER(hh, registered_handles, entry, tmp)
-	{
-		HASH_DEL(registered_handles, entry);
-		nregistered--;
-		free(entry);
-	}
-
-	if (starpu_get_env_number_default("STARPU_MAX_MEMORY_USE", 0))
-		_STARPU_DISP("Memory used for %d data handles: %lu MiB\n", maxnregistered, (unsigned long) (maxnregistered * sizeof(struct _starpu_data_state)) >> 20);
-
-	STARPU_ASSERT(nregistered == 0);
-	registered_handles = NULL;
+	_starpu_data_interface_fini();
 }
 
-#ifdef STARPU_OPENMP
-void _starpu_omp_unregister_region_handles(struct starpu_omp_region *region)
-{
-	_starpu_spin_lock(&region->registered_handles_lock);
-	struct handle_entry *entry=NULL, *tmp=NULL;
-	HASH_ITER(hh, (region->registered_handles), entry, tmp)
-	{
-		entry->handle->removed_from_context_hash = 1;
-		HASH_DEL(region->registered_handles, entry);
-		starpu_data_unregister(entry->handle);
-		free(entry);
-	}
-	_starpu_spin_unlock(&region->registered_handles_lock);
-}
-
-void _starpu_omp_unregister_task_handles(struct starpu_omp_task *task)
-{
-	struct handle_entry *entry=NULL, *tmp=NULL;
-	HASH_ITER(hh, task->registered_handles, entry, tmp)
-	{
-		entry->handle->removed_from_context_hash = 1;
-		HASH_DEL(task->registered_handles, entry);
-		starpu_data_unregister(entry->handle);
-		free(entry);
-	}
-}
-#endif
-
 struct starpu_data_interface_ops *_starpu_data_interface_get_ops(unsigned interface_id)
 {
 	switch (interface_id)
@@ -137,195 +97,47 @@ struct starpu_data_interface_ops *_starpu_data_interface_get_ops(unsigned interf
 		case STARPU_MULTIFORMAT_INTERFACE_ID:
 			return &starpu_interface_multiformat_ops;
 
+		case STARPU_COO_INTERFACE_ID:
+			return &starpu_interface_coo_ops;
+
+		case STARPU_TENSOR_INTERFACE_ID:
+			return &starpu_interface_tensor_ops;
+
+		case STARPU_NDIM_INTERFACE_ID:
+			return &starpu_interface_ndim_ops;
+
 		default:
-			STARPU_ABORT();
-			return NULL;
-	}
-}
-
-/* Register the mapping from PTR to HANDLE.  If PTR is already mapped to
- * some handle, the new mapping shadows the previous one.   */
-void _starpu_data_register_ram_pointer(starpu_data_handle_t handle, void *ptr)
-{
-	struct handle_entry *entry;
-
-	_STARPU_MALLOC(entry, sizeof(*entry));
-
-	entry->pointer = ptr;
-	entry->handle = handle;
-
-#ifdef STARPU_OPENMP
-	struct starpu_omp_task *task = _starpu_omp_get_task();
-	if (task)
-	{
-		if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
 		{
-			struct starpu_omp_region *parallel_region = task->owner_region;
-			_starpu_spin_lock(&parallel_region->registered_handles_lock);
-			HASH_ADD_PTR(parallel_region->registered_handles, pointer, entry);
-			_starpu_spin_unlock(&parallel_region->registered_handles_lock);
-		}
-		else
-		{
-			HASH_ADD_PTR(task->registered_handles, pointer, entry);
-		}
-	}
-	else
-#endif
-	{
-		struct handle_entry *old_entry;
-
-		_starpu_spin_lock(&registered_handles_lock);
-		HASH_FIND_PTR(registered_handles, &ptr, old_entry);
-		if (old_entry)
-		{
-			/* Already registered this pointer, avoid undefined
-			 * behavior of duplicate in hash table */
-			_starpu_spin_unlock(&registered_handles_lock);
-			free(entry);
-		}
-		else
-		{
-			nregistered++;
-			if (nregistered > maxnregistered)
-				maxnregistered = nregistered;
-			HASH_ADD_PTR(registered_handles, pointer, entry);
-			_starpu_spin_unlock(&registered_handles_lock);
-		}
-	}
-}
-
-starpu_data_handle_t starpu_data_lookup(const void *ptr)
-{
-	starpu_data_handle_t result;
-
-#ifdef STARPU_OPENMP
-	struct starpu_omp_task *task = _starpu_omp_get_task();
-	if (task)
-	{
-		if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
-		{
-			struct starpu_omp_region *parallel_region = task->owner_region;
-			_starpu_spin_lock(&parallel_region->registered_handles_lock);
+			if (interface_id-STARPU_MAX_INTERFACE_ID > _id_to_ops_array_size || _id_to_ops_array == NULL || _id_to_ops_array[interface_id-STARPU_MAX_INTERFACE_ID]==NULL)
 			{
-				struct handle_entry *entry;
-
-				HASH_FIND_PTR(parallel_region->registered_handles, &ptr, entry);
-				if(STARPU_UNLIKELY(entry == NULL))
-					result = NULL;
-				else
-					result = entry->handle;
+				_STARPU_MSG("There is no 'struct starpu_data_interface_ops' registered for interface %d\n", interface_id);
+				STARPU_ABORT();
+				return NULL;
 			}
-			_starpu_spin_unlock(&parallel_region->registered_handles_lock);
-		}
-		else
-		{
-			struct handle_entry *entry;
-
-			HASH_FIND_PTR(task->registered_handles, &ptr, entry);
-			if(STARPU_UNLIKELY(entry == NULL))
-				result = NULL;
 			else
-				result = entry->handle;
+				return _id_to_ops_array[interface_id-STARPU_MAX_INTERFACE_ID];
 		}
 	}
-	else
-#endif
-	{
-		_starpu_spin_lock(&registered_handles_lock);
-		{
-			struct handle_entry *entry;
-
-			HASH_FIND_PTR(registered_handles, &ptr, entry);
-			if(STARPU_UNLIKELY(entry == NULL))
-				result = NULL;
-			else
-				result = entry->handle;
-		}
-		_starpu_spin_unlock(&registered_handles_lock);
-	}
-
-	return result;
 }
 
 /*
  * Start monitoring a piece of data
  */
-
-static void _starpu_register_new_data(starpu_data_handle_t handle,
-					int home_node, uint32_t wt_mask)
+static void _starpu_register_new_data(starpu_data_handle_t handle, int home_node, uint32_t wt_mask)
 {
-	void *ptr;
-
 	STARPU_ASSERT(handle);
 
-	/* initialize the new lock */
-	_starpu_data_requester_prio_list_init(&handle->req_list);
-	handle->refcnt = 0;
-	handle->unlocking_reqs = 0;
-	handle->busy_count = 0;
-	handle->busy_waiting = 0;
-	STARPU_PTHREAD_MUTEX_INIT(&handle->busy_mutex, NULL);
-	STARPU_PTHREAD_COND_INIT(&handle->busy_cond, NULL);
-	_starpu_spin_init(&handle->header_lock);
-
 	/* first take care to properly lock the data */
 	_starpu_spin_lock(&handle->header_lock);
 
-	/* there is no hierarchy yet */
-	handle->nchildren = 0;
-	handle->nplans = 0;
-	handle->switch_cl = NULL;
-	handle->partitioned = 0;
-	handle->readonly = 0;
-	handle->active = 1;
-	handle->active_ro = 0;
 	handle->root_handle = handle;
-	handle->father_handle = NULL;
-	handle->active_children = NULL;
-	handle->active_readonly_children = NULL;
-	handle->nactive_readonly_children = 0;
-	handle->nsiblings = 0;
-	handle->siblings = NULL;
-	handle->sibling_index = 0; /* could be anything for the root */
+	//handle->father_handle = NULL;
+	//handle->nsiblings = 0;
+	//handle->siblings = NULL;
+	//handle->sibling_index = 0; /* could be anything for the root */
 	handle->depth = 1; /* the tree is just a node yet */
-        handle->mpi_data = NULL; /* invalid until set */
 
-	handle->is_not_important = 0;
-
-	handle->sequential_consistency =
-		starpu_data_get_default_sequential_consistency_flag();
-	handle->initialized = home_node != -1;
-	handle->ooc = 1;
-
-	STARPU_PTHREAD_MUTEX_INIT(&handle->sequential_consistency_mutex, NULL);
-	handle->last_submitted_mode = STARPU_R;
-	handle->last_sync_task = NULL;
-	handle->last_submitted_accessors.task = NULL;
-	handle->last_submitted_accessors.next = &handle->last_submitted_accessors;
-	handle->last_submitted_accessors.prev = &handle->last_submitted_accessors;
-	handle->post_sync_tasks = NULL;
-
-	/* Tell helgrind that the race in _starpu_unlock_post_sync_tasks is fine */
-	STARPU_HG_DISABLE_CHECKING(handle->post_sync_tasks_cnt);
-	handle->post_sync_tasks_cnt = 0;
-
-	/* By default, there are no methods available to perform a reduction */
-	handle->redux_cl = NULL;
-	handle->init_cl = NULL;
-
-	handle->reduction_refcnt = 0;
-	_starpu_data_requester_prio_list_init(&handle->reduction_req_list);
-	handle->reduction_tmp_handles = NULL;
-	handle->write_invalidation_req = NULL;
-
-#ifdef STARPU_USE_FXT
-	handle->last_submitted_ghost_sync_id_is_valid = 0;
-	handle->last_submitted_ghost_sync_id = 0;
-	handle->last_submitted_ghost_accessors_id = NULL;
-#endif
-
-	handle->wt_mask = wt_mask;
+	handle->active = 1;
 
 	/* Store some values directly in the handle not to recompute them all
 	 * the time. */
@@ -333,13 +145,22 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 
 	handle->home_node = home_node;
 
-	if (_starpu_global_arbiter)
-		/* Just for testing purpose */
-		starpu_data_assign_arbiter(handle, _starpu_global_arbiter);
-	else
-		handle->arbiter = NULL;
-	_starpu_data_requester_prio_list_init(&handle->arbitered_req_list);
-	handle->last_locality = -1;
+	handle->wt_mask = wt_mask;
+
+	//handle->aliases = 0;
+	//handle->readonly_dup = NULL;
+	//handle->readonly_dup_of = NULL;
+
+	//handle->is_not_important = 0;
+
+	handle->sequential_consistency = starpu_data_get_default_sequential_consistency_flag();
+	handle->initialized = home_node != -1;
+	//handle->readonly = 0;
+	handle->ooc = 1;
+
+	/* By default, there are no methods available to perform a reduction */
+	//handle->redux_cl = NULL;
+	//handle->init_cl = NULL;
 
 	/* that new data is invalid from all nodes perpective except for the
 	 * home node */
@@ -350,45 +171,36 @@ static void _starpu_register_new_data(starpu_data_handle_t handle,
 		replicate = &handle->per_node[node];
 
 		replicate->memory_node = node;
-		replicate->relaxed_coherency = 0;
-		replicate->refcnt = 0;
+		//replicate->relaxed_coherency = 0;
+		//replicate->refcnt = 0;
+		//replicate->nb_tasks_prefetch = 0;
 
 		if ((int) node == home_node)
 		{
 			/* this is the home node with the only valid copy */
 			replicate->state = STARPU_OWNER;
 			replicate->allocated = 1;
-			replicate->automatically_allocated = 0;
+			//replicate->automatically_allocated = 0;
 			replicate->initialized = 1;
 		}
 		else
 		{
 			/* the value is not available here yet */
 			replicate->state = STARPU_INVALID;
-			replicate->allocated = 0;
-			replicate->initialized = 0;
+			//replicate->allocated = 0;
+			//replicate->initialized = 0;
 		}
-	}
 
-	handle->per_worker = NULL;
-	handle->user_data = NULL;
+		replicate->mapped = STARPU_UNMAPPED;
+	}
 
 	/* now the data is available ! */
 	_starpu_spin_unlock(&handle->header_lock);
-
-	for (node = 0; node < STARPU_MAXNODES; node++)
-	{
-		if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
-			continue;
-
-		ptr = starpu_data_handle_to_pointer(handle, node);
-		if (ptr != NULL)
-			_starpu_data_register_ram_pointer(handle, ptr);
-	}
+	(void)STARPU_ATOMIC_ADD(&nregistered, 1);
+	_starpu_perf_counter_update_max_int32(&maxnregistered, nregistered);
 }
 
-void
-_starpu_data_initialize_per_worker(starpu_data_handle_t handle)
+void _starpu_data_initialize_per_worker(starpu_data_handle_t handle)
 {
 	unsigned worker;
 	unsigned nworkers = starpu_worker_get_count();
@@ -402,24 +214,27 @@ _starpu_data_initialize_per_worker(starpu_data_handle_t handle)
 	for (worker = 0; worker < nworkers; worker++)
 	{
 		struct _starpu_data_replicate *replicate;
-		unsigned node;
+		//unsigned node;
 		replicate = &handle->per_worker[worker];
-		replicate->allocated = 0;
-		replicate->automatically_allocated = 0;
+		//replicate->allocated = 0;
+		//replicate->automatically_allocated = 0;
 		replicate->state = STARPU_INVALID;
-		replicate->refcnt = 0;
+		//replicate->refcnt = 0;
 		replicate->handle = handle;
 		//replicate->nb_tasks_prefetch = 0;
 
-		for (node = 0; node < STARPU_MAXNODES; node++)
-		{
-			replicate->request[node] = NULL;
-		}
+		//for (node = 0; node < STARPU_MAXNODES; node++)
+		//{
+		//	replicate->request[node] = NULL;
+		//	replicate->last_request[node] = NULL;
+		//}
+		//replicate->load_request = NULL;
 
 		/* Assuming being used for SCRATCH for now, patched when entering REDUX mode */
 		replicate->relaxed_coherency = 1;
-		replicate->initialized = 0;
+		//replicate->initialized = 0;
 		replicate->memory_node = starpu_worker_get_memory_node(worker);
+		replicate->mapped = STARPU_UNMAPPED;
 
 		_STARPU_CALLOC(replicate->data_interface, 1, interfacesize);
 		/* duplicate  the content of the interface on node 0 */
@@ -447,13 +262,52 @@ int _starpu_data_handle_init(starpu_data_handle_t handle, struct starpu_data_int
 	STARPU_HG_DISABLE_CHECKING(handle->busy_count);
 
 	handle->magic = 42;
-	handle->ops = interface_ops;
-	handle->mf_node = mf_node;
-	handle->mpi_data = NULL;
 
+	/* When not specified, the fields are initialized in _starpu_register_new_data and _starpu_data_partition */
+
+	_starpu_data_requester_prio_list_init0(&handle->req_list);
+	//handle->refcnt = 0;
+	//handle->unlocking_reqs = 0;
+	//handle->current_mode = STARPU_NONE;
+	_starpu_spin_init(&handle->header_lock);
+
+	//handle->busy_count = 0;
+	//handle->busy_waiting = 0;
+	STARPU_PTHREAD_MUTEX_INIT0(&handle->busy_mutex, NULL);
+	STARPU_PTHREAD_COND_INIT0(&handle->busy_cond, NULL);
+#ifdef STARPU_BUBBLE
+	STARPU_PTHREAD_MUTEX_INIT0(&handle->unpartition_mutex, NULL);
+#endif
+
+	//handle->root_handle
+	//handle->father_handle
+	//handle->active_children = NULL;
+	//handle->active_nchildren = 0;
+	//handle->active_readonly_children = NULL;
+	//handle->active_readonly_nchildren = NULL;
+	//handle->nactive_readonly_children = 0;
+	//handle->nsiblings
+	//handle->siblings
+	//handle->sibling_index
+	//handle->depth
+
+	/* there is no hierarchy yet */
+	//handle->children = NULL;
+	//handle->nchildren = 0;
+	//handle->nplans = 0;
+	//handle->switch_cl = NULL;
+	//handle->switch_cl_nparts = 0;
+	//handle->partitioned = 0;
+	//handle->part_readonly = 0;
+
+	//handle->active
+	//handle->active_ro = 0;
+
+	//handle->per_node below
+
+	handle->ops = interface_ops;
 	size_t interfacesize = interface_ops->interface_size;
 
-	_starpu_memory_stats_init(handle);
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		_starpu_memory_stats_init_per_node(handle, node);
@@ -468,6 +322,77 @@ int _starpu_data_handle_init(starpu_data_handle_t handle, struct starpu_data_int
 		if (handle->ops->init) handle->ops->init(replicate->data_interface);
 	}
 
+	//handle->per_worker = NULL;
+	//handle->ops above
+
+	//handle->footprint
+
+	//handle->home_node
+	//handle->wt_mask
+	//handle->aliases = 0;
+	//handle->is_not_important
+	//handle->sequential_consistency
+	//handle->initialized
+	//handle->readonly
+	//handle->ooc
+	//handle->lazy_unregister = 0;
+	//handle->removed_from_context_hash = 0;
+
+	STARPU_PTHREAD_MUTEX_INIT0(&handle->sequential_consistency_mutex, NULL);
+
+	handle->last_submitted_mode = STARPU_R;
+	//handle->last_sync_task = NULL;
+	//handle->last_submitted_accessors.task = NULL;
+	handle->last_submitted_accessors.next = &handle->last_submitted_accessors;
+	handle->last_submitted_accessors.prev = &handle->last_submitted_accessors;
+
+#ifdef STARPU_USE_FXT
+	//handle->last_submitted_ghost_sync_id_is_valid = 0;
+	//handle->last_submitted_ghost_sync_id = 0;
+	//handle->last_submitted_ghost_accessors_id = NULL;
+#endif
+
+	//handle->post_sync_tasks = NULL;
+	/* Tell helgrind that the race in _starpu_unlock_post_sync_tasks is fine */
+	STARPU_HG_DISABLE_CHECKING(handle->post_sync_tasks_cnt);
+	//handle->post_sync_tasks_cnt = 0;
+
+	//handle->redux_cl
+	//handle->init_cl
+
+	//handle->reduction_refcnt = 0;
+
+	_starpu_data_requester_prio_list_init0(&handle->reduction_req_list);
+
+	//handle->reduction_tmp_handles = NULL;
+
+	//handle->write_invalidation_req = NULL;
+
+	//handle->mpi_data = NULL; /* invalid until set */
+
+	_starpu_memory_stats_init(handle);
+
+	handle->mf_node = mf_node;
+
+	//handle->unregister_hook = NULL;
+
+	if (_starpu_global_arbiter)
+		/* Just for testing purpose */
+		starpu_data_assign_arbiter(handle, _starpu_global_arbiter);
+	else
+	{
+		//handle->arbiter = NULL;
+	}
+	_starpu_data_requester_prio_list_init0(&handle->arbitered_req_list);
+
+	handle->last_locality = -1;
+
+	//handle->dimensions = 0;
+	//handle->coordinates = {};
+
+	//handle->user_data = NULL;
+	//handle->sched_data = NULL;
+
 	return 0;
 }
 
@@ -480,22 +405,56 @@ starpu_data_handle_t _starpu_data_handle_allocate(struct starpu_data_interface_o
 	return handle;
 }
 
-void starpu_data_register(starpu_data_handle_t *handleptr, int home_node,
-			  void *data_interface,
-			  struct starpu_data_interface_ops *ops)
+void _starpu_data_register_ops(struct starpu_data_interface_ops *ops)
+{
+	/* check the interfaceid is set */
+	STARPU_ASSERT(ops->interfaceid != STARPU_UNKNOWN_INTERFACE_ID);
+
+	if ((unsigned)ops->interfaceid >= STARPU_MAX_INTERFACE_ID)
+	{
+		if ((unsigned)ops->interfaceid > _id_to_ops_array_size)
+		{
+			if (!_id_to_ops_array_size)
+			{
+				_id_to_ops_array_size = 16;
+			}
+			else
+			{
+				_id_to_ops_array_size *= 2;
+			}
+			_STARPU_REALLOC(_id_to_ops_array, _id_to_ops_array_size * sizeof(struct starpu_data_interface_ops *));
+		}
+		_id_to_ops_array[ops->interfaceid-STARPU_MAX_INTERFACE_ID] = ops;
+	}
+}
+
+void starpu_data_register_ops(struct starpu_data_interface_ops *ops)
+{
+	if (ops->interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
+	{
+		ops->interfaceid = starpu_data_interface_get_next_id();
+	}
+	_starpu_data_register_ops(ops);
+}
+
+void starpu_data_register(starpu_data_handle_t *handleptr, int home_node, void *data_interface, struct starpu_data_interface_ops *ops)
 {
 	starpu_data_handle_t handle = _starpu_data_handle_allocate(ops, home_node);
 
 	STARPU_ASSERT(handleptr);
 	*handleptr = handle;
 
-	/* check the interfaceid is set */
-	STARPU_ASSERT(ops->interfaceid != STARPU_UNKNOWN_INTERFACE_ID);
+	if (ops->interfaceid == STARPU_UNKNOWN_INTERFACE_ID)
+	{
+		ops->interfaceid = starpu_data_interface_get_next_id();
+	}
 
 	/* fill the interface fields with the appropriate method */
 	STARPU_ASSERT(ops->register_data_handle);
 	ops->register_data_handle(handle, home_node, data_interface);
 
+	_starpu_data_register_ops(ops);
+
 	_starpu_register_new_data(handle, home_node, 0);
 	_STARPU_TRACE_HANDLE_DATA_REGISTER(handle);
 }
@@ -526,20 +485,6 @@ void *starpu_data_handle_to_pointer(starpu_data_handle_t handle, unsigned node)
 	return NULL;
 }
 
-int starpu_data_pointer_is_inside(starpu_data_handle_t handle, unsigned node, void *ptr)
-{
-	/* Check whether the operation is supported and the node has actually
-	 * been allocated.  */
-	if (!starpu_data_test_if_allocated_on_node(handle, node))
-		return 0;
-	if (handle->ops->pointer_is_inside)
-	{
-		return handle->ops->pointer_is_inside(starpu_data_get_interface_on_node(handle, node), node, ptr);
-	}
-	/* Don't know :/ */
-	return -1;
-}
-
 void *starpu_data_get_local_ptr(starpu_data_handle_t handle)
 {
 	return starpu_data_handle_to_pointer(handle, starpu_worker_get_local_memory_node());
@@ -550,69 +495,6 @@ struct starpu_data_interface_ops* starpu_data_get_interface_ops(starpu_data_hand
 	return handle->ops;
 }
 
-/*
- * Stop monitoring a piece of data
- */
-void _starpu_data_unregister_ram_pointer(starpu_data_handle_t handle, unsigned node)
-{
-	if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
-		return;
-
-#ifdef STARPU_OPENMP
-	if (handle->removed_from_context_hash)
-		return;
-#endif
-	const void *ram_ptr = starpu_data_handle_to_pointer(handle, node);
-
-	if (ram_ptr != NULL)
-	{
-		/* Remove the PTR -> HANDLE mapping.  If a mapping from PTR
-		 * to another handle existed before (e.g., when using
-		 * filters), it becomes visible again.  */
-		struct handle_entry *entry;
-#ifdef STARPU_OPENMP
-		struct starpu_omp_task *task = _starpu_omp_get_task();
-		if (task)
-		{
-			if (task->flags & STARPU_OMP_TASK_FLAGS_IMPLICIT)
-			{
-				struct starpu_omp_region *parallel_region = task->owner_region;
-				_starpu_spin_lock(&parallel_region->registered_handles_lock);
-				HASH_FIND_PTR(parallel_region->registered_handles, &ram_ptr, entry);
-				STARPU_ASSERT(entry != NULL);
-				HASH_DEL(registered_handles, entry);
-				_starpu_spin_unlock(&parallel_region->registered_handles_lock);
-			}
-			else
-			{
-				HASH_FIND_PTR(task->registered_handles, &ram_ptr, entry);
-				STARPU_ASSERT(entry != NULL);
-				HASH_DEL(task->registered_handles, entry);
-			}
-		}
-		else
-#endif
-		{
-
-			_starpu_spin_lock(&registered_handles_lock);
-			HASH_FIND_PTR(registered_handles, &ram_ptr, entry);
-			if (entry)
-			{
-				if (entry->handle == handle)
-				{
-					nregistered--;
-					HASH_DEL(registered_handles, entry);
-				}
-				else
-					/* don't free it, it's not ours */
-					entry = NULL;
-			}
-			_starpu_spin_unlock(&registered_handles_lock);
-		}
-		free(entry);
-	}
-}
-
 void _starpu_data_free_interfaces(starpu_data_handle_t handle)
 {
 	unsigned node;
@@ -686,24 +568,27 @@ void _starpu_check_if_valid_and_fetch_data_on_node(starpu_data_handle_t handle,
 	unsigned nnodes = starpu_memory_nodes_get_count();
 	int valid = 0;
 
+	_starpu_spin_lock(&handle->header_lock);
 	for (node = 0; node < nnodes; node++)
 	{
 		if (handle->per_node[node].state != STARPU_INVALID)
 		{
 			/* we found a copy ! */
 			valid = 1;
+			break;
 		}
 	}
+	_starpu_spin_unlock(&handle->header_lock);
 	if (valid)
 	{
-		int ret = _starpu_fetch_data_on_node(handle, handle->home_node, replicate, STARPU_R, 0, STARPU_FETCH, 0, NULL, NULL, 0, origin);
+		int ret = _starpu_fetch_data_on_node(handle, handle->home_node, replicate, STARPU_R, 0, NULL, STARPU_FETCH, 0, NULL, NULL, 0, origin);
 		STARPU_ASSERT(!ret);
-		_starpu_release_data_on_node(handle, 0, replicate);
+		_starpu_release_data_on_node(handle, 0, STARPU_NONE, replicate);
 	}
 	else
 	{
 		_starpu_spin_lock(&handle->header_lock);
-		if (!_starpu_notify_data_dependencies(handle))
+		if (!_starpu_notify_data_dependencies(handle, STARPU_NONE))
 			_starpu_spin_unlock(&handle->header_lock);
 	}
 }
@@ -729,10 +614,43 @@ static void _starpu_data_unregister_fetch_data_callback(void *_arg)
 
 void _starpu_data_set_unregister_hook(starpu_data_handle_t handle, _starpu_data_handle_unregister_hook func)
 {
- 	STARPU_ASSERT(handle->unregister_hook == NULL);
+	STARPU_ASSERT(handle->unregister_hook == NULL);
 	handle->unregister_hook = func;
 }
 
+/*
+ * We are about to unregister this R/O data. There might be still other aliases,
+ * in which case this returns 0. If not, users are not supposed to see it
+ * any more, so detach it from their sight and return 1 to let unregistration happen.
+ */
+static int _starpu_ro_data_detach(starpu_data_handle_t handle)
+{
+	_starpu_spin_lock(&handle->header_lock);
+	if (handle->aliases)
+	{
+		handle->aliases--;
+		_starpu_spin_unlock(&handle->header_lock);
+		return 0;
+	}
+	if (handle->readonly_dup)
+	{
+		STARPU_ASSERT(handle->readonly_dup->readonly_dup_of == handle);
+		handle->readonly_dup->readonly_dup_of = NULL;
+		handle->readonly_dup = NULL;
+	}
+	if (handle->readonly_dup_of)
+	{
+		STARPU_ASSERT(handle->readonly_dup_of->readonly_dup == handle);
+		handle->readonly_dup_of->readonly_dup = NULL;
+		handle->readonly_dup_of = NULL;
+	}
+	/* So that unregistration can use write dependencies to wait for
+	 * anything to finish */
+	handle->readonly = 0;
+	_starpu_spin_unlock(&handle->header_lock);
+	return 1;
+}
+
 /* Unregister the data handle, perhaps we don't need to update the home_node
  * (in that case coherent is set to 0)
  * nowait is for internal use when we already know for sure that we won't have to wait.
@@ -746,6 +664,9 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 	/* TODO: also check that it has the latest coherency */
 	STARPU_ASSERT(!(nowait && handle->busy_count != 0));
 
+	if (!_starpu_ro_data_detach(handle))
+		return;
+
 	int sequential_consistency = handle->sequential_consistency;
 	if (sequential_consistency && !nowait)
 	{
@@ -757,8 +678,8 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 
 		/* If sequential consistency is enabled, wait until data is available */
 		if ((handle->nplans && !handle->nchildren) || handle->siblings)
-			_starpu_data_partition_access_submit(handle, 1);
-		_starpu_data_wait_until_available(handle, STARPU_RW, "starpu_data_unregister");
+			_starpu_data_partition_access_submit(handle, !handle->readonly);
+		_starpu_data_wait_until_available(handle, handle->readonly?STARPU_R:STARPU_RW, "starpu_data_unregister");
 	}
 
 	if (coherent && !nowait)
@@ -770,12 +691,12 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 		int home_node = handle->home_node;
 		if (home_node >= 0)
 		{
-			struct _starpu_unregister_callback_arg arg;
+			struct _starpu_unregister_callback_arg arg = { 0 };
 			arg.handle = handle;
 			arg.memory_node = (unsigned)home_node;
 			arg.terminated = 0;
-			STARPU_PTHREAD_MUTEX_INIT(&arg.mutex, NULL);
-			STARPU_PTHREAD_COND_INIT(&arg.cond, NULL);
+			STARPU_PTHREAD_MUTEX_INIT0(&arg.mutex, NULL);
+			STARPU_PTHREAD_COND_INIT0(&arg.cond, NULL);
 
 			if (!_starpu_attempt_to_submit_data_request_from_apps(handle, STARPU_R,
 					_starpu_data_unregister_fetch_data_callback, &arg))
@@ -795,6 +716,8 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 			STARPU_PTHREAD_COND_DESTROY(&arg.cond);
 		}
 
+		/* Driver porters: adding your driver here is optional, only needed for the support of multiple formats.  */
+
 		/* If this handle uses a multiformat interface, we may have to convert
 		 * this piece of data back into the CPU format.
 		 * XXX : This is quite hacky, could we submit a task instead ?
@@ -830,15 +753,6 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 					cl = mf_ops->opencl_to_cpu_cl;
 					break;
 				}
-#endif
-#ifdef STARPU_USE_MIC
-				case STARPU_MIC_RAM:
-				{
-					struct starpu_multiformat_data_interface_ops *mf_ops;
-					mf_ops = (struct starpu_multiformat_data_interface_ops *) handle->ops->get_mf_ops(format_interface);
-					cl = mf_ops->mic_to_cpu_cl;
-					break;
-				}
 #endif
 				case STARPU_CPU_RAM:      /* Impossible ! */
 				default:
@@ -859,10 +773,14 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 	if (!coherent)
 	{
 		/* Should we postpone the unregister operation ? */
-		if ((handle->busy_count > 0) && handle->lazy_unregister)
+		if (handle->lazy_unregister)
 		{
-			_starpu_spin_unlock(&handle->header_lock);
-			return;
+			if (handle->busy_count > 0)
+			{
+				_starpu_spin_unlock(&handle->header_lock);
+				return;
+			}
+			handle->lazy_unregister = 0;
 		}
 	}
 
@@ -870,6 +788,11 @@ static void _starpu_data_unregister(starpu_data_handle_t handle, unsigned cohere
 	handle->busy_waiting = 1;
 	_starpu_spin_unlock(&handle->header_lock);
 
+	/* Request unmapping of any mapped data */
+	unsigned node;
+	for (node = 0; node < STARPU_MAXNODES; node++)
+		_starpu_data_unmap(handle, node);
+
 retry_busy:
 	/* Wait for all requests to finish (notably WT requests) */
 	STARPU_PTHREAD_MUTEX_LOCK(&handle->busy_mutex);
@@ -887,7 +810,7 @@ retry_busy:
 	}
 	STARPU_PTHREAD_MUTEX_UNLOCK(&handle->busy_mutex);
 
-	/* Unregister MPI things after having waiting for MPI reqs etc. to settle down */
+	/* Unregister MPI things after having waited for MPI reqs etc. to settle down */
 	if (handle->unregister_hook)
 	{
 		handle->unregister_hook(handle);
@@ -906,15 +829,12 @@ retry_busy:
 	size_t size = _starpu_data_get_alloc_size(handle);
 
 	/* Destroy the data now */
-	unsigned node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct _starpu_data_replicate *local = &handle->per_node[node];
 		STARPU_ASSERT(!local->refcnt);
 		if (local->allocated)
 		{
-			_starpu_data_unregister_ram_pointer(handle, node);
-
 		/* free the data copy in a lazy fashion */
 			if (local->automatically_allocated)
 				_starpu_request_mem_chunk_removal(handle, local, node, size);
@@ -942,14 +862,21 @@ retry_busy:
 
 	_starpu_data_clear_implicit(handle);
 	free(handle->active_readonly_children);
+	free(handle->active_readonly_nchildren);
 
 	STARPU_PTHREAD_MUTEX_DESTROY(&handle->busy_mutex);
 	STARPU_PTHREAD_COND_DESTROY(&handle->busy_cond);
 	STARPU_PTHREAD_MUTEX_DESTROY(&handle->sequential_consistency_mutex);
+#ifdef STARPU_BUBBLE
+	STARPU_PTHREAD_MUTEX_DESTROY(&handle->unpartition_mutex);
+#endif
 
 	STARPU_HG_ENABLE_CHECKING(handle->post_sync_tasks_cnt);
 	STARPU_HG_ENABLE_CHECKING(handle->busy_count);
 
+	_starpu_data_requester_prio_list_deinit(&handle->req_list);
+	_starpu_data_requester_prio_list_deinit(&handle->reduction_req_list);
+
 	if (handle->switch_cl)
 	{
 		free(handle->switch_cl->dyn_nodes);
@@ -957,6 +884,7 @@ retry_busy:
 	}
 	_STARPU_TRACE_HANDLE_DATA_UNREGISTER(handle);
 	free(handle);
+	(void)STARPU_ATOMIC_ADD(&nregistered, -1);
 }
 
 void starpu_data_unregister(starpu_data_handle_t handle)
@@ -984,7 +912,7 @@ static void _starpu_data_unregister_submit_cb(void *arg)
 	 * when we releases the handle below, it will be destroyed by
 	 * _starpu_data_check_not_busy */
 	STARPU_ASSERT(handle->busy_count);
-        _starpu_spin_unlock(&handle->header_lock);
+	_starpu_spin_unlock(&handle->header_lock);
 
 	starpu_data_release_on_node(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL);
 }
@@ -994,6 +922,9 @@ void starpu_data_unregister_submit(starpu_data_handle_t handle)
 	STARPU_ASSERT_MSG(handle->magic == 42, "data %p is invalid (was it already registered?)", handle);
 	STARPU_ASSERT_MSG(!handle->lazy_unregister, "data %p can not be unregistered twice", handle);
 
+	if (!_starpu_ro_data_detach(handle))
+		return;
+
 	/* Wait for all task dependencies on this handle before putting it for free */
 	starpu_data_acquire_on_node_cb(handle, STARPU_ACQUIRE_NO_NODE_LOCK_ALL, handle->initialized?STARPU_RW:STARPU_W, _starpu_data_unregister_submit_cb, handle);
 }
@@ -1017,21 +948,29 @@ static void _starpu_data_invalidate(void *data)
 #endif
 
 	unsigned node;
+
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct _starpu_data_replicate *local = &handle->per_node[node];
 
 		if (local->mc && local->allocated && local->automatically_allocated)
 		{
-			_starpu_data_unregister_ram_pointer(handle, node);
+			unsigned mapping;
+			for (mapping = 0; mapping < STARPU_MAXNODES; mapping++)
+				if (handle->per_node[mapping].mapped == (int) node)
+					break;
 
-			/* free the data copy in a lazy fashion */
-			_starpu_request_mem_chunk_removal(handle, local, node, size);
+			if (mapping == STARPU_MAXNODES)
+			{
+				/* free the data copy in a lazy fashion */
+				_starpu_request_mem_chunk_removal(handle, local, node, size);
+			}
 		}
 
 		if (local->state != STARPU_INVALID)
 			_STARPU_TRACE_DATA_STATE_INVALID(handle, node);
 		local->state = STARPU_INVALID;
+		local->initialized = 0;
 	}
 
 	if (handle->per_worker)
@@ -1100,18 +1039,41 @@ int starpu_data_interface_get_next_id(void)
 	return _data_interface_number-1;
 }
 
-int starpu_data_pack(starpu_data_handle_t handle, void **ptr, starpu_ssize_t *count)
+int starpu_data_pack_node(starpu_data_handle_t handle, unsigned node, void **ptr, starpu_ssize_t *count)
 {
 	STARPU_ASSERT_MSG(handle->ops->pack_data, "The datatype interface %s (%d) does not have a pack operation", handle->ops->name, handle->ops->interfaceid);
-	return handle->ops->pack_data(handle, starpu_worker_get_local_memory_node(), ptr, count);
+	return handle->ops->pack_data(handle, node, ptr, count);
+}
+
+int starpu_data_pack(starpu_data_handle_t handle, void **ptr, starpu_ssize_t *count)
+{
+	return starpu_data_pack_node(handle, starpu_worker_get_local_memory_node(), ptr, count);
+}
+
+int starpu_data_peek_node(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	STARPU_ASSERT_MSG(handle->ops->peek_data, "The datatype interface %s (%d) does not have a peek operation", handle->ops->name, handle->ops->interfaceid);
+	int ret;
+	ret = handle->ops->peek_data(handle, node, ptr, count);
+	return ret;
+}
+
+int starpu_data_peek(starpu_data_handle_t handle, void *ptr, size_t count)
+{
+	return starpu_data_peek_node(handle, starpu_worker_get_local_memory_node(), ptr, count);
+}
+
+int starpu_data_unpack_node(starpu_data_handle_t handle, unsigned node, void *ptr, size_t count)
+{
+	STARPU_ASSERT_MSG(handle->ops->unpack_data, "The datatype interface %s (%d) does not have an unpack operation", handle->ops->name, handle->ops->interfaceid);
+	int ret;
+	ret = handle->ops->unpack_data(handle, node, ptr, count);
+	return ret;
 }
 
 int starpu_data_unpack(starpu_data_handle_t handle, void *ptr, size_t count)
 {
-	STARPU_ASSERT_MSG(handle->ops->unpack_data, "The datatype interface %s (%d) does not have an unpack operation", handle->ops->name, handle->ops->interfaceid);
-	int ret;
-	ret = handle->ops->unpack_data(handle, starpu_worker_get_local_memory_node(), ptr, count);
-	return ret;
+	return starpu_data_unpack_node(handle, starpu_worker_get_local_memory_node(), ptr, count);
 }
 
 size_t starpu_data_get_size(starpu_data_handle_t handle)
@@ -1178,3 +1140,65 @@ unsigned starpu_data_get_coordinates_array(starpu_data_handle_t handle, unsigned
 
 	return dimensions;
 }
+
+void starpu_data_print(starpu_data_handle_t handle, unsigned node, FILE *stream)
+{
+	if (handle->ops == NULL)
+		fprintf(stream, "Undefined");
+	else
+	{
+		switch (handle->ops->interfaceid)
+		{
+		case(STARPU_MATRIX_INTERFACE_ID):
+			fprintf(stream, "Matrix");
+			break;
+		case(STARPU_BLOCK_INTERFACE_ID):
+			fprintf(stream, "Block");
+			break;
+		case(STARPU_VECTOR_INTERFACE_ID):
+			fprintf(stream, "Vector");
+			break;
+		case(STARPU_CSR_INTERFACE_ID):
+			fprintf(stream, "CSR");
+			break;
+		case(STARPU_BCSR_INTERFACE_ID):
+			fprintf(stream, "BCSR");
+			break;
+		case(STARPU_VARIABLE_INTERFACE_ID):
+			fprintf(stream, "Variable");
+			break;
+		case(STARPU_VOID_INTERFACE_ID):
+			fprintf(stream, "Void");
+			break;
+		case(STARPU_MULTIFORMAT_INTERFACE_ID):
+			fprintf(stream, "Multfiformat");
+			break;
+		case(STARPU_COO_INTERFACE_ID):
+			fprintf(stream, "COO");
+			break;
+		case(STARPU_TENSOR_INTERFACE_ID):
+			fprintf(stream, "Tensor");
+			break;
+		case(STARPU_UNKNOWN_INTERFACE_ID):
+			fprintf(stream, "UNKNOWN");
+			break;
+		default:
+			fprintf(stream, "User interface with id %d", handle->ops->interfaceid);
+			break;
+		}
+	}
+	void *data_interface = NULL;
+	if (starpu_data_test_if_allocated_on_node(handle, node))
+		data_interface = starpu_data_get_interface_on_node(handle, node);
+	if (starpu_data_test_if_allocated_on_node(handle, handle->home_node))
+		data_interface = starpu_data_get_interface_on_node(handle, handle->home_node);
+	if (handle->ops && handle->ops->describe && data_interface)
+	{
+		char buffer[1024];
+		handle->ops->describe(data_interface, buffer, sizeof(buffer));
+		fprintf(stream, " %s\n", buffer);
+	}
+	else
+		fprintf(stream, "\n");
+
+}
