@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011,2012                                Inria
- * Copyright (C) 2011-2017,2019                           CNRS
- * Copyright (C) 2011-2013,2015,2016,2018-2019            Université de Bordeaux
+ * Copyright (C) 2011-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +14,14 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/* Driver porters: adding your driver here is optional, only needed for the support of multiple formats.  */
+
 #include <starpu.h>
+#ifdef BUILDING_STARPU
+#include <datawizard/memory_nodes.h>
+#endif
+
+/* Driver porters: adding your driver here is optional, only needed for the support of multiple formats. */
 
 static int copy_ram_to_ram(void *src_interface, unsigned src_node STARPU_ATTRIBUTE_UNUSED, void *dst_interface, unsigned dst_node);
 #ifdef STARPU_USE_CUDA
@@ -34,12 +39,6 @@ static int copy_opencl_to_opencl(void *src_interface, unsigned src_node STARPU_A
 static int copy_ram_to_opencl_async(void *src_interface, unsigned src_node STARPU_ATTRIBUTE_UNUSED, void *dst_interface, unsigned dst_node, cl_event *event);
 static int copy_opencl_to_ram_async(void *src_interface, unsigned src_node STARPU_ATTRIBUTE_UNUSED, void *dst_interface, unsigned dst_node, cl_event *event);
 #endif
-#ifdef STARPU_USE_MIC
-static int copy_ram_to_mic(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
-static int copy_mic_to_ram(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
-static int copy_ram_to_mic_async(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
-static int copy_mic_to_ram_async(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node);
-#endif
 
 static const struct starpu_data_copy_methods multiformat_copy_data_methods_s =
 {
@@ -61,21 +60,14 @@ static const struct starpu_data_copy_methods multiformat_copy_data_methods_s =
 	.ram_to_opencl = copy_ram_to_opencl,
 	.opencl_to_ram = copy_opencl_to_ram,
 	.opencl_to_opencl = copy_opencl_to_opencl,
-        .ram_to_opencl_async = copy_ram_to_opencl_async,
+	.ram_to_opencl_async = copy_ram_to_opencl_async,
 	.opencl_to_ram_async = copy_opencl_to_ram_async,
 #endif
-#ifdef STARPU_USE_MIC
-	.ram_to_mic = copy_ram_to_mic,
-	.mic_to_ram = copy_mic_to_ram,
-	.ram_to_mic_async = copy_ram_to_mic_async,
-	.mic_to_ram_async = copy_mic_to_ram_async,
-#endif
 };
 
-static void register_multiformat_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface);
+static void register_multiformat_handle(starpu_data_handle_t handle, int home_node, void *data_interface);
 static starpu_ssize_t allocate_multiformat_buffer_on_node(void *data_interface_, unsigned dst_node);
 static void *multiformat_to_pointer(void *data_interface, unsigned node);
-static int multiformat_pointer_is_inside(void *data_interface, unsigned node, void *ptr);
 static void free_multiformat_buffer_on_node(void *data_interface, unsigned node);
 static size_t multiformat_interface_get_size(starpu_data_handle_t handle);
 static uint32_t footprint_multiformat_interface_crc32(starpu_data_handle_t handle);
@@ -97,7 +89,6 @@ struct starpu_data_interface_ops starpu_interface_multiformat_ops =
 	.register_data_handle  = register_multiformat_handle,
 	.allocate_data_on_node = allocate_multiformat_buffer_on_node,
 	.to_pointer            = multiformat_to_pointer,
-	.pointer_is_inside     = multiformat_pointer_is_inside,
 	.free_data_on_node     = free_multiformat_buffer_on_node,
 	.copy_methods          = &multiformat_copy_data_methods_s,
 	.get_size              = multiformat_interface_get_size,
@@ -125,10 +116,6 @@ static void *multiformat_to_pointer(void *data_interface, unsigned node)
 #ifdef STARPU_USE_OPENCL
 		case STARPU_OPENCL_RAM:
 			return multiformat_interface->opencl_ptr;
-#endif
-#ifdef STARPU_USE_MIC
-		case STARPU_MIC_RAM:
-			return multiformat_interface->mic_ptr;
 #endif
 		default:
 			STARPU_ABORT();
@@ -136,42 +123,12 @@ static void *multiformat_to_pointer(void *data_interface, unsigned node)
 	return NULL;
 }
 
-static int multiformat_pointer_is_inside(void *data_interface, unsigned node, void *ptr)
-{
-	struct starpu_multiformat_interface *multiformat_interface = data_interface;
-
-	switch(starpu_node_get_kind(node))
-	{
-		case STARPU_CPU_RAM:
-			return (char*) ptr >= (char*) multiformat_interface->cpu_ptr &&
-				(char*) ptr < (char*) multiformat_interface->cpu_ptr + multiformat_interface->nx * multiformat_interface->ops->cpu_elemsize;
-#ifdef STARPU_USE_CUDA
-		case STARPU_CUDA_RAM:
-			return (char*) ptr >= (char*) multiformat_interface->cuda_ptr &&
-				(char*) ptr < (char*) multiformat_interface->cuda_ptr + multiformat_interface->nx * multiformat_interface->ops->cuda_elemsize;
-#endif
-#ifdef STARPU_USE_OPENCL
-		case STARPU_OPENCL_RAM:
-			return (char*) ptr >= (char*) multiformat_interface->opencl_ptr &&
-				(char*) ptr < (char*) multiformat_interface->opencl_ptr + multiformat_interface->nx * multiformat_interface->ops->opencl_elemsize;
-#endif
-#ifdef STARPU_USE_MIC
-		case STARPU_MIC_RAM:
-			return (char*) ptr >= (char*) multiformat_interface->mic_ptr &&
-				(char*) ptr < (char*) multiformat_interface->mic_ptr + multiformat_interface->nx * multiformat_interface->ops->mic_elemsize;
-#endif
-		default:
-			STARPU_ABORT();
-	}
-	return -1;
-}
-
-static void register_multiformat_handle(starpu_data_handle_t handle, unsigned home_node, void *data_interface)
+static void register_multiformat_handle(starpu_data_handle_t handle, int home_node, void *data_interface)
 {
 	struct starpu_multiformat_interface *multiformat_interface;
 	multiformat_interface = (struct starpu_multiformat_interface *) data_interface;
 
-	unsigned node;
+	int node;
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
 		struct starpu_multiformat_interface *local_interface =
@@ -185,9 +142,6 @@ static void register_multiformat_handle(starpu_data_handle_t handle, unsigned ho
 #endif
 #ifdef STARPU_USE_OPENCL
 			local_interface->opencl_ptr = multiformat_interface->opencl_ptr;
-#endif
-#ifdef STARPU_USE_MIC
-			local_interface->mic_ptr    = multiformat_interface->mic_ptr;
 #endif
 		}
 		else
@@ -198,9 +152,6 @@ static void register_multiformat_handle(starpu_data_handle_t handle, unsigned ho
 #endif
 #ifdef STARPU_USE_OPENCL
 			local_interface->opencl_ptr = NULL;
-#endif
-#ifdef STARPU_USE_MIC
-			local_interface->mic_ptr    = NULL;
 #endif
 		}
 		local_interface->id = multiformat_interface->id;
@@ -221,7 +172,6 @@ void starpu_multiformat_data_register(starpu_data_handle_t *handleptr,
 		.cpu_ptr    = ptr,
 		.cuda_ptr   = NULL,
 		.opencl_ptr = NULL,
-		.mic_ptr    = NULL,
 		.nx         = nobjects,
 		.ops        = format_ops
 	};
@@ -246,9 +196,6 @@ static int multiformat_compare(void *data_interface_a, void *data_interface_b)
 #endif
 #ifdef STARPU_USE_OPENCL
 		&& (multiformat_a->ops->opencl_elemsize == multiformat_b->ops->opencl_elemsize)
-#endif
-#ifdef STARPU_USE_MIC
-		&& (multiformat_a->ops->mic_elemsize == multiformat_b->ops->mic_elemsize)
 #endif
 		;
 }
@@ -309,20 +256,9 @@ static starpu_ssize_t allocate_multiformat_buffer_on_node(void *data_interface_,
 		goto fail_opencl;
 	multiformat_interface->opencl_ptr = (void *) addr;
 #endif
-#ifdef STARPU_USE_MIC
-	size = multiformat_interface->nx * multiformat_interface->ops->mic_elemsize;
-	allocated_memory += size;
-	addr = starpu_malloc_on_node(dst_node, size);
-	if (!addr)
-		goto fail_mic;
-	multiformat_interface->mic_ptr = (void *) addr;
-#endif
 
 	return allocated_memory;
 
-#ifdef STARPU_USE_MIC
-fail_mic:
-#endif
 #ifdef STARPU_USE_OPENCL
 	starpu_free_on_node(dst_node, (uintptr_t) multiformat_interface->opencl_ptr, multiformat_interface->nx * multiformat_interface->ops->opencl_elemsize);
 fail_opencl:
@@ -354,21 +290,13 @@ static void free_multiformat_buffer_on_node(void *data_interface, unsigned node)
 				   multiformat_interface->nx * multiformat_interface->ops->opencl_elemsize);
 	multiformat_interface->opencl_ptr = NULL;
 #endif
-#ifdef STARPU_USE_MIC
-	starpu_free_on_node(node, (uintptr_t) multiformat_interface->mic_ptr,
-				   multiformat_interface->nx * multiformat_interface->ops->mic_elemsize);
-	multiformat_interface->mic_ptr = NULL;
-#endif
 }
 
-
-
-
 /*
  * Copy methods
  */
-static int copy_ram_to_ram(void *src_interface, unsigned src_node STARPU_ATTRIBUTE_UNUSED,
-			   void *dst_interface, unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
+static int copy_ram_to_ram(void *src_interface, unsigned src_node,
+			   void *dst_interface, unsigned dst_node)
 {
 	struct starpu_multiformat_interface *src_multiformat;
 	struct starpu_multiformat_interface *dst_multiformat;
@@ -382,6 +310,7 @@ static int copy_ram_to_ram(void *src_interface, unsigned src_node STARPU_ATTRIBU
 
 	size_t size = dst_multiformat->nx * dst_multiformat->ops->cpu_elemsize;
 	memcpy(dst_multiformat->cpu_ptr, src_multiformat->cpu_ptr, size);
+	starpu_interface_data_copy(src_node, dst_node, size);
 
 	return 0;
 }
@@ -414,7 +343,7 @@ static int copy_cuda_common(void *src_interface, unsigned src_node STARPU_ATTRIB
 			}
 			status = cudaMemcpy(dst_multiformat->cpu_ptr, src_multiformat->cpu_ptr, size, kind);
 			if (!status)
-				status = cudaThreadSynchronize();
+				status = cudaDeviceSynchronize();
 			if (STARPU_UNLIKELY(status))
 				STARPU_CUDA_REPORT_ERROR(status);
 			break;
@@ -424,7 +353,7 @@ static int copy_cuda_common(void *src_interface, unsigned src_node STARPU_ATTRIB
 			size = src_multiformat->nx * src_multiformat->ops->cuda_elemsize;
 			status = cudaMemcpy(dst_multiformat->cuda_ptr, src_multiformat->cuda_ptr, size, kind);
 			if (!status)
-				status = cudaThreadSynchronize();
+				status = cudaDeviceSynchronize();
 			if (STARPU_UNLIKELY(status))
 				STARPU_CUDA_REPORT_ERROR(status);
 
@@ -435,7 +364,7 @@ static int copy_cuda_common(void *src_interface, unsigned src_node STARPU_ATTRIB
 			size = src_multiformat->nx * src_multiformat->ops->cuda_elemsize;
 			status = cudaMemcpy(dst_multiformat->cuda_ptr, src_multiformat->cuda_ptr, size, kind);
 			if (!status)
-				status = cudaThreadSynchronize();
+				status = cudaDeviceSynchronize();
 			if (STARPU_UNLIKELY(status))
 				STARPU_CUDA_REPORT_ERROR(status);
 			break;
@@ -443,6 +372,7 @@ static int copy_cuda_common(void *src_interface, unsigned src_node STARPU_ATTRIB
 		default:
 			STARPU_ABORT();
 	}
+	starpu_interface_data_copy(src_node, dst_node, size);
 
 	return 0;
 }
@@ -469,7 +399,9 @@ static int copy_cuda_common_async(void *src_interface, unsigned src_node STARPU_
 
 	size_t size;
 	cudaError_t status;
+	double start;
 
+	starpu_interface_start_driver_copy_async(src_node, dst_node, &start);
 	switch (kind)
 	{
 		case cudaMemcpyHostToDevice:
@@ -492,9 +424,9 @@ static int copy_cuda_common_async(void *src_interface, unsigned src_node STARPU_
 		case cudaMemcpyDeviceToHost:
 		{
 			size = src_multiformat->nx * src_multiformat->ops->cuda_elemsize;
-			status = cudaMemcpy(dst_multiformat->cuda_ptr, src_multiformat->cuda_ptr, size, kind);
+			status = cudaMemcpyAsync(dst_multiformat->cuda_ptr, src_multiformat->cuda_ptr, size, kind, stream);
 			if (!status)
-				status = cudaThreadSynchronize();
+				status = cudaDeviceSynchronize();
 			if (STARPU_UNLIKELY(status))
 				STARPU_CUDA_REPORT_ERROR(status);
 
@@ -511,6 +443,7 @@ static int copy_cuda_common_async(void *src_interface, unsigned src_node STARPU_
 		default:
 			STARPU_ABORT();
 	}
+	starpu_interface_end_driver_copy_async(src_node, dst_node, start);
 
 	return 0;
 }
@@ -554,18 +487,18 @@ static int copy_cuda_peer_common(void *src_interface, unsigned src_node,
 					     size, stream);
 		starpu_interface_end_driver_copy_async(src_node, dst_node, start);
 		/* All good ! Still, returning -EAGAIN, because we will need to
-                   check the transfert completion later */
+		   check the transfert completion later */
 		if (status == cudaSuccess)
 			return -EAGAIN;
 	}
 
 	/* Either a synchronous transfert was requested, or the asynchronous one
-           failed. */
+	   failed. */
 	status = cudaMemcpyPeer(dst_multiformat->cuda_ptr, dst_dev,
 				src_multiformat->cuda_ptr, src_dev,
 				size);
 	if (!status)
-		status = cudaThreadSynchronize();
+		status = cudaDeviceSynchronize();
 	if (STARPU_UNLIKELY(status != cudaSuccess))
 		STARPU_CUDA_REPORT_ERROR(status);
 
@@ -643,10 +576,11 @@ static int copy_ram_to_opencl_async(void *src_interface, unsigned src_node,
 					       0,
 					       event,
 					       &ret);
-        if (STARPU_UNLIKELY(err))
-                STARPU_OPENCL_REPORT_ERROR(err);
+	if (STARPU_UNLIKELY(err))
+		STARPU_OPENCL_REPORT_ERROR(err);
 
-	starpu_interface_data_copy(src_node, dst_node, size);
+	if (!event)
+		starpu_interface_data_copy(src_node, dst_node, size);
 	return ret;
 }
 
@@ -683,29 +617,29 @@ static int copy_opencl_to_ram_async(void *src_interface, unsigned src_node,
 					       0,
 					       event,
 					       &ret);
-        if (STARPU_UNLIKELY(err))
-                STARPU_OPENCL_REPORT_ERROR(err);
-
-	starpu_interface_data_copy(src_node, dst_node, size);
+	if (STARPU_UNLIKELY(err))
+		STARPU_OPENCL_REPORT_ERROR(err);
 
+	if (!event)
+		starpu_interface_data_copy(src_node, dst_node, size);
 
 	return ret;
 }
 
 static int copy_ram_to_opencl(void *src_interface, unsigned src_node STARPU_ATTRIBUTE_UNUSED,
-                              void *dst_interface, unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
+			      void *dst_interface, unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
 {
-        return copy_ram_to_opencl_async(src_interface, src_node, dst_interface, dst_node, NULL);
+	return copy_ram_to_opencl_async(src_interface, src_node, dst_interface, dst_node, NULL);
 }
 
 static int copy_opencl_to_ram(void *src_interface, unsigned src_node STARPU_ATTRIBUTE_UNUSED,
 				void *dst_interface, unsigned dst_node STARPU_ATTRIBUTE_UNUSED)
 {
-        return copy_opencl_to_ram_async(src_interface, src_node, dst_interface, dst_node, NULL);
+	return copy_opencl_to_ram_async(src_interface, src_node, dst_interface, dst_node, NULL);
 }
 
 static int copy_opencl_to_opencl(void *src_interface, unsigned src_node,
-                                 void *dst_interface, unsigned dst_node)
+				 void *dst_interface, unsigned dst_node)
 {
 	(void) src_interface;
 	(void) dst_interface;
@@ -716,70 +650,3 @@ static int copy_opencl_to_opencl(void *src_interface, unsigned src_node,
 	return 0;
 }
 #endif
-
-#ifdef STARPU_USE_MIC
-static int copy_mic_common_ram_to_mic(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node,
-						   int (*copy_func)(void *, unsigned, void *, unsigned, size_t))
-{
-	struct starpu_multiformat_interface *src_multiformat = src_interface;
-	struct starpu_multiformat_interface *dst_multiformat = dst_interface;
-
-	STARPU_ASSERT(src_multiformat != NULL);
-	STARPU_ASSERT(dst_multiformat != NULL);
-	STARPU_ASSERT(dst_multiformat->ops != NULL);
-
-	size_t size = dst_multiformat->nx * dst_multiformat->ops->mic_elemsize;
-	if (src_multiformat->mic_ptr == NULL)
-	{
-		src_multiformat->mic_ptr = malloc(size);
-		if (src_multiformat->mic_ptr == NULL)
-			return -ENOMEM;
-	}
-
-	copy_func(src_multiformat->cpu_ptr, src_node, dst_multiformat->cpu_ptr, dst_node, size);
-
-	starpu_interface_data_copy(src_node, dst_node, size);
-
-	return 0;
-}
-
-static int copy_mic_common_mic_to_ram(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node,
-						   int (*copy_func)(void *, unsigned, void *, unsigned, size_t))
-{
-	struct starpu_multiformat_interface *src_multiformat = src_interface;
-	struct starpu_multiformat_interface *dst_multiformat = dst_interface;
-
-	STARPU_ASSERT(src_multiformat != NULL);
-	STARPU_ASSERT(dst_multiformat != NULL);
-	STARPU_ASSERT(dst_multiformat->ops != NULL);
-
-	size_t size = src_multiformat->nx * src_multiformat->ops->mic_elemsize;
-	copy_func(src_multiformat->mic_ptr, src_node, dst_multiformat->mic_ptr, dst_node, size);
-
-	starpu_interface_data_copy(src_node, dst_node, size);
-
-	return 0;
-}
-
-static int copy_ram_to_mic(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node)
-{
-	return copy_mic_common_ram_to_mic(src_interface, src_node, dst_interface, dst_node, _starpu_mic_copy_ram_to_mic);
-}
-
-static int copy_mic_to_ram(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node)
-{
-	return copy_mic_common_mic_to_ram(src_interface, src_node, dst_interface, dst_node, _starpu_mic_copy_mic_to_ram);
-}
-
-static int copy_ram_to_mic_async(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node)
-{
-	copy_mic_common_ram_to_mic(src_interface, src_node, dst_interface, dst_node, _starpu_mic_copy_ram_to_mic_async);
-	return -EAGAIN;
-}
-
-static int copy_mic_to_ram_async(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node)
-{
-	copy_mic_common_mic_to_ram(src_interface, src_node, dst_interface, dst_node, _starpu_mic_copy_mic_to_ram_async);
-	return -EAGAIN;
-}
-#endif
