@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012,2014,2017                      Université de Bordeaux
- * Copyright (C) 2011                                     Inria
- * Copyright (C) 2010,2011,2013,2015,2017                 CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,11 +18,15 @@
 #ifndef __DRIVER_COMMON_H__
 #define __DRIVER_COMMON_H__
 
+/** @file */
+
 #include <starpu.h>
 #include <starpu_util.h>
 #include <core/jobs.h>
 #include <common/utils.h>
 
+#pragma GCC visibility push(hidden)
+
 void _starpu_driver_start_job(struct _starpu_worker *args, struct _starpu_job *j, struct starpu_perfmodel_arch* perf_arch,
 			      int rank, int profiling);
 void _starpu_driver_end_job(struct _starpu_worker *args, struct _starpu_job *j, struct starpu_perfmodel_arch* perf_arch,
@@ -32,6 +34,17 @@ void _starpu_driver_end_job(struct _starpu_worker *args, struct _starpu_job *j,
 void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_worker *worker_args,
 					struct starpu_perfmodel_arch* perf_arch, int profiling);
 
+/** Get from the scheduler a task to be executed on the worker \p workerid */
 struct starpu_task *_starpu_get_worker_task(struct _starpu_worker *args, int workerid, unsigned memnode);
+/** Get from the scheduler tasks to be executed on the workers \p workers */
 int _starpu_get_multi_worker_task(struct _starpu_worker *workers, struct starpu_task ** tasks, int nworker, unsigned memnode);
+
+void *_starpu_map_allocate(size_t length, unsigned node);
+int _starpu_map_deallocate(void* map_addr, size_t length);
+char* _starpu_get_fdname_from_mapaddr(uintptr_t map_addr, size_t *offset, size_t length);
+void *_starpu_sink_map(char *fd_name, size_t offset, size_t length);
+int _starpu_sink_unmap(uintptr_t map_addr, size_t length);
+
+#pragma GCC visibility pop
+
 #endif // __DRIVER_COMMON_H__
