@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -41,9 +41,6 @@
 
 char *_starpu_opencl_program_dir;
 
-#define _STARPU_STRINGIFY_(x) #x
-#define _STARPU_STRINGIFY(x) _STARPU_STRINGIFY_(x)
-
 static
 int _starpu_opencl_locate_file(const char *source_file_name, char **located_file_name, char **located_dir_name)
 {
@@ -113,8 +110,7 @@ int _starpu_opencl_locate_file(const char *source_file_name, char **located_file
 	return ret;
 }
 
-cl_int starpu_opencl_load_kernel(cl_kernel *kernel, cl_command_queue *queue, struct starpu_opencl_program *opencl_programs,
-                                 const char *kernel_name, int devid)
+cl_int starpu_opencl_load_kernel(cl_kernel *kernel, cl_command_queue *queue, struct starpu_opencl_program *opencl_programs, const char *kernel_name, int devid)
 {
 	cl_int err;
 	cl_device_id device;
@@ -186,7 +182,7 @@ char *_starpu_opencl_load_program_binary(const char *filename, size_t *len)
 	struct stat statbuf;
 	FILE        *fh;
 	char        *binary;
-        int         err;
+	int         err;
 
 	fh = fopen(filename, "r");
 	if (fh == 0)
