@@ -1,9 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2010-2017                                Université de Bordeaux
- * Copyright (C) 2010-2017                                CNRS
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,6 +26,7 @@
 #include <profiling/bound.h>
 #include <core/jobs.h>
 #include <core/workers.h>
+#include <datawizard/memory_nodes.h>
 
 #ifdef STARPU_HAVE_GLPK_H
 #include <glpk.h>
@@ -126,8 +126,7 @@ static int recordprio;
 
 static starpu_pthread_mutex_t mutex = STARPU_PTHREAD_MUTEX_INITIALIZER;
 
-/* Initialization */
-void starpu_bound_start(int deps, int prio)
+static void _starpu_bound_clear(int record, int deps, int prio)
 {
 	struct bound_task_pool *tp;
 	struct bound_task *t;
@@ -145,7 +144,7 @@ void starpu_bound_start(int deps, int prio)
 	td = tag_deps;
 	tag_deps = NULL;
 
-	_starpu_bound_recording = 1;
+	_starpu_bound_recording = record;
 	recorddeps = deps;
 	recordprio = prio;
 
@@ -161,6 +160,17 @@ void starpu_bound_start(int deps, int prio)
 	while (t != NULL)
 	{
 		struct bound_task *next = t->next;
+		unsigned i,j;
+		for (i = 0; i < STARPU_NARCH; i++)
+		{
+			if (t->duration[i])
+			{
+				for (j = 0; t->duration[i][j]; j++)
+					free(t->duration[i][j]);
+				free(t->duration[i]);
+			}
+		}
+		free(t->deps);
 		free(t);
 		t = next;
 	}
@@ -173,6 +183,17 @@ void starpu_bound_start(int deps, int prio)
 	}
 }
 
+void starpu_bound_clear(void)
+{
+	_starpu_bound_clear(0, 0, 0);
+}
+
+/* Initialization */
+void starpu_bound_start(int deps, int prio)
+{
+	_starpu_bound_clear(1, deps, prio);
+}
+
 /* Whether we will include it in the computation */
 static int good_job(struct _starpu_job *j)
 {
@@ -193,7 +214,7 @@ static int good_job(struct _starpu_job *j)
 static double** initialize_arch_duration(int maxdevid, unsigned* maxncore_table)
 {
 	int devid, maxncore;
-	double **arch_model;
+	double ** arch_model;
 	_STARPU_MALLOC(arch_model, sizeof(*arch_model)*(maxdevid+1));
 	arch_model[maxdevid] = NULL;
 	for(devid=0; devid<maxdevid; devid++)
@@ -210,11 +231,9 @@ static double** initialize_arch_duration(int maxdevid, unsigned* maxncore_table)
 static void initialize_duration(struct bound_task *task)
 {
 	struct _starpu_machine_config *conf = _starpu_get_machine_config();
-	task->duration[STARPU_CPU_WORKER] = initialize_arch_duration(1,&conf->topology.nhwcpus); 
-	task->duration[STARPU_CUDA_WORKER] = initialize_arch_duration(conf->topology.nhwcudagpus,NULL); 
-	task->duration[STARPU_OPENCL_WORKER] = initialize_arch_duration(conf->topology.nhwopenclgpus,NULL); 
-	task->duration[STARPU_MIC_WORKER] = initialize_arch_duration(conf->topology.nhwmicdevices,conf->topology.nmiccores); 
-	task->duration[STARPU_SCC_WORKER] = initialize_arch_duration(conf->topology.nhwscc,NULL); 
+	enum starpu_worker_archtype type;
+	for (type = 0; type < STARPU_NARCH; type++)
+		task->duration[type] = initialize_arch_duration(conf->topology.nhwdevices[type], conf->topology.nworker[type]);
 }
 
 static struct starpu_perfmodel_device device =
@@ -238,8 +257,7 @@ static void new_task(struct _starpu_job *j)
 	if (j->bound_task)
 		return;
 
-	_STARPU_MALLOC(t, sizeof(*t));
-	memset(t, 0, sizeof(*t));
+	_STARPU_CALLOC(t, 1, sizeof(*t));
 	t->id = j->job_id;
 	t->tag_id = j->task->tag_id;
 	t->use_tag = j->task->use_tag;
@@ -257,7 +275,7 @@ static void new_task(struct _starpu_job *j)
 /* A new task was submitted, record it */
 void _starpu_bound_record(struct _starpu_job *j)
 {
-	if (!_starpu_bound_recording)
+	if (STARPU_LIKELY(!_starpu_bound_recording))
 		return;
 
 	if (!good_job(j))
@@ -377,7 +395,7 @@ static struct bound_task *find_job(unsigned long id)
 }
 
 /* Job J depends on previous job of id ID (which is already finished) */
-void _starpu_bound_job_id_dep(starpu_data_handle_t handle, struct _starpu_job *j, unsigned long id)
+void _starpu_bound_job_id_dep_size(size_t size, struct _starpu_job *j, unsigned long id)
 {
 	struct bound_task *t, *dep_t;
 	int i;
@@ -409,7 +427,7 @@ void _starpu_bound_job_id_dep(starpu_data_handle_t handle, struct _starpu_job *j
 		if (t->deps[i].dep == dep_t)
 		{
 			/* Found, just add size */
-			t->deps[i].size += _starpu_data_get_size(handle);
+			t->deps[i].size += size;
 			break;
 		}
 	if (i == t->depsn)
@@ -417,11 +435,22 @@ void _starpu_bound_job_id_dep(starpu_data_handle_t handle, struct _starpu_job *j
 		/* Not already there, add */
 		_STARPU_REALLOC(t->deps, ++t->depsn * sizeof(t->deps[0]));
 		t->deps[t->depsn-1].dep = dep_t;
-		t->deps[t->depsn-1].size = _starpu_data_get_size(handle);
+		t->deps[t->depsn-1].size = size;
 	}
 	STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
 }
 
+void _starpu_bound_job_id_dep(starpu_data_handle_t handle, struct _starpu_job *j, unsigned long id)
+{
+	if (!_starpu_bound_recording || !recorddeps)
+		return;
+
+	if (!good_job(j))
+		return;
+
+	_starpu_bound_job_id_dep_size(_starpu_data_get_size(handle), j, id);
+}
+
 void starpu_bound_stop(void)
 {
 	STARPU_PTHREAD_MUTEX_LOCK(&mutex);
@@ -444,7 +473,8 @@ static void _starpu_get_tasks_times(int nw, int nt, double *times)
 				.footprint_is_computed = 1,
 			};
 			struct starpu_perfmodel_arch* arch = starpu_worker_get_perf_archtype(w, STARPU_NMAX_SCHED_CTXS);
-			double length = _starpu_history_based_job_expected_perf(tp->cl->model, arch, &j, j.nimpl);
+			double length = _starpu_history_based_job_expected_perf(tp->cl->model, arch, &j, j.nimpl)
+			              - _starpu_history_based_job_expected_deviation(tp->cl->model, arch, &j, j.nimpl);
 			if (isnan(length))
 				times[w*nt+t] = NAN;
 			else
@@ -535,7 +565,8 @@ void starpu_bound_print_lp(FILE *output)
 				struct starpu_perfmodel_arch* arch = starpu_worker_get_perf_archtype(w, STARPU_NMAX_SCHED_CTXS);
 				if (_STARPU_IS_ZERO(t1->duration[arch->devices[0].type][arch->devices[0].devid][arch->devices[0].ncores]))
 				{
-					double length = _starpu_history_based_job_expected_perf(t1->cl->model, arch, &j,j.nimpl);
+					double length = _starpu_history_based_job_expected_perf(t1->cl->model, arch, &j,j.nimpl)
+					              - _starpu_history_based_job_expected_deviation(t1->cl->model, arch, &j,j.nimpl);
 					if (isnan(length))
 						/* Avoid problems with binary coding of doubles */
 						t1->duration[arch->devices[0].type][arch->devices[0].devid][arch->devices[0].ncores] = NAN;
@@ -1094,7 +1125,7 @@ static glp_prob *_starpu_bound_glp_resolve(int integer)
 #endif /* STARPU_HAVE_GLPK_H */
 
 /* Print the computed bound as well as the optimized distribution of tasks */
-void starpu_bound_print(FILE *output, int integer STARPU_ATTRIBUTE_UNUSED)
+void starpu_bound_print(FILE *output, int integer)
 {
 #ifdef STARPU_HAVE_GLPK_H
 	if (recorddeps)
@@ -1140,12 +1171,13 @@ void starpu_bound_print(FILE *output, int integer STARPU_ATTRIBUTE_UNUSED)
 	}
 	STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
 #else /* STARPU_HAVE_GLPK_H */
+	(void) integer;
 	fprintf(output, "Please rebuild StarPU with glpk installed.\n");
 #endif /* STARPU_HAVE_GLPK_H */
 }
 
 /* Compute and return the bound */
-void starpu_bound_compute(double *res, double *integer_res STARPU_ATTRIBUTE_UNUSED, int integer STARPU_ATTRIBUTE_UNUSED)
+void starpu_bound_compute(double *res, double *integer_res, int integer)
 {
 #ifdef STARPU_HAVE_GLPK_H
 	double ret;
@@ -1170,6 +1202,8 @@ void starpu_bound_compute(double *res, double *integer_res STARPU_ATTRIBUTE_UNUS
 	STARPU_PTHREAD_MUTEX_UNLOCK(&mutex);
 	*res = ret;
 #else /* STARPU_HAVE_GLPK_H */
+	(void) integer_res;
+	(void) integer;
 	*res = 0.;
 #endif /* STARPU_HAVE_GLPK_H */
 }
