@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2017                                CNRS
- * Copyright (C) 2013,2014                                Université de Bordeaux
- * Copyright (C) 2012,2013                                Inria
+ * Copyright (C) 2010-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -515,7 +513,7 @@ if (PARALLEL) {
 	}
 #endif
 	STARPU_ASSERT(n == n1*n2);
-	STARPU_ASSERT(n1 < (1ULL << J_BITS));
+	STARPU_ASSERT((unsigned long long) n1 < (1ULL << J_BITS));
 
 
 #ifdef __STARPU_USE_CUDA
@@ -526,7 +524,7 @@ if (PARALLEL) {
 	}
 #endif
 	STARPU_ASSERT(m == m1*m2);
-	STARPU_ASSERT(m1 < (1ULL << J_BITS));
+	STARPU_ASSERT((unsigned long long) m1 < (1ULL << J_BITS));
 
 	/* distribute the n2*m2 second ffts into DIV_2D_N*DIV_2D_M packages */
 	n3 = n2 / DIV_2D_N;
@@ -545,7 +543,7 @@ if (PARALLEL) {
 	plan->number = STARPU_ATOMIC_ADD(&starpufft_last_plan_number, 1) - 1;
 
 	/* 4bit limitation in the tag space */
-	STARPU_ASSERT(plan->number < (1ULL << NUMBER_BITS));
+	STARPU_ASSERT((unsigned long long) plan->number < (1ULL << NUMBER_BITS));
 }
 
 	plan->dim = 2;
@@ -624,14 +622,21 @@ if (PARALLEL) {
 
 if (PARALLEL) {
 	/* Allocate buffers. */
-	plan->twisted1 = STARPUFFT(malloc)(plan->totsize * sizeof(*plan->twisted1));
-	memset(plan->twisted1, 0, plan->totsize * sizeof(*plan->twisted1));
-	plan->fft1 = STARPUFFT(malloc)(plan->totsize * sizeof(*plan->fft1));
-	memset(plan->fft1, 0, plan->totsize * sizeof(*plan->fft1));
-	plan->twisted2 = STARPUFFT(malloc)(plan->totsize * sizeof(*plan->twisted2));
-	memset(plan->twisted2, 0, plan->totsize * sizeof(*plan->twisted2));
-	plan->fft2 = STARPUFFT(malloc)(plan->totsize * sizeof(*plan->fft2));
-	memset(plan->fft2, 0, plan->totsize * sizeof(*plan->fft2));
+	plan->twisted1_size = plan->totsize * sizeof(*plan->twisted1);
+	plan->twisted1 = STARPUFFT(malloc)(plan->twisted1_size);
+	memset(plan->twisted1, 0, plan->twisted1_size);
+
+	plan->fft1_size = plan->totsize * sizeof(*plan->fft1);
+	plan->fft1 = STARPUFFT(malloc)(plan->fft1_size);
+	memset(plan->fft1, 0, plan->fft1_size);
+
+	plan->twisted2_size = plan->totsize * sizeof(*plan->twisted2);
+	plan->twisted2 = STARPUFFT(malloc)(plan->twisted2_size);
+	memset(plan->twisted2, 0, plan->twisted2_size);
+
+	plan->fft2_size = plan->totsize * sizeof(*plan->fft2);
+	plan->fft2 = STARPUFFT(malloc)(plan->fft2_size);
+	memset(plan->fft2, 0, plan->fft2_size);
 
 	/* Allocate handle arrays */
 	plan->twisted1_handle = malloc(plan->totsize1 * sizeof(*plan->twisted1_handle));
