@@ -1,8 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2017                                CNRS
- * Copyright (C) 2015-2016                                Université de Bordeaux
- * Copyright (C) 2013                                     Corentin Salingue
+ * Copyright (C) 2015-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013	    Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -36,35 +35,52 @@
 #ifdef STARPU_HAVE_MEMCHECK_H
 #include <valgrind/memcheck.h>
 #else
-#define VALGRIND_MAKE_MEM_DEFINED(addr, size) (void)0
+#define VALGRIND_MAKE_MEM_DEFINED_IF_ADDRESSABLE(addr, size) (void)0
 #endif
 
 #ifdef STARPU_QUICK_CHECK
 #  define NDATA 4
-#  define NITER 16
+#  define NITER 8
 #elif !defined(STARPU_LONG_CHECK)
 #  define NDATA 32
-#  define NITER 256
+#  define NITER 128
 #else
 #  define NDATA 128
-#  define NITER 1024
+#  define NITER 512
 #endif
 #  define MEMSIZE 1
 #  define MEMSIZE_STR "1"
 
 #if !defined(STARPU_HAVE_SETENV)
 #warning setenv is not defined. Skipping test
+int main(void)
+{
+	return STARPU_TEST_SKIPPED;
+}
+#elif STARPU_MAXNODES == 1
+/* Cannot register a disk */
 int main(int argc, char **argv)
 {
 	return STARPU_TEST_SKIPPED;
 }
 #else
 
-const struct starpu_data_copy_methods my_vector_copy_data_methods_s;
+static int (*any_to_any)(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, void *async_data);
+
+/* We need a ram-to-ram copy for NUMA machine, use any_to_any for that */
+static int ram_to_ram(void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node)
+{
+	return any_to_any(src_interface, src_node, dst_interface, dst_node, NULL);
+}
+
+const struct starpu_data_copy_methods my_vector_copy_data_methods_s =
+{
+	.ram_to_ram = ram_to_ram
+};
 struct starpu_data_interface_ops starpu_interface_my_vector_ops;
 
 void starpu_my_vector_data_register(starpu_data_handle_t *handleptr, int home_node,
-                        uintptr_t ptr, uint32_t nx, size_t elemsize)
+			uintptr_t ptr, uint32_t nx, size_t elemsize)
 {
 	struct starpu_vector_interface vector =
 	{
@@ -72,9 +88,10 @@ void starpu_my_vector_data_register(starpu_data_handle_t *handleptr, int home_no
 		.ptr = ptr,
 		.nx = nx,
 		.elemsize = elemsize,
-                .dev_handle = ptr,
+		.dev_handle = ptr,
 		.slice_base = 0,
-                .offset = 0
+		.offset = 0,
+		.allocsize = nx * elemsize,
 	};
 
 	starpu_data_register(handleptr, home_node, &vector, &starpu_interface_my_vector_ops);
@@ -84,10 +101,11 @@ static unsigned values[NDATA];
 
 static void zero(void *buffers[], void *args)
 {
+	(void)args;
 	struct starpu_vector_interface *vector = (struct starpu_vector_interface *) buffers[0];
 	unsigned *val = (unsigned*) STARPU_VECTOR_GET_PTR(vector);
 	*val = 0;
-	VALGRIND_MAKE_MEM_DEFINED(val, STARPU_VECTOR_GET_NX(vector) * STARPU_VECTOR_GET_ELEMSIZE(vector));
+	VALGRIND_MAKE_MEM_DEFINED_IF_ADDRESSABLE(val, STARPU_VECTOR_GET_NX(vector) * STARPU_VECTOR_GET_ELEMSIZE(vector));
 }
 
 static void inc(void *buffers[], void *args)
@@ -130,20 +148,28 @@ static struct starpu_codelet check_cl =
 	.modes = { STARPU_R },
 };
 
-int dotest(struct starpu_disk_ops *ops, char *base, void (*vector_data_register)(starpu_data_handle_t *handleptr, int home_node, uintptr_t ptr, uint32_t nx, size_t elemsize))
+int dotest(struct starpu_disk_ops *ops, char *base, void (*vector_data_register)(starpu_data_handle_t *handleptr, int home_node, uintptr_t ptr, uint32_t nx, size_t elemsize), const char *text)
 {
-	int *A, *C;
 	starpu_data_handle_t handles[NDATA];
 
+	if (starpu_getenv_number_default("STARPU_DIDUSE_BARRIER", 0))
+		/* This would hang */
+		return STARPU_TEST_SKIPPED;
+
+	FPRINTF(stderr, "Testing <%s>\n", text);
 	/* Initialize StarPU without GPU devices to make sure the memory of the GPU devices will not be used */
+	// Ignore environment variables as we want to force the exact number of workers
 	struct starpu_conf conf;
 	int ret = starpu_conf_init(&conf);
 	if (ret == -EINVAL)
 		return EXIT_FAILURE;
-	conf.ncuda = 0;
-	conf.nopencl = 0;
+	conf.precedence_over_environment_variables = 1;
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
 	ret = starpu_init(&conf);
-	if (ret == -ENODEV) goto enodev;
+	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 
 	/* Initialize path and name */
 	/* register swap disk */
@@ -157,20 +183,44 @@ int dotest(struct starpu_disk_ops *ops, char *base, void (*vector_data_register)
 	for (i = 0; i < NDATA; i++)
 	{
 		vector_data_register(&handles[i], -1, 0, (MEMSIZE*1024*1024*2) / NDATA, sizeof(char));
-		starpu_task_insert(&zero_cl, STARPU_W, handles[i], 0);
+		ret = starpu_task_insert(&zero_cl, STARPU_W, handles[i], 0);
+		if (ret == -ENODEV) goto enodev;
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 	}
 	memset(values, 0, sizeof(values));
 
+	/* Work out of core */
 	for (i = 0; i < NITER; i++)
 	{
 		j = rand()%NDATA;
-		starpu_task_insert(&inc_cl, STARPU_RW, handles[j], STARPU_VALUE, &j, sizeof(j), 0);
+		ret = starpu_task_insert(&inc_cl, STARPU_RW, handles[j], STARPU_VALUE, &j, sizeof(j), 0);
+		if (ret == -ENODEV) goto enodev;
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
+	}
+	starpu_task_wait_for_all();
+
+	/* forcibly evict some data, just for fun */
+	for (i = 0; i < NDATA; i++)
+	{
+		if ((rand() % 2) == 0)
+			starpu_data_evict_from_node(handles[i], STARPU_MAIN_RAM);
+	}
+
+	/* And work out of core again */
+	for (i = 0; i < NITER; i++)
+	{
+		j = rand()%NDATA;
+		ret = starpu_task_insert(&inc_cl, STARPU_RW, handles[j], STARPU_VALUE, &j, sizeof(j), 0);
+		if (ret == -ENODEV) goto enodev;
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 	}
 
 	/* Check and free data */
 	for (i = 0; i < NDATA; i++)
 	{
-		starpu_task_insert(&check_cl, STARPU_R, handles[i], STARPU_VALUE, &i, sizeof(i), 0);
+		ret = starpu_task_insert(&check_cl, STARPU_R, handles[i], STARPU_VALUE, &i, sizeof(i), 0);
+		if (ret == -ENODEV) goto enodev;
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 		starpu_data_unregister(handles[i]);
 	}
 
@@ -181,15 +231,15 @@ int dotest(struct starpu_disk_ops *ops, char *base, void (*vector_data_register)
 
 enoent:
 	FPRINTF(stderr, "Couldn't write data: ENOENT\n");
-	starpu_shutdown();
 enodev:
+	starpu_shutdown();
 	return STARPU_TEST_SKIPPED;
 }
 
 static int merge_result(int old, int new)
 {
-	if (new == EXIT_FAILURE)
-		return EXIT_FAILURE;
+	if (new == EXIT_FAILURE || new == STARPU_TEST_SKIPPED)
+		return new;
 	if (old == 0)
 		return 0;
 	return new;
@@ -202,6 +252,10 @@ int main(void)
 	char s[128];
 	char *ptr;
 
+#ifdef STARPU_HAVE_SETENV
+	setenv("STARPU_CALIBRATE_MINIMUM", "1", 1);
+#endif
+
 	snprintf(s, sizeof(s), "/tmp/%s-disk-XXXXXX", getenv("USER"));
 	ptr = _starpu_mkdtemp(s);
 	if (!ptr)
@@ -213,18 +267,26 @@ int main(void)
 	setenv("STARPU_LIMIT_CPU_MEM", MEMSIZE_STR, 1);
 
 	/* Build an vector-like interface which doesn't have the any_to_any helper, to force making use of pack/unpack */
+	any_to_any = starpu_interface_vector_ops.copy_methods->any_to_any;
 	memcpy(&starpu_interface_my_vector_ops, &starpu_interface_vector_ops, sizeof(starpu_interface_my_vector_ops));
 	starpu_interface_my_vector_ops.copy_methods = &my_vector_copy_data_methods_s;
 
-	ret = merge_result(ret, dotest(&starpu_disk_stdio_ops, s, starpu_vector_data_register));
-	ret = merge_result(ret, dotest(&starpu_disk_stdio_ops, s, starpu_my_vector_data_register));
-	ret = merge_result(ret, dotest(&starpu_disk_unistd_ops, s, starpu_vector_data_register));
-	ret = merge_result(ret, dotest(&starpu_disk_unistd_ops, s, starpu_my_vector_data_register));
+	ret = merge_result(ret, dotest(&starpu_disk_stdio_ops, s, starpu_vector_data_register, "Stdio with read/write vector ops"));
+	if (ret == STARPU_TEST_SKIPPED) goto skipped;
+	ret = merge_result(ret, dotest(&starpu_disk_stdio_ops, s, starpu_my_vector_data_register, "Stdio with pack/unpack vector ops"));
+	if (ret == STARPU_TEST_SKIPPED) goto skipped;
+	ret = merge_result(ret, dotest(&starpu_disk_unistd_ops, s, starpu_vector_data_register, "unistd with read/write vector ops"));
+	if (ret == STARPU_TEST_SKIPPED) goto skipped;
+	ret = merge_result(ret, dotest(&starpu_disk_unistd_ops, s, starpu_my_vector_data_register, "unistd with pack/unpack vector ops"));
+	if (ret == STARPU_TEST_SKIPPED) goto skipped;
 #ifdef STARPU_LINUX_SYS
-	ret = merge_result(ret, dotest(&starpu_disk_unistd_o_direct_ops, s, starpu_vector_data_register));
-	ret = merge_result(ret, dotest(&starpu_disk_unistd_o_direct_ops, s, starpu_my_vector_data_register));
+	ret = merge_result(ret, dotest(&starpu_disk_unistd_o_direct_ops, s, starpu_vector_data_register, "unistd_direct with read/write vector ops"));
+	if (ret == STARPU_TEST_SKIPPED) goto skipped;
+	ret = merge_result(ret, dotest(&starpu_disk_unistd_o_direct_ops, s, starpu_my_vector_data_register, "unistd_direct with pack/unpack vector ops"));
+	if (ret == STARPU_TEST_SKIPPED) goto skipped;
 #endif
 
+skipped:
 	ret2 = rmdir(s);
 	STARPU_CHECK_RETURN_VALUE(ret2, "rmdir '%s'\n", s);
 
