@@ -3,7 +3,7 @@
 #
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2019-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+# Copyright (C) 2019-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -17,6 +17,10 @@
 # See the GNU Lesser General Public License in COPYING.LGPL for more details.
 #
 
+"""
+Plot statistics produced when running an application with STARPU_MPI_STATS=1
+"""
+
 import sys
 import re
 import os
@@ -38,11 +42,11 @@ def usage():
 
 if len(sys.argv) >= 2:
     if sys.argv[1] == '-v' or sys.argv[1] == '--version':
-        print("%s (StarPU) 1.3.10" % PROGNAME)
+        print("%s (StarPU) 1.4.1" % PROGNAME)
         sys.exit(0)
     if sys.argv[1] == '-h' or sys.argv[1] == '--help':
         usage()
-if (len(sys.argv) == 1):
+if len(sys.argv) == 1:
     usage()
 
 if len(sys.argv) >= 2 and sys.argv[1] == '-png':
@@ -58,30 +62,26 @@ else:
 nodes=0
 file = open(outputfile, "r")
 for line in file.readlines():
-    match = re.search('\TOTAL', line)
-    if match:
+    if re.search('TOTAL', line):
         (node,stuff)=line.split(sep="[")[2].split("]")
-        if (int(node) > nodes):
+        if int(node) > nodes:
             nodes=int(node)
 file.close()
 nodes=nodes+1
 
 # extract volume of comm and bandwidth between all pair of nodes
-volumes = [[0 for x in range(nodes)] for y in range(nodes)]
-bandwidth = [[0 for x in range(nodes)] for y in range(nodes)]
+volumes = [[0 for _ in range(nodes)] for _ in range(nodes)]
+bandwidth = [[0 for _ in range(nodes)] for _ in range(nodes)]
 file = open(outputfile, "r")
 for line in file.readlines():
-    match = re.search('\[starpu_comm_stats]', line)
-    if match:
-        match = re.search('TOTAL', line)
-        if not match:
-            (head,volB,B,volMB,MB,bwB,B,bwMB,MB) = line.split()
-            (src,dst)=head.split(sep="[")[2].split(sep="]")[0].split(sep=":")
-            volumes[int(src)][int(dst)] = float(volB)
-            bandwidth[int(src)][int(dst)] = float(bwB)
+    if re.search(r'\[starpu_comm_stats]', line) and not re.search('TOTAL', line) and re.search('MB/s', line):
+        (head,volB,B,volMB,MB,bwB,B,bwMB,MB) = line.split()
+        (src,dst)=head.split(sep="[")[2].split(sep="]")[0].split(sep=":")
+        volumes[int(src)][int(dst)] = float(volB)
+        bandwidth[int(src)][int(dst)] = float(bwB)
 file.close()
 
-def writeData(filename, nodes, data):
+def write_data(filename, nodes, data):
     ofile=open(filename, "w")
     for dst in range(nodes):
         for src in range(nodes):
@@ -89,7 +89,7 @@ def writeData(filename, nodes, data):
         ofile.write("\n")
     ofile.close()
 
-def generateGnuplotScript(filename, datafilename, outputfile, nodes):
+def generate_gnuplot_script(filename, datafilename, outputfile, nodes):
     ofile=open(filename, "w")
     srctics=""
     dsttics=""
@@ -99,18 +99,20 @@ def generateGnuplotScript(filename, datafilename, outputfile, nodes):
     ofile.write("set term %s\n" % outputformat)
     ofile.write("set output \"%s.%s\"\n" % (outputfile, outputext))
     ofile.write("set view map scale 1\nset style data lines\n")
-    ofile.write("set palette model RGB defined ( 0 'white', 100 'black' )\n")
+    ofile.write("set palette gray\n")
     ofile.write("set xtics (%s\"src%d\" %d)\n" % (srctics, nodes-1, nodes-1))
     ofile.write("set ytics (%s\"dst%d\" %d)\n" % (dsttics, nodes-1, nodes-1))
     ofile.write("plot '%s' matrix with image\n" % datafilename)
     ofile.close()
 
 # generate gnuplot volume data and script file
-writeData(outputfile+"_volume.data", nodes, volumes)
-generateGnuplotScript(outputfile+"_volume.gp", outputfile+"_volume.data", outputfile+"_volume_heatmap", nodes)
+write_data(outputfile+"_volume.data", nodes, volumes)
+generate_gnuplot_script(outputfile+"_volume.gp", outputfile+"_volume.data", outputfile+"_volume_heatmap", nodes)
 os.system("gnuplot " + outputfile+"_volume.gp")
+print("Generated file \"%s.%s\"" % (outputfile+"_volume.data", outputext))
 
 # generate gnuplot bandwidth data and script file
-writeData(outputfile+"_bw.data", nodes, bandwidth)
-generateGnuplotScript(outputfile+"_bw.gp", outputfile+"_bw.data", outputfile+"_bw_heatmap", nodes)
+write_data(outputfile+"_bw.data", nodes, bandwidth)
+generate_gnuplot_script(outputfile+"_bw.gp", outputfile+"_bw.data", outputfile+"_bw_heatmap", nodes)
 os.system("gnuplot " + outputfile+"_bw.gp")
+print("Generated file \"%s.%s\"" % (outputfile+"_bw.data", outputext))
